/*
    Copyright (C) 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "echartbase.h"
#include "echartbase.moc"

#include "echartmarkers.h"
#include "echartmarkersconfig.h"

#include <eaction.h>
#include <efiledialog.h>

#include <QPainter>
#include <QPaintEvent>
#include <QMenu>

EChartBase::EChartBase(QWidget *parent) :
  EWidget(parent)
{
	// margins will be recalculated also based on point size of labelFont
	marginVertical = 10;
	marginHorizontal = 10;
	// is calculated at the viewPort function with then define labelFont() size.
	legendMarkersMargin = 2 * EChartMarkers::config()->size();

	colorNames << "darkBlue" << "darkGreen" << "red";
	colorNames << "green" << "blue" << "yellow" << "black" << "darkRed";
	colorNames << "darkCyan" << "darkMagenta" <<  "darkGrey";
	colorNames << "cyan" << "magenta";

	markers << &EChartMarkers::circle << &EChartMarkers::rhomb << &EChartMarkers::square;
	markers << &EChartMarkers::triangleUp << &EChartMarkers::triangleLeft;
	markers << &EChartMarkers::triangleBottom << &EChartMarkers::triangleRight;

	setChartTitle("Chart Title");

	myTitleFont.setBold(true);
	myTitleFont.setPointSize(12);

	setContextMenuPolicy(Qt::CustomContextMenu);
	connect(this, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(contextMenu(QPoint)));

	setNumberPrecision(2);
	setNumberFormat('f');
}

void EChartBase::setChartTitle(QString chartTitle)
{
	myChartTitle = chartTitle;
	update();
}

void EChartBase::setNumberPrecision(int precision)
{
	myNumberPrecision = precision;
}

void EChartBase::setNumberFormat(char format)
{
	myNumberFormat = format;
}

void EChartBase::addDataSet(EChartValues dataSet, const QString label, QColor color)
{
	if(dataSets.keys().contains(label)){
		emit information(tr("Replacing dataset"));
		dataSets.remove(label);
		dataSetsColors.remove(label);
	}

	if(!color.isValid()){
		int row = dataSets.keys().size() % colorNames.size();
		color.setNamedColor(colorNames[row]);
	}

	int marker = dataSets.uniqueKeys().size() % markers.size();
	dataSets.insert(label, dataSet);
	dataSetsColors.insert(label, color);
	dataSetsMarkers.insert(label, markers[marker]);
	update();
}

bool EChartBase::removeDataSet(const QString label)
{
	if(!dataSets.uniqueKeys().contains(label))
		return false;
	dataSets.remove(label);
	dataSetsColors.remove(label);
	dataSetsMarkers.remove(label);
	update();
	return true;
}

void EChartBase::setTitleFont(QFont font)
{
	myTitleFont = font;
	update();
}

QFont EChartBase::titleFont() const
{
	return myTitleFont;
}

void EChartBase::setLabelFont(QFont font)
{
	myLabelFont = font;
	update();
}

QFont EChartBase::labelFont() const
{
	return myLabelFont;
}

EAction *EChartBase::exportAction() const
{
	EAction *action = new EAction(tr("Export"));
	connect(action, SIGNAL(triggered()), this, SLOT(exportChart()));
	return action;
}

void EChartBase::exportChart()
{
	QString defaultFilter = "PNG (*.png)";
	QString filters = defaultFilter +";;JPeg (*.jpg);;TIFF (*.tiff)";
	QFileInfo file(EFileDialog::getSaveFileName(this, tr("Select file for chart export"), QString(), filters, &defaultFilter));

	// format
	const char * format = "png";
	if(file.suffix().toLower() == "jpg")
		format = "jpg";
	if(file.suffix().toLower() == "tiff")
		format = "tiff";

	QImage image(QSize(1000, 565), QImage::Format_RGB32);
	QPainter painter;
	painter.begin(&image);
	exportChart(&painter, QRect(QPoint(0, 0), image.size()));
	painter.end();

	if(!image.save(file.absoluteFilePath(), format))
		emit error(tr("Could not export chart"));
	else
		emit information(tr("Successfully exported chart to %1").arg(file.absoluteFilePath()));
}

void EChartBase::exportChart(QPainter *painter, QRect boundingRect)
{
	if(!painter)
		return;

	painter->save();
	painter->setRenderHint(QPainter::Antialiasing);
	QRect viewPort = createViewPort(painter, boundingRect);
	drawChart(painter, viewPort);
	legend(QPoint(viewPort.right() + marginHorizontal, viewPort.top() + painter->fontMetrics().height()), painter);
	painter->restore();
	// repaint the widget at the application, to restore viewport size, etc
	update();
}

void EChartBase::clear()
{
	foreach(QString label, dataSets.uniqueKeys())
		removeDataSet(label);
	update();
}

void EChartBase::contextMenu(QPoint pos) const
{
	QMenu menu;
	menu.addAction(exportAction()->actionWithText());
	menu.exec(mapToGlobal(pos));
}

void EChartBase::paintEvent(QPaintEvent *event)
{
	QPainter painter(this);
	painter.setRenderHint(QPainter::Antialiasing);

	painter.save();
	QRect viewPort = createViewPort(&painter, QRect(QPoint(0, 0), size()));
	painter.restore();
	painter.save();
	drawChart(&painter, viewPort);
	painter.restore();
	legend(QPoint(viewPort.right() + marginHorizontal, viewPort.top() + painter.fontMetrics().height()), &painter);
	event->accept();
}

void EChartBase::legend(QPointF origin, QPainter *painter)
{
	if(!painter)
		return;

	painter->save();
	painter->setFont(labelFont());

	float xStart = origin.x();
	float y = origin.y();
	foreach(QString label, dataSets.uniqueKeys()){
		if(label.isEmpty())
			continue;

		painter->setPen(dataSetsColors.value(label));
		painter->setBrush(dataSetsColors.value(label));
		dataSetsMarkers.value(label)(painter, QPointF(xStart, y - qreal(EChartMarkers::config()->size())/2.0).toPoint());
		painter->drawText(xStart + legendMarkersMargin, y, label);
		y += 1.5 * painter->fontMetrics().height();
	}
	painter->restore();
}

QRect EChartBase::createViewPort(QPainter *painter, QRect boundingRect)
{
	painter->save();

	// the marker should as big as an average char of the label font
	QFontMetrics metric(labelFont(), painter->device());

	marginVertical = metric.height();
	marginHorizontal = metric.width("n");

	EChartMarkers::config()->setSize(marginHorizontal);
	legendMarkersMargin = EChartMarkers::config()->size() + marginHorizontal;

	painter->setBrush(palette().brush(backgroundRole()));
	painter->drawRect(boundingRect);

	painter->setFont(titleFont());
	int marginTop = marginVertical + painter->fontMetrics().height();
	painter->drawText(QRect(boundingRect.left(), boundingRect.top(),
													boundingRect.width(), marginTop),
										Qt::AlignHCenter | Qt::AlignVCenter, chartTitle());

	painter->setFont(labelFont());
	// determine margin right (Legend)
	int marginRight = 0;
	foreach(QString label, dataSets.uniqueKeys()){
		int length = painter->fontMetrics().width(label) + legendMarkersMargin;
		if(marginRight < length)
			marginRight = length;
	}
	marginRight +=  marginHorizontal + legendMarkersMargin;

	// draw chart area
	viewPort = QRect(boundingRect.left() + marginHorizontal,
									 boundingRect.top() + marginTop,
									 boundingRect.width() - marginHorizontal - marginRight,
									 boundingRect.height() - marginTop - marginVertical);
	painter->setBrush(Qt::white);
	painter->setPen(Qt::white);
	painter->drawRect(viewPort);

	painter->restore();

	return viewPort;
}
