/*
    Copyright (C) 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "echartline.h"
#include "echartline.moc"

#include "echartmarkers.h"
#include "echartmarkersconfig.h"

#include <QPainter>
#include <QMouseEvent>

#include <cmath>
#include <algorithm>

EChartLine::EChartLine(QWidget *parent)
	: EChartBase(parent)
{
	setXTitle("X Label");
	setYTitle("Y Label");
	setNumberPrecision(2);
	setNumberFormat('f');
	setXTicksAngle(0);

	xNumeric = false;
	yNumeric = false;

	// enable mouse tracking to recieve mouse move events,
	// even if no mouse button is pressed
	setMouseTracking(true);
}

void EChartLine::setYTitle(QString yTitle)
{
	myYTitle = yTitle;
	update();
}

void EChartLine::setXTitle(QString xTitle)
{
	myXTitle = xTitle;
	update();
}

void EChartLine::setXTicksAngle(int angle)
{
	if(angle < 0 || angle > 360)
		return;
	myXTicksAngle = angle;
	update();
}

int EChartLine::xTicksAngle() const
{
	return myXTicksAngle;
}

void EChartLine::setXTicks(const QList<QVariant> xTicks)
{
	this->xTicks = xTicks;

	// check if x ticks are all numeric (interopation)
	xNumeric = true;
	foreach(QVariant value, xTicks){
		bool test = false;
		value.toReal(&test);
		if(!test){
			xNumeric = false;
			break;
		}
	}

	update();
}

void EChartLine::setYTicks(const QList<QVariant> yTicks)
{
	this->yTicks = yTicks;

	// check if y ticks are all numeric (interopation)
	yNumeric = true;
	foreach(QVariant value, yTicks){
		bool test = false;
		value.toReal(&test);
		if(!test){
			yNumeric = false;
			break;
		}
	}

	update();
}

void EChartLine::drawChart(QPainter *painter, QRect viewPort)
{
	if(!painter)
		return;

	painter->save();

	// draw axis
	painter->setPen(Qt::black);
	painter->setBrush(Qt::black);
	painter->setFont(labelFont());
	// first determine paddings
	// determine maxLength x-tick label
	QString maxLabel;
	foreach(QVariant label, xTicks){
		if(label.toString().length() > maxLabel.length())
			maxLabel = label.toString();
	}

	int paddingBottom = marginVertical + painter->fontMetrics().height();
	// if the x-ticks label are not horizontal, keep the maximum (vertical) space free
	if(xTicksAngle() != 0)
		paddingBottom = std::min(0.2 * qreal(painter->fontMetrics().height()) + painter->fontMetrics().width(maxLabel), 0.3 * qreal(viewPort.height()));

//	foreach(QVariant value, xTicks){
//		if(paddingBottom < painter->fontMetrics().width(value.toString()))
//			paddingBottom = painter->fontMetrics().height();
//	}
	int paddingLeft = 2 * marginHorizontal + painter->fontMetrics().width(yTitle());
	foreach(QVariant value, yTicks){
		QString tick = value.toString();
		if(yNumeric)
			tick = QString::number(value.toReal(), numberFormat(), numberPrecision());
		if(paddingLeft < painter->fontMetrics().width(tick))
			paddingLeft = painter->fontMetrics().width(tick) + marginHorizontal;
	}

	origin = QPoint(viewPort.left() + paddingLeft, viewPort.bottom() - paddingBottom);
	// x-axis
	int paddingBottomText = origin.y() + 1.3 * painter->fontMetrics().height();
	QPoint xEnd(viewPort.right() - painter->fontMetrics().width(xTitle()) - marginHorizontal, origin.y());
	painter->drawLine(origin, xEnd);
	painter->drawLine(QPoint(xEnd.x() - 2, xEnd.y() + 2), xEnd);
	painter->drawLine(QPoint(xEnd.x() - 2, xEnd.y() - 2), xEnd);
	painter->drawText(xEnd.x() + marginHorizontal, paddingBottomText, xTitle());
	// add ticks
	qreal distanceX = qreal(xEnd.x() - origin.x())/qreal(xTicks.size());

	QMatrix xTicksMatrix;
	xTicksMatrix = xTicksMatrix.rotate(xTicksAngle()).inverted();
	for(int i = 0; i < xTicks.size(); i++){
		QPoint xP(origin.x() + i * distanceX, origin.y());
		if(i > 0)
			painter->drawLine(xP.x(), xP.y() + 2, xP.x(), xP.y() - 2);
		QString tick = xTicks[i].toString();
		if(xNumeric)
			tick = QString::number(xTicks[i].toReal(), numberFormat(), numberPrecision());
		// draw the may be rotated label
		int labelDeltaX = std::min(floor(qreal(painter->fontMetrics().width(tick))/2.0), distanceX / 2.0);
		QPoint labelStart(xP.x() - labelDeltaX, paddingBottomText);
		QPoint labelEnd(xP.x() + distanceX / 2.0, paddingBottomText - painter->fontMetrics().height());
		if(xTicksAngle()){
			labelStart = QPoint(xP.x(), paddingBottomText);
			labelEnd = QPoint(xP.x() + painter->fontMetrics().height(), viewPort.bottom() - marginVertical);
		}
		painter->rotate(xTicksAngle());
		painter->drawText(QRect(xTicksMatrix.map(labelStart), xTicksMatrix.map(labelEnd)), Qt::AlignLeft | Qt::AlignVCenter, tick);
		painter->resetTransform();
	}
	// y-axis
	QPoint yEnd(origin.x(), viewPort.top() + marginVertical);
	painter->drawLine(origin, yEnd);
	painter->drawLine(QPoint(yEnd.x() - 2, yEnd.y() + 2), yEnd);
	painter->drawLine(QPoint(yEnd.x() + 2, yEnd.y() + 2), yEnd);
	painter->drawText(viewPort.left() + marginHorizontal, viewPort.top() + 1.3 * painter->fontMetrics().height(), yTitle());
	// add ticks
	// distanceY is < 0, because the y-Axis is from top to bottom
	qreal distanceY = qreal(yEnd.y() - origin.y())/qreal(yTicks.size());
	for(int i = 0; i < yTicks.size(); i++){
		QPoint yP(origin.x(), origin.y() + i * distanceY);
		if(i > 0)
			painter->drawLine(yP.x() - 2, yP.y(), yP.x() + 2, yP.y());
		QString tick = yTicks[i].toString();
		if(yNumeric)
			tick = QString::number(yTicks[i].toReal(), numberFormat(), numberPrecision());
		painter->drawText(yP.x() - painter->fontMetrics().width(tick) - 5, yP.y() + floor(qreal(painter->fontMetrics().height())/2.0), tick);
	}

	// draw lines
	foreach(QString label, dataSets.uniqueKeys()){
		QList<QPointF> usedPoints;
		QPoint lastPoint;
		painter->setBrush(dataSetsColors.value(label));
		painter->setPen(dataSetsColors.value(label));
		void (*marker)(QPainter *, QPoint) = dataSetsMarkers.value(label);
		EChartValues values = dataSets.value(label);
		foreach(QVariant xValue, values.xValues()){
			float xCount = xTicks.indexOf(xValue);
			// if xCount < 0, the xValue does not exist, therefore the xValue is set to origin
			// and the same with y value
			if(xCount < 0){
				bool test = false;
				xCount = xValue.toFloat(&test);
				if(!xNumeric || !test)
					xCount = 0;
				else{
					// we need to find the ticks on left and right side of xCount
					bool found = false;
					for(int i = 0; i < xTicks.size(); i++){
						if(xTicks[i].toFloat() > xCount){
							// it's left outside the chart
							if(i == 0)
								xCount = 0;
							else{
								// devide with distanceX, because it is multiplied later on, but
								// the interpolation already does it
								xCount = linearInterpolation(distanceX, xTicks[i].toFloat(), xTicks[i - 1].toFloat(), distanceX * (i - 1), xCount)/distanceX;
							}
							found = true;
							break;
						}
					}
					// if not found, it is outside at the right end
					if(!found)
						xCount = xTicks.size() - 1;
				}
			}
			float yCount = yTicks.indexOf(values.y(xValue));
			if(yCount < 0){
				bool test = false;
				yCount = values.y(xValue).toFloat(&test);
				if(!yNumeric || !test)
					yCount = 0;
				else{
					// we need to find the ticks on bottom and top side of yCount
					bool found = false;
					for(int i = 0; i < yTicks.size(); i++){
						if(yTicks[i].toFloat() > yCount){
							// it's left outside the chart
							if(i == 0)
								yCount = 0;
							else{
								// devide with distanceY, because it is multiplied later on, but
								// the interpolation already does it
								yCount = linearInterpolation(distanceY, yTicks[i].toFloat(), yTicks[i - 1].toFloat(), distanceY * (i - 1), yCount)/distanceY;
							}
							found = true;
							break;
						}
					}
					// if not found, it is outside at the top end
					if(!found)
						yCount = yTicks.size() - 1;
				}
			}
			QPoint currentPoint(origin.x() + distanceX * xCount, origin.y() + distanceY * yCount);
			usedPoints << currentPoint;
			marker(painter, currentPoint);
			if(!lastPoint.isNull())
				painter->drawLine(lastPoint, currentPoint);
			lastPoint = currentPoint;
		}
		if(linesDrawn.contains(label))
			linesDrawn.remove(label);
		linesDrawn.insert(label, usedPoints);
	}

	// draw current position
	if(currentPosition.isNull()){
		painter->restore();
		return;
	}
	if(painter->device()->devType() == QInternal::Widget){
		// draw marker lines
		QPen pen;
		pen.setColor(Qt::lightGray);
		painter->setPen(pen);
		QString text = QString::number(currentYValue, numberFormat(), numberPrecision());
		painter->drawText(origin.x() - painter->fontMetrics().width(text) - 5,
										 currentPosition.y() + floor(qreal(painter->fontMetrics().height())/2.0),
										 text);
		text = QString::number(currentXValue, numberFormat(), numberPrecision());
		painter->drawText(currentPosition.x() - qreal(painter->fontMetrics().width(text))/2.0,
										 origin.y() + painter->fontMetrics().height() + 5,
										 text);
		pen.setStyle(Qt::DashDotLine);
		painter->setPen(pen);
		painter->drawLine(currentPosition.x(), yEnd.y(), currentPosition.x(), origin.y() + 2);
		painter->drawLine(origin.x() - 2, currentPosition.y(), xEnd.x(), currentPosition.y());
	}

	painter->restore();
}

void EChartLine::mouseMoveEvent(QMouseEvent *event)
{
	bool curPosChanged = !currentPosition.isNull();
	currentPosition = QPointF();
	QPointF pos = event->posF();

	if(!viewPort.contains(pos.toPoint())){
		pos = QPointF();
		currentXValue = 0;
		currentYValue = 0;
		update();
		return;
	}

	// find nearest line
	QString chosenLine;
	qreal minDistance = 0;
	currentXValue = 0;
	int chosenIndex = 0;
	foreach(QString label, linesDrawn.uniqueKeys()){
		QList<QPointF> points = linesDrawn.value(label);
		for(int i = 1; i < points.size(); i++){
			QPointF p1 = points[i];
			QPointF p0 = points[i - 1];
			// calculate the line between the p1 and p0: y = m * x + n
			// and the orthogonal line from that line to the current mouse position: y = ms * x + ns
			// Afterwards calculate the point of interception if this two lines: (ys, xs)
			// At the last step, calculate the distance between current mouse position and the
			// point of interception: d
			//
			// calculate the slopes
			qreal m = (p1.y() - p0.y())/(p1.x() - p0.x());
			qreal ms = -1.0/m;
			// calculate the intercepts
			qreal n = p1.y() - m * p1.x();
			qreal ns = pos.y() - ms * pos.x();
			// calculate the point (ys, xs) of intersection between the two lines
			qreal xs = (n - ns)/(ms - m);
			qreal ys = m * xs + n;
			// calculate the distance from current mouse position to (ys, xs)
			qreal d = pow(pow(pos.x()-xs, 2) + pow(pos.y() - ys, 2), 0.5);
			// a maximum distance before a line is "fetched"
			if(d < 10){
				if((minDistance > d) || (minDistance == 0)){
					minDistance = d;
					currentPosition = QPointF(xs, ys);
					chosenLine = label;
					currentXValue = (xs - p0.x())/(p1.x() - p0.x());
					chosenIndex = i;
				}
			}
		}
	}

	if(chosenLine.isEmpty()){
		currentPosition = QPointF();
		if(curPosChanged)
			update();
		return;
	}

	if(xNumeric && yNumeric){
		// determine current value
		EChartValues values = dataSets.value(chosenLine);
		qreal xValue0 = values.xValues()[chosenIndex - 1].toReal();
		qreal xValue1 = values.xValues()[chosenIndex].toReal();
		qreal yValue0 = values.y(xValue0).toReal();
		qreal yValue1 = values.y(xValue1).toReal();
		currentXValue = xValue0 + currentXValue * (xValue1 - xValue0);
		currentYValue = linearInterpolation(yValue1 - yValue0, xValue1, xValue0, yValue0, currentXValue);
	}
	update();
}

float EChartLine::linearInterpolation(const float dy, const float x1, const float x0, const float y0, const float x) const
{
	return dy/(x1 - x0) * (x - x0) + y0;
}
