/*
    Copyright (C) 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "echartpie.h"
#include "echartpie.moc"

#include "echartmarkers.h"

#include <QPainter>

#include <cmath>
#include <algorithm>

EChartPie::EChartPie(QWidget *parent) :
  EChartBase(parent)
{
	setPercents(false);
	PI = acos(double(-1));

	// replace markers
	for(int i = 0; i < markers.size(); i++)
		markers[i] = &EChartMarkers::square;

	setNumberPrecision(0);
	setNumberFormat('f');
}

void EChartPie::setPercents(bool percent)
{
	arePercents = percent;
}

void EChartPie::drawChart(QPainter *painter, QRect viewPort)
{
	if(!painter)
		return;

	painter->save();

	QMap<QString, EChartValues> setsNew;
	if(percents()){
		setsNew = dataSets;
	}
	else{
		// calculate total
		float sum = 0;
		foreach(QString label, dataSets.uniqueKeys()){
			if(dataSets.value(label).xValues().size() != 1){
				painter->restore();
				return;
			}
			QVariant value = dataSets.value(label).xValues()[0];
			bool test;
			float val = value.toFloat(&test);
			if(!test){
				painter->restore();
				return;
			}
			sum += val;
		}
		foreach(QString label, dataSets.uniqueKeys()){
			EChartValues newValues;
			EChartValues old = dataSets.value(label);
			float piece = old.xValues()[0].toFloat() / sum;
			newValues.add(piece, QString());
			setsNew.insert(label, newValues);
		}
	}

	// setting pie rect
	// getting biggest label
	int labelMax = 0;
	foreach(QString label, setsNew.uniqueKeys()){
		int length = painter->fontMetrics().width(percentLabel(setsNew.value(label).xValues()[0].toFloat()));
		if(length > labelMax)
			labelMax = length;
	}

	int marginPieLeft = 2 * marginHorizontal + labelMax;
	int marginPieTop = marginVertical + painter->fontMetrics().height();
	int square = std::min(viewPort.width() - 2 * labelMax - marginPieLeft - marginHorizontal,
												viewPort.height() - marginPieTop - painter->fontMetrics().height()) - marginVertical;
	QRectF pieRect = QRect(viewPort.left() + marginPieLeft,
												 viewPort.top() + marginPieTop,
												 square, square);
	double radius = pieRect.width() / 2.0;
	double area = PI * pow(radius, 2);
	double legendRadius = radius + marginHorizontal;

	// calc angles
	QMap<QString, int> angles;
	int startAngle = 0;
	foreach(QString label, setsNew.uniqueKeys()){
		QVariant value = setsNew.value(label).xValues()[0];
		double piece = area * value.toFloat();
		double spanAngle = 16.0 * 360.0 * piece / PI / pow(radius, 2);
		angles.insert(label, spanAngle);
		startAngle += spanAngle;
	}

	// draw pie
	startAngle = 0;
	painter->setFont(labelFont());
	foreach(QString label, angles.uniqueKeys()){
		float angle = angles.value(label);

		painter->setPen(dataSetsColors.value(label));
		painter->setBrush(dataSetsColors.value(label));
		painter->drawPie(pieRect, startAngle, angle);
		QPoint legendPos(pieRect.x() + float(pieRect.width()) / 2.0, pieRect.y() + float(pieRect.height()) / 2.0);
		float alpha = (startAngle + (angle / 2.0)) / 16.0 * PI / 180.0;
		QString legend = percentLabel(setsNew.value(label).xValues()[0].toFloat());
		int xNull = legendPos.x();
		int yNull = legendPos.y();
		legendPos.setX(legendPos.x() + legendRadius * cos(alpha));
		if(legendPos.x() < xNull)
			legendPos.setX(legendPos.x() - painter->fontMetrics().width(legend));
		legendPos.setY(legendPos.y() - legendRadius * sin(alpha));
		if(legendPos.y() > yNull)
			legendPos.setY(legendPos.y() + painter->fontMetrics().height());
		painter->drawText(legendPos, legend);
		startAngle += angle;
	}

	painter->restore();
}

QString EChartPie::percentLabel(float percentage)
{
	return QString("%1%").arg(QString::number(percentage * 100.0, numberFormat(), numberPrecision()));
}
