/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "elibconfig.h"
#include "ecmdlineargs.h"

#include "ecmdlineoption.h"
#include "epart.h"
#include "eversion.h"

#include <QApplication>
#include <QTextStream>
#include <QObject>
#include <QStringList>
#include <QMessageBox>

#ifndef Q_OS_WIN32
	#include <unistd.h> 
#endif
#include <stdio.h>
#ifdef Q_OS_WIN32
	#include <io.h>
#endif

#ifdef BINDINGS_KDE
  #include <KCmdLineOptions>
	#include <kdeversion.h>
#endif

ECmdLineArgs::ECmdLineArgs(EPart *part)
{
	argc = 0;
	this->part = part;
//#ifndef BINDINGS_KDE
	// kde adds this options automatically
	add("help", "h", QObject::tr("Show this help and exit"));
	add("version", "v", QObject::tr("Show the version"));
//#endif
}

ECmdLineArgs::~ECmdLineArgs()
{
	qDeleteAll(options);
}

bool ECmdLineArgs::init(int argc, char** argv)
{
	this->argc = argc;
	this->argv = argv;
	if(isSet("help")){
		showHelp();
		return false;
	}
	if(isSet("version")){
		showVersion();
		return false;
	}
	return true;
}

void ECmdLineArgs::add(const QByteArray& name,
											 const QByteArray& alias,
											 const QString& description,
											 const QByteArray& defaultValue)
{
	add(new ECmdLineOption(name, alias, description, defaultValue));
}

void ECmdLineArgs::add(ECmdLineOption* option)
{
	if(!option)
		return;
	options << option;
}

bool ECmdLineArgs::isSet(const QByteArray& option) const
{
	return isSet(findOption(option));
}

QString ECmdLineArgs::getOption(const QByteArray& option) const
{
	ECmdLineOption *opt = findOption(option);
	if(!opt)
		return QString();
	if(opt->isSet())
		return opt->value();
	// isSet may be wasn't called on this option
	// therefore call it to set the value.
	isSet(opt);
	return opt->value();
}

ECmdLineOption* ECmdLineArgs::findOption(const QByteArray& option) const
{
	foreach(ECmdLineOption *opt, options){
		if(opt->name().contains(option))
			return opt;
	}
	return 0;
}

bool ECmdLineArgs::isSet(ECmdLineOption* option) const
{
	if(!option)
		return false;

	if(option->isSet())
		return true;

	for(int i = 1; i < argc; i++){
		if(argv[i] == option->name() ||	(!option->alias().isEmpty() && argv[i] == option->alias())){
			if(option->requiresArgument() && i + 1 >= argc)
				return false;

			option->setIsSet(true);
			if(option->requiresArgument())
				option->setValue(argv[i + 1]);
			return true;
		}
	}

	// detect arguments (they must appear in the order
	// they are added)
	int startOption = 1;
	foreach(ECmdLineOption* option, options){
		if(option->isArgument()){
			for(int i = startOption; i < argc; i++){
				if(QString(argv[i]).left(1) != "-"){
					// it is an argument
					startOption = i + 1;
					option->isSet();
					option->setValue(argv[i]);
					break;
				}
			}
		}
	}
	return false;
}

void ECmdLineArgs::addToKCmdLineOptions(KCmdLineOptions* option) const
{
	if(!option)
		return;

	foreach(ECmdLineOption *eOpt, options)
		eOpt->addToKCmdLineOptions(option);
}

void ECmdLineArgs::showHelp() const
{
	QStringList usageOptions;
	QStringList usageArguments;
	QStringList helpOptions;
	QStringList helpArguments;
	foreach(ECmdLineOption* option, options){
		if(option->isArgument()){
			usageArguments << option->usage();
			helpArguments << option->helpLine();
		}
		else{
			usageOptions << option->usage();
			helpOptions << option->helpLine();
		}
	}

	QString appName = QApplication::applicationName();
	QString appVersion = QApplication::applicationVersion();
	if(appName.isEmpty()){
		if(part && !part->name().isEmpty())
			appName = part->name();
		else
			appName = "Application";
	}
	if(appVersion.isEmpty()){
		if(part && !part->versionString().isEmpty())
			appVersion = part->versionString();
		else
			appVersion = "na";
	}

  QStringList out;
	out << QString("%1: %2").arg(appName).arg(appVersion);
	if(part)
    out << QObject::tr("Usage: epartexe %1 %2 %3")
                  .arg(usageOptions.join(" "))
                  .arg(usageArguments.join(" "))
                  .arg(QApplication::applicationName()) << "\n";
	if(part && !part->helpDescription().isEmpty()){
		out << part->helpDescription() << "\n";
	}
// 	out << tr("For details see manpage \"meganizer\" (Execute man meganizer)");
// 	out;
	if(helpOptions.size() > 0)
		out << QObject::tr("Options:");
	foreach(QString string, helpOptions)
		out << "  " << string;
	if(helpArguments.size() > 0)
		out << QObject::tr("Arguments:");
	foreach(QString string, helpArguments)
		out << "  " << string << "\n";

  // check if we are called from terminal
  // see http://stackoverflow.com/questions/1312922/detect-if-stdin-is-a-terminal-or-pipe-in-c-c-qt
#ifdef Q_OS_WIN32
	if(_isatty(_fileno(stdout))){
#else
	if(isatty(fileno(stdout))){
#endif
		QTextStream outStream(stdout);
		outStream << out.join(QString("\n"));
	}
	else{
		// if(qApp == NULL) is not true here, but we know,
		// that there is no QApplication, therefore it is
		// created
		QApplication app((int&)argc, argv);
		QMessageBox::information(NULL, QObject::tr("Help"), out.join("\n"));
	}
}

void ECmdLineArgs::showVersion() const
{
	QString appName = QApplication::applicationName();
	QString appVersion = QApplication::applicationVersion();
	if(appName.isEmpty()){
		if(part && !part->name().isEmpty())
			appName = part->name();
		else
			appName = "Application";
	}
	if(appVersion.isEmpty()){
		if(part && !part->versionString().isEmpty())
			appVersion = part->versionString();
		else
			appVersion = "na";
	}

	QStringList out;
	// print Qt version
	out << "Qt";
	out << QString("  compiled against: %1").arg(QT_VERSION_STR);
	out << QString("  running with:     %1").arg(qVersion());
#ifdef BINDINGS_KDE
	// print kde version
	out << "KDE";
	out << QString("  compiled against: %1").arg(KDE_VERSION_STRING);
	out << QString("  running with:     %1").arg(KDE::versionString());
#endif
	// LibEngsas version
	out << "LibEngsas";
	if(part)
		out << QString("  compiled against: %1").arg(part->libEngsasVersion());
	else
		out << QString("  compiled against: na");
	out << QString("  running with:     %1").arg(EngSaS::version());
	// App version
	out << QString("%1: %2").arg(appName).arg(appVersion);

	// see showVersion()
#ifdef Q_OS_WIN32
	if(_isatty(_fileno(stdout))){
#else
	if(isatty(fileno(stdout))){
#endif
		QTextStream outStream(stdout);
		outStream << out.join(QString("\n"));
	}
	else{
		// if(qApp == NULL) is not true here, but we know,
		// that there is no QApplication, therefore it is
		// created
		QApplication app((int&)argc, argv);
		QMessageBox::information(NULL, QObject::tr("Help"), out.join("\n"));
	}
}
