/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "elibconfig.h"
#include "ecmdlineoption.h"

#include <QRegExp>

#ifdef BINDINGS_KDE
  #include <KCmdLineOptions>
#endif

ECmdLineOption::ECmdLineOption(const QByteArray& name,
															 const QByteArray& alias,
															 const QString& description,
															 const QByteArray& defaultValue)
{
	myRequiresArgument = false;
	myIsArgument = false;
	myIsOptional = false;
	setName(name);
	setAlias(alias);
	setDescription(description);
	setDefaultValue(defaultValue);
	setIsSet(false);
}

void ECmdLineOption::setName(const QByteArray& name)
{
	if(name.isEmpty())
		return;
	
	myName = name;
	QRegExp reg("[\\s]*<[^>]+>");
	QString tmp(myName);
	if(tmp.contains(reg)){
		myRequiresArgument = true;
		tmp.replace(reg, "");
		myName = tmp.toUtf8();
	}
	if(myName.left(1) == "+"){
		myIsArgument = true;
		myRequiresArgument = false;
		myName.replace("+", "");
		setAlias("");
	}
	if(myName.contains("[") && myName.contains("]")){
		myIsOptional = true;
		myName.replace("[", "");
		myName.replace("]", "");
	}
	if(!isArgument() && myName.left(1) != "-")
		myName = "--"+ myName;
}

QByteArray ECmdLineOption::name() const
{
	return myName;
}

bool ECmdLineOption::requiresArgument() const
{
	return myRequiresArgument;
}

bool ECmdLineOption::isArgument() const
{
	return myIsArgument;
}

bool ECmdLineOption::isOptional() const
{
	return myIsOptional;
}

void ECmdLineOption::setAlias(const QByteArray& alias)
{
	// arguments can not have an alias
	if(isArgument())
		return;
	if(alias.isEmpty())
		return;
	myAlias = alias;
	if(myAlias.left(1) != "-")
		myAlias = "-"+ alias;
}

QByteArray ECmdLineOption::alias() const
{
	return myAlias;
}

void ECmdLineOption::setDescription(const QString& description)
{
	myDescription = description;
}

QString ECmdLineOption::description() const
{
	return myDescription;
}

void ECmdLineOption::setDefaultValue(const QByteArray& defaultValue)
{
	myDefaultValue = defaultValue;
}

QByteArray ECmdLineOption::defaultValue() const
{
	return myDefaultValue;
}

void ECmdLineOption::setIsSet(bool status)
{
	myIsSet = status;
}

bool ECmdLineOption::isSet() const
{
	return myIsSet;
}

void ECmdLineOption::setValue(const QByteArray& value)
{
	myValue = value;
}

QByteArray ECmdLineOption::value() const
{
	if(myValue.isEmpty())
		return defaultValue();
	return myValue;
}

QString ECmdLineOption::usage() const
{
	if(isArgument()){
		if(isOptional())
			return QString("[%1]").arg(QString(name()));
		else
			return name();
	}
	QString string = name();
	if(!alias().isEmpty())
		string = alias();
	if(requiresArgument())
		string += " <argument>";
	if(isOptional())
		return QString("[%1]").arg(string);
	else
		return string;
}

QString ECmdLineOption::helpLine() const
{
	if(isArgument()){
		QString string = name();
		if(isOptional())
			string = QString("[%1]").arg(string);
		string += QString(" %1").arg(description());
		if(!defaultValue().isEmpty())
			string += QString(" (Default: %1)").arg(QString(defaultValue()));
		return string;
	}
	QString string;
	if(!alias().isEmpty())
		string += alias();
	else
		string += "   ";
	
	string = QString("%1 %2").arg(string).arg(QString(name()));
	
	if(requiresArgument())
		string = QString("%1 %2").arg(string).arg("<argument>");
	
	if(isOptional())
		string = QString("[%1]").arg(string);
	
	string = QString("%1 %2").arg(string).arg(description());
	
	if(!defaultValue().isEmpty())
		string += QString(" (Default: %1)").arg(QString(defaultValue()));
	return string;
}

void ECmdLineOption::addToKCmdLineOptions(KCmdLineOptions* option) const
{
#ifndef BINDINGS_KDE
	Q_UNUSED(option);
	return;
#else
	if(!option)
		return;
	
	QRegExp reg("^[-]+");
	QString opt = name();
	opt.replace(reg, "");
	QString al = alias();
	al.replace(reg, "");
	if(!al.isEmpty())
		option->add(al.toUtf8());

	if(isOptional())
		opt = QString("[%1]").arg(opt);
	if(isArgument())
		opt = QString("+%1").arg(opt);
	if(requiresArgument())
		opt += " <argument>";
	option->add(opt.toUtf8(), ki18n(description().toUtf8()), defaultValue());
#endif
}
