/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "edbbackup.h"
#include "edbbackup.moc"
#include "ui_edbbackup.h"

#include "esqldatabase.h"
#include "efiledialog.h"

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QSettings>
#include <QDate>
#include <QDateTime>
#include <QTextCodec>
#include <QSqlDriver>
#include <QApplication>

EDBBackup::EDBBackup(ESetupWizardObject* newObject, QWidget* parent, Qt::WindowFlags flags)
	: ESetupWizardWidget(newObject, parent, flags), ui(new Ui::EDBBackup)
{
	codecName = "UTF-8";
	doAbort = false;
	
	ui->setupUi(this);
	setName(windowTitle());
	connect(ui->fileButton, SIGNAL(clicked()), this, SLOT(setDumpFile()));

	// create default file name
	QSettings settings;
	QString lastDir = settings.value("lastDumpDir", "").toString();
	if(!lastDir.isEmpty()){
		dumpFile.setFile(lastDir +"/"+ QApplication::applicationName() +"-"+ QDate::currentDate().toString("yyyyMMdd") +".sql");
		ui->fileEdit->setText(dumpFile.absoluteFilePath());
	}
}

QString EDBBackup::mysqlEscape(QString input)
{
	input.replace("\\", "\\\\");
	input.replace("\\x00", "\\\\x00");
	input.replace("\\n", "\\\\n");
	input.replace("\\r", "\\\\r");
	input.replace("\'", "\\\'");
	input.replace("\"", "\\\"");
	input.replace("\\x1a", "\\\\x1a");
	return input;
}

bool EDBBackup::finish()
{
	ui->logEdit->setPlainText("");
	doAbort = false;
	emit exportStarted();
	actualStep(tr("Opening file %1").arg(ui->fileEdit->text()));
	// create file if needed and open it for writing
	dumpFile.setFile(ui->fileEdit->text());
	QFile file(dumpFile.absoluteFilePath());
	if(!dumpFile.isWritable()){
		if(dumpFile.exists()){
			log(tr("Output file is not writable, please select a writeable file and try again!\n"
										"File: %1")
										.arg(dumpFile.absoluteFilePath()));
			setDumpFile();
			emit exportFinished(false);
			return false;
		}
		else{
			// create it
			if(!file.open(QFile::WriteOnly)){
				log(tr("Output file is not writable, please select a writeable file and try again!\n"
											"File: %1")
											.arg(dumpFile.absoluteFilePath()));
				setDumpFile();
				emit exportFinished(false);
				return false;
			}
		}
	}
	if(!file.isOpen()){
		if(!file.open(QFile::WriteOnly)){
			log(tr("Output file is not writable, please select a writeable file and try again!\n"
										"File: %1")
										.arg(dumpFile.absoluteFilePath()));
			setDumpFile();
			emit exportFinished(false);
			return false;
		}
	}
	QSettings settings;
	settings.setValue("lastDumpDir", dumpFile.absolutePath());
	
	// try to open database
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB){
		if(!db.open()){
			log(tr("Could not connect to Database"));
			file.close();
			emit exportFinished(false);
			return false;
		}
	}
	
	// do all during one transaction, if transactions are supported
	bool transactions = db.driver()->hasFeature(QSqlDriver::Transactions);
	if(transactions)
		db.transaction();
	
	// configure stream
	actualStep(tr("Configure file"));
	QTextStream stream(&file);
	stream.setCodec(QTextCodec::codecForName(codecName.toAscii()));
	
	// write header
	actualStep(tr("Writing header"));
	stream << QString("-- %1 %2\n").arg(settings.value("appNameShort").toString()).arg(settings.value("appVersion").toString());
	stream << QString("-- Host: %1\n").arg(db.hostName());
	stream << QString("-- Database: %1\n").arg(db.databaseName());
	stream << QString("-- User: %1\n").arg(db.userName());
	stream << QString("-- Date: %1\n").arg(QDateTime::currentDateTime().toString(tr("MM/dd/yyyy hh:mm:ss")));
	stream << QString("-- Codec: %1\n").arg(codecName);
	stream << QString("-- -----------------------------------------------------------------\n");
	stream << "\n";
	stream << "-- Disableing foreignkeychecks during inserts\n";
	stream << "SET foreign_key_checks = 0;\n";
	stream << "\n";

	// get list of tables and views
	QStringList dbTables = db.tables ( QSql::Tables );
	QStringList dbViews = db.tables ( QSql::Views );
	setRange(dbTables.size() + dbViews.size());
	bool result = true;
	
	// dump tables
	if(!doAbort){
		actualStep(tr("Dumping tables"));
		foreach(QString table, dbTables){
			QString resString = tr("OK");
			if(!dumpTable(table, stream)){
				result = false;
				resString = tr("FAILED");
			}
			stream << "\n\n";
			log(QString("  %1 - %2").arg(table).arg(resString));
			finishedStep();
			if(doAbort){
				result = false;
				break;
			}
			QApplication::processEvents();
		}
	}
	
	// dump views
	if(!doAbort){
		actualStep(tr("Sorting views"));
		dbViews = sortViews(dbViews);
		
		actualStep(tr("Dump views"));
		foreach(QString view, dbViews){
			QString resString = tr("OK");
			if(!dumpView(view, stream)){
				result = false;
				resString = tr("FAILED");
			}
			stream << "\n\n";
			log(QString("  %1 - %2").arg(view).arg(resString));
			finishedStep();
			if(doAbort){
				result = false;
				break;
			}
			QApplication::processEvents();
		}
	}
	
	if(!doAbort){
		actualStep(tr("Writing footer"));
		stream << "\n";
		stream << "-- Enable foreignkeychecks after inserts\n";
		stream << "SET foreign_key_checks = 1;\n";
	}
	
	if(closeDB)
		db.close();
	file.close();
	
	if(doAbort){
		actualStep(tr("Aborted"));
	}
	else{
		if(result){
			actualStep(tr("Finished!"));
		}
		else{
			actualStep(tr("Finished with errors!"));
		}
	}
	
	if(transactions)
		db.commit();
	
	emit exportFinished(result);
	return result;
}

bool EDBBackup::saveSettings(QString& error)
{
	Q_UNUSED(error);
	return false;
}

void EDBBackup::abort()
{
	doAbort = true;
}

void EDBBackup::setDumpFile()
{
  QStringList filters;
	QString defaultFilter = QString("%1 (*.sql)").arg(tr("SQL-Files"));
	filters << defaultFilter;
	filters << QString("%1 (*.*)").arg(tr("All Files"));

	QSettings settings;
	QString wTitle = tr("Select export file");
	dumpFile = QFileInfo(EFileDialog::getSaveFileName(this, wTitle, ui->fileEdit->text(), filters.join(";;"), &defaultFilter));
	ui->fileEdit->setText(dumpFile.absoluteFilePath());
}

bool EDBBackup::dumpTable(QString table, QTextStream &stream)
{
	ui->tableLabel->setText(table +":");
	QSqlQuery query;
	
	// dump create statement
	query.exec(QString("SHOW CREATE TABLE %1").arg(table));
	if(!query.first())
		return false;
	
	QString tableCreateStatement = query.record().value("Create Table").toString();
	if(tableCreateStatement.isEmpty())
		return false;
	
	stream << "--\n";
	stream << QString("-- Table structure for table `%1`\n").arg(table);
	stream << "--\n";
	stream << QString("DROP TABLE IF EXISTS `%1`;\n").arg(table);
	stream << tableCreateStatement +";";
	stream << "\n";
	
	// dump content
	query.exec(QString("SELECT * FROM %1").arg(table));
	if(query.size() < 1){
		stream << "--\n";
		stream << QString("-- No data in table `%1`\n").arg(table);
		stream << "--\n";
	}
	else{
		stream << "--\n";
		stream << QString("-- Dumping data for table `%1`\n").arg(table);
		stream << "--\n";
		stream << QString("LOCK TABLES `%1` WRITE;\n").arg(table);
	
		ui->tableProgress->setRange(0, query.size());
		while(query.next()){
			stream << QString("INSERT INTO `%1` VALUES (").arg(table);
			for(int i = 0; i < query.record().count(); i++){
				QVariant field = query.record().value(i);
				if(field.isNull()){
					stream << "NULL";
				}
				else if(field.type() == QMetaType::QByteArray){
					// we store byte stream as hex value, therefore 0x is prepended
					// to indicate MySQL that it is an hex string
 					stream << "0x" + field.toByteArray().toHex();
				}
				else{
					bool check;
					field.toInt(&check);
					if(check){
						stream << field.toString();
					}
					else{
						stream << "'"+ mysqlEscape(field.toString()) +"'";
					}
				}
				if(i < query.record().count() - 1)
					stream << ",";
			}
			stream << ");\n";
			ui->tableProgress->setValue(ui->tableProgress->value() + 1);
		}
		stream << "UNLOCK TABLES;\n";
	}
	
	ui->tableLabel->setText("");
	ui->tableProgress->setRange(0,100);
	ui->tableProgress->setValue(0);
	return true;
}

bool EDBBackup::dumpView(QString view, QTextStream& stream)
{
	QSqlQuery query;
	
	// dump create statement
	query.exec(QString("SHOW CREATE VIEW %1").arg(view));
	if(!query.first())
		return false;
	
	QString viewCreateStatement = query.record().value("Create View").toString();
	if(viewCreateStatement.isEmpty())
		return false;
	viewCreateStatement.replace(QRegExp("DEFINER=[^\\s]+"), "");
	viewCreateStatement.replace(QRegExp("SQL SECURITY DEFINER"), "SQL SECURITY INVOKER");
	
	stream << "--\n";
	stream << QString("-- Create statement for view `%1`\n").arg(view);;
	stream << "--\n";
	stream << QString("DROP VIEW IF EXISTS `%1`;\n").arg(view);
	stream << viewCreateStatement +";";
	stream << "\n";
	
	return true;
}

QStringList EDBBackup::sortViews(QStringList views)
{
	QRegExp fromRegExp("from [\\(]+`([\\w\\d]+)`[\\s\\.]{1}");
	QRegExp joinRegExp("join `([\\w\\d]+)`[\\s\\.]{1}");
	QMultiMap<QString, QString> dependencies;
	QStringList result;
	
	foreach(QString view, views){
		if(doAbort)
			return result;
		QSqlQuery query;
	
		// dump create statement
		query.exec(QString("SHOW CREATE VIEW %1").arg(view));
		if(!query.first())
			return result;
	
		QString viewCreateStatement = query.record().value("Create View").toString();
		if(viewCreateStatement.isEmpty())
			return result;
		
		QStringList deps;
		QStringList fromList = viewCreateStatement.split(QRegExp("from [\\(]+`"));
		// the first one is not interesting, because it's the part in front of the from
		fromList.removeFirst();
		foreach(QString from, fromList){
			QString subStr = from.left(from.indexOf("`"));
			if(!subStr.isEmpty())
				deps << subStr;
		}
		
		QStringList joinList = viewCreateStatement.split(QRegExp("join `"));
		joinList.removeFirst();
		foreach(QString join, joinList){
			QString subStr = join.left(join.indexOf("`"));
			if(!subStr.isEmpty())
				deps << subStr;
		}
		
		// insert all views to the list
		foreach(QString dep, deps){
			if(views.contains(dep))
				dependencies.insert(view, dep);
		}
		
		QApplication::processEvents();
	}
	
	// sorting views
	// first put all without any dependencies
	foreach(QString view, views){
		if(doAbort)
			return result;
		if(dependencies.values(view).size() == 0){
			result << view;
			views.removeAll(view);
		}
		QApplication::processEvents();
	}
	
	// try to add all other in some rounds
	// we try 15 times all views, if this is not enough, we give up
	int maxTries = views.size() * 15;
	int i = 0;
	
	QStringList::const_iterator iter;
	iter = views.constBegin();
	while(views.size() > 0){
		if(doAbort)
			return result;
		if(i >= maxTries){
			log(tr("Could not calculate dependencies of all views, some views are missing!"));
			break;
		}
		if(iter == views.constEnd())
			iter = views.constBegin();
		
		bool allFound = true;
		foreach(QString dep, dependencies.values((*iter))){
			if(result.indexOf(dep) < 0)
				allFound = false;
		}
		if(allFound){
			result << (*iter);
			views.removeAll((*iter));
		}

		iter++;
		i++;
		
		QApplication::processEvents();
	}
	
	return result;
}

void EDBBackup::setRange(int num)
{
	ui->progressBar->setRange(0, num);
}

void EDBBackup::finishedStep()
{
	ui->progressBar->setValue ( ui->progressBar->value() + 1 );
}

void EDBBackup::actualStep(QString msg)
{
	ui->stepLabel->setText(msg);
	ui->logEdit->appendPlainText(msg);
}

void EDBBackup::log(QString msg)
{
	ui->logEdit->appendPlainText(msg);
}
