/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/


#include "ehelpcenter.h"
#include "ehelpcenter.moc"

#include "esettings.h"

#include <QProcess>
#include <QSettings>
#include <QLibraryInfo>
#include <QDir>
#include <QFileInfo>
#include <QLocale>
#include <QApplication>

EHelpCenter::EHelpCenter ( EHelpCenterSetup setup, QObject* parent ) : EObject ( parent )
{
	Q_UNUSED(parent);
	settings = setup;
	assistantProcess = 0;
}

EHelpCenter::~EHelpCenter()
{
	if (assistantProcess){
		if(assistantProcess->state() == QProcess::Running) {
			assistantProcess->terminate();
			assistantProcess->waitForFinished(3000);
		}
		delete assistantProcess;
	}
}

void EHelpCenter::showDocumentation ( QString page )
{
	if (!startAssistant())
		return;

	QString cmd;
	// check wether a keyowrd is used
	if(page.contains("keyword:", Qt::CaseInsensitive)){
		cmd = QString("activateKeyword %1").arg(page.right(page.length() - page.indexOf(":") - 1));
	}
	else{
		if(page.isEmpty())
			page = settings.defaultPage;
		
		// prepend /
		if(page.left(1) != "/")
			page = "/"+ page;

		QString domain = settings.domain;
		if(domain.isEmpty())
			domain = QString("de.%1.%2")
								.arg(QApplication::organizationName())
								.arg(QApplication::applicationName());
		if(!domain.startsWith("qthelp://"))
			domain = "qthelp://"+ domain;
		QString cmd = QString("setSource %1%2").arg(domain).arg(page);
	}

	assistantProcess->write(cmd.toLocal8Bit() +'\0');
	
	// make sure, that the toc points to the same document as the content
	assistantProcess->write(QString("SyncContents").toLocal8Bit() + '\0');
}

bool EHelpCenter::startAssistant()
{
	if(!assistantProcess)
		assistantProcess = new QProcess(this);

	// in the case it is already running, also the setup
	// can be skipped
	if (assistantProcess->state() == QProcess::Running)
		return true;

	// setting up the QtAssistant instance
	// find correct language file
	ESettings eSettings;
	QLocale qLocale;
	QString lang = eSettings.value (
		"defaultLang",
		qLocale.name().left(2).toUpper()
	).toString();

	// find collection file (qhcFile)
	QStringList filter;
	filter.append ( QString("*%1*.qhc").arg(settings.qhcFile) );
	foreach(QDir dir, settings.directories){
		QStringList docFiles = dir.entryList ( filter, QDir::Files | QDir::Readable );
		if(docFiles.size() > 0){
			qhcFile = QFileInfo(dir.path() + "/" + fileForLang(docFiles));
			break;
		}
	}
	
	// find all other possible qch files
	filter.clear();
	filter << "*.qch";
	foreach(QDir dir, settings.directories){
		QStringList docFiles = dir.entryList ( filter, QDir::Files | QDir::Readable );
		if(docFiles.size() > 0){
			// remove all strings, which are mostly similar to the qhc file
			QStringList remove = docFiles.filter(QString("%1").arg(settings.qhcFile));
			foreach(QString del, remove)
				docFiles.removeAll(del);
			if(docFiles.size() > 0)
				qchFiles << QFileInfo(dir.path() + "/" + fileForLang(docFiles));
		}
	}


	// now create an instance of QtAssistant
 #if defined(Q_OS_WIN32)
	QString app = QLatin1String("assistant");
 #else
	QString app  = QLibraryInfo::location(QLibraryInfo::BinariesPath) + QDir::separator();
	#if defined(Q_OS_MAC)
		app += QLatin1String("Assistant.app/Contents/MacOS/Assistant");
	#else
		app += QLatin1String("assistant");
	#endif
 #endif

	QStringList args;
	if(qhcFile.isReadable())
		args << QLatin1String("-collectionFile") << qhcFile.absoluteFilePath().toLatin1();
	foreach(QFileInfo qchFile, qchFiles){
		if(qchFile.isReadable())
			args << QLatin1String("-register") << qchFile.absoluteFilePath().toLatin1();
	}
	args << QLatin1String("-enableRemoteControl");
	assistantProcess->start(app, args);
	if (!assistantProcess->waitForStarted(5000)) {
			emit error(tr("Could not start Qt Assistant from %1.").arg(app));
			return false;
	}
	
	// expanding the tree
	// assistantProcess->write(QString("ExpandToc 2").toLocal8Bit() + '\0');
	
	return true;
}

QString EHelpCenter::fileForLang(QStringList files)
{
	ESettings settings;
	QLocale qLocale;
	QString lang = settings.value
	(
		"defaultLang",
		qLocale.name().left(2).toUpper()
	).toString();

	// find files with wanted language
	QStringList langFiles = files.filter("_"+ lang +".", Qt::CaseInsensitive);
	if(langFiles.size() != 1 || lang.isEmpty()){
		// choose the english version, which can be identified
		// by en, EN or no language suffix.
		langFiles = files.filter("_en.", Qt::CaseInsensitive);
		if(langFiles.size() == 1)
			return langFiles[0];
		langFiles = files.filter(QRegExp("_[A-Za-z]{2}\\."));
		if(langFiles.size() == 1)
			return langFiles[0];
		return QString();
	}

	return langFiles[0];
}
