/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "elibrary.h"
#include "elibconfig.h"

#include "epart.h"
#include "esettingsmodelitem.h"
#include "esettingsgeneral.h"
#include "emimetypesettingswidget.h"
#include "esettings.h"
#include "eaboutperson.h"
#include "epluginloader.h"
#include "emodulelistwidgetitem.h"

#include <QApplication>
#include <QPluginLoader>
#include <QUrl>
#include <QSettings>
#include <QResource>
#include <QTranslator>
#include <QLibraryInfo>

#ifdef BINDINGS_KDE
  #include <KApplication>
#endif

QDir ELibrary::ePartDirectory()
{
	return QDir( QString( ENGSAS_EPART_DIR ) );
}

EPart* ELibrary::loadEPartFirstTime(QString ePart, QString& error)
{
	// load and test EPart
	QString file;
	QFileInfo info(ePart);
	if(info.isAbsolute()){
		if(!info.isReadable()){
			error.append(QString("EPart not found!\n%1\n").arg(info.absoluteFilePath()));
			return NULL;
		}
		file = info.absoluteFilePath();
	}
	else{
		QDir dir = ELibrary::ePartDirectory();
		if ( !dir.exists() ) {
			error.append(QString("EPart directory not found!\n%1\n" ).arg ( dir.absolutePath() ));
			return NULL;
		}

#ifdef Q_OS_WIN32
		QString suffix = "dll";
#else
		QString suffix = "so";
#endif
		QStringList filter;
		filter << QString("*%1*").arg(ePart);
		QStringList files = dir.entryList (filter, QDir::Files | QDir::Readable);
		if(files.size() > 1){
			filter[0] = QString("%1*").arg(ePart);
			files = dir.entryList (filter, QDir::Files | QDir::Readable);
			// can be also smaller than 1, if we've guessed badly
			if(files.size() != 1){
				filter[0] = QString("*%1.%2").arg(ePart).arg(suffix);
				files = dir.entryList (filter, QDir::Files | QDir::Readable);
				if(files.size() != 1){
					filter[0] = QString("%1.%2").arg(ePart).arg(suffix);
					files = dir.entryList (filter, QDir::Files | QDir::Readable);
				}
			}
		}
		
		if ( files.size() != 1 ) {
			error.append(QString("%1 not found at \n%2\n" ).arg(ePart).arg ( dir.absolutePath() ));
			error.append(QString("\nFilters: %1\nFound files: %2\n").arg(filter.join(", ")).arg(files.join(", ")));
			return NULL;
		}
		file = dir.absoluteFilePath(files[0]);
	}

	EPluginLoader<EPart> eLoader;
	EModuleListWidgetItem *item = new EModuleListWidgetItem();
	EPart *part = eLoader.doLoadPlugin(file, item);
	if(item)
		error.append(item->errorMessage());
  return part;
}

EPart* ELibrary::loadEPart(EPart *part, QString ePart, QString& error, bool doSetupApplication)
{
	if(!part)
		return NULL;

	if(!loadTranslations(part, error))
		return NULL;
	if(!loadResources(part, error))
		return NULL;
	if(doSetupApplication)
		if(!setupApplication(part, error))
			return NULL;

	// so unload EPart and load it again (to load it with translated strings)
	delete part;
	part = ELibrary::loadEPartFirstTime(ePart, error);
	if(!part)
		return NULL;

	return part;
}

bool ELibrary::setupApplication(EPart* part, QString& error)
{
	return setupApplication(part->shortName(), part->versionString(), error);
}

bool ELibrary::setupApplication(QString shortName, QString version, QString& error)
{
	if(shortName.isEmpty()){
		error.append("Empty short name is not allowed as application name\n");
		return false;
	}
	QApplication::setOrganizationName("EngSaS");
	QApplication::setApplicationName(shortName);
	QApplication::setApplicationVersion(version);
	return true;
}

bool ELibrary::loadTranslations(EPart* part, QString& error)
{
	return loadTranslations(part->translationDir(), error);
}

bool ELibrary::loadTranslations(QString dir, QString& error)
{
	if(dir.isEmpty())
		return true;
	QDir qDir = QDir(dir);

  // try to load translation-files
	QLocale qLocale;
	ESettings settings;
	QString lang = settings.value (
		"defaultLang",
		qLocale.name().left(2).toUpper()
	).toString();
	// define file ending
	QStringList filter;
	filter.append ( "*.qm" );

	if ( !qDir.exists() ) {
		error.append("Directory with translations does not exists\n");
		return false;
	}

	QStringList newFiles = qDir.entryList ( filter, QDir::Files | QDir::Readable );
	foreach ( QString file, newFiles ) {
		// Needed for the translation of the app
		// only add the translations of the chosen language
		if(file.contains(lang, Qt::CaseInsensitive)){
			QTranslator *translator = new QTranslator ( qApp );
			translator->load ( qDir.path() + "/" + file );
			qApp->installTranslator ( translator );
		}
	}

	// load translations of LibEngsas
	qDir = QDir( QString( SHAREDIR ) +"/lang" );
	if ( !qDir.exists() ){
		error.append("Could not find translation directory of LibEngsas\n");
		return false;
	}

	newFiles = qDir.entryList ( filter, QDir::Files | QDir::Readable );
	foreach ( QString file, newFiles ) {
		// Needed for the translation of the app
		// only add the translations of the chosen language
		if(file.contains(lang, Qt::CaseInsensitive)){
			QTranslator *translator = new QTranslator ( qApp );
			translator->load ( qDir.path() + "/" + file );
			qApp->installTranslator ( translator );
		}
	}

	// load default translations of qt
	// this must be done only on unix, because on windows,
	// the qt-translation file is also at lang-directory
 #ifndef Q_OS_WIN32
	QTranslator qtTranslator;
	qtTranslator.load("qt_" + QLocale::system().name(),
	QLibraryInfo::location(QLibraryInfo::TranslationsPath));
	qApp->installTranslator(&qtTranslator);
#endif
	return true;
}

bool ELibrary::loadResources(EPart* part, QString& error)
{
	return loadResources(part->resourceDir(), error);
}

bool ELibrary::loadResources(QString dir, QString& error)
{
  QDir qDir = QDir(dir);
	QStringList filter;
	filter.append ( "*.rcc" );

	if ( !qDir.exists() ) {
		error.append("Resource dir not found!\n");
		return false;
	}

	QStringList newFiles = qDir.entryList ( filter, QDir::Files | QDir::Readable );
	foreach ( QString file, newFiles ) {
		QResource::registerResource ( qDir.path() + "/" + file );
	}

	qDir = QDir(dir);

	if ( !qDir.exists() ) {
		error.append("Resource dir not found!\n");
		return false;
	}

	// load resources of LibEngsas
	qDir = QDir( QString ( SHAREDIR ) + "/rcc" );
	if(!qDir.exists()){
// 		error.append("Resource dir of LibEngsas not found!");
		return true;
	}
	newFiles = qDir.entryList ( filter, QDir::Files | QDir::Readable );
	foreach ( QString file, newFiles ) {
		QResource::registerResource ( qDir.path() + "/" + file );
	}
	return true;
}

QString ELibrary::manufacturer()
{
	return "EngSaS - Engineering Solutions and Services Langenbach";
}

QUrl ELibrary::website()
{
	return QObject::tr("http://en.engsas.de");
}

ESettingsModelItem* ELibrary::settingsModelItem()
{
	ESettingsModelItem *item = new ESettingsModelItem(QObject::tr("Common Settings"), 0);
	item->addChild(new ESettingsGeneral());
#ifdef BINDINGS_KDE
  if(KApplication::kApplication()) {
    item->addChild(new ESettingsModelItem(QObject::tr("Mime Types are handled by KDE."), 0));
  } else {
#endif
	item->addChild(new EMimeTypeSettingsWidget());
#ifdef BINDINGS_KDE
  }
#endif
	return item;
}

EngSaS::AboutPersons ELibrary::aboutPersons()
{
	EngSaS::AboutPersons result;
	result << new EAboutPerson("Henri", "Girard", EAboutPerson::Translator);
	return result;
}
