/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "emainwindow.h"
#include "emainwindow.moc"

#include <QApplication>
#include <QMessageBox>

#include "ewidget.h"
#include "eobject.h"
#include "efiledialog.h"
#include "eaction.h"
#include "esettingsdialog.h"
#include "eaboutdialog.h"
#include "esettingsmodel.h"
#include "elibrary.h"
#include "ehelpcenter.h"
#include "edependenciesdialog.h"
#include "esetupwizard.h"
#include "eaboutmodel.h"

#include <QSettings>
#include <QFile>
#include <QMenuBar>
#include <QMenu>
#include <QStatusBar>
#include <QSignalMapper>

EMainWindow::EMainWindow(QWidget* parent, Qt::WindowFlags flags)
	: QMainWindow(parent, flags)
{
	helpCenter = 0;
	recentlyUsedMapper = new QSignalMapper(this);
	connect(recentlyUsedMapper, SIGNAL(mapped(QString)), this, SLOT(fileOpen(QString)));
	
	if(QApplication::organizationName().isEmpty())
		QApplication::setOrganizationName("EngSaS");

	setupUi();
}

EMainWindow::~EMainWindow()
{
	close();
	if(helpCenter)
		delete helpCenter;
}

void EMainWindow::setWindowTitle(const QString& title)
{
	QMainWindow::setWindowTitle ( QString("%1 - %2").arg(QApplication::applicationName()).arg(title) );
}

void EMainWindow::enableActions(EngSaS::AvailableActions enabledActions)
{
	// disable all first
	fileNewAction->setVisible(false);
	fileOpenAction->setVisible(false);
	fileRecentlyUsedMenu->menuAction()->setVisible(false);
	fileSaveAction->setVisible(false);
	fileSaveAsAction->setVisible(false);
	filePrintAction->setVisible(false);
	editUndoAction->setVisible(false);
	editRedoAction->setVisible(false);
	editCutAction->setVisible(false);
	editCopyAction->setVisible(false);
	editPasteAction->setVisible(false);
	viewReloadAction->setVisible(false);
	
	// reenable according to the flags
	if(enabledActions.testFlag(EngSaS::FileNew))
		fileNewAction->setVisible(true);
	if(enabledActions.testFlag(EngSaS::FileOpen)){
		fileOpenAction->setVisible(true);
		fileRecentlyUsedMenu->menuAction()->setVisible(true);
	}
	if(enabledActions.testFlag(EngSaS::FileSave)){
		fileSaveAction->setVisible(true);
		fileSaveAsAction->setVisible(true);
	}
	if(enabledActions.testFlag(EngSaS::FileSaveAs))
		fileSaveAsAction->setVisible(true);
	if(enabledActions.testFlag(EngSaS::FilePrint))
		filePrintAction->setVisible(true);
	if(enabledActions.testFlag(EngSaS::EditUndo))
		editUndoAction->setVisible(true);
	if(enabledActions.testFlag(EngSaS::EditRedo))
		editRedoAction->setVisible(true);
	if(enabledActions.testFlag(EngSaS::EditCut))
		editCutAction->setVisible(true);
	if(enabledActions.testFlag(EngSaS::EditCopy))
		editCopyAction->setVisible(true);
	if(enabledActions.testFlag(EngSaS::EditPaste))
		editPasteAction->setVisible(true);
	if(enabledActions.testFlag(EngSaS::ViewReload))
		viewReloadAction->setVisible(true);

	controleMenuStati();
}

void EMainWindow::connectESlots(QObject* object)
{
	EObject::connectByPass(this, object);
}

void EMainWindow::fileNew()
{
	EWidget *widget = getCentralWidget();
	if(!widget)
		return;
	QString msg;
	if(!widget->fileNew(msg)){
		if(msg.isEmpty())
			msg = tr("Could not create new file!");
		emit error(msg);
	}
}

void EMainWindow::fileOpen()
{
	EWidget *widget = getCentralWidget();
	if(!widget)
		return;
	QSettings settings;
	QString selectedFilter = widget->fileFilters(EngSaS::Load);
	QString newFile = EFileDialog::getOpenFileName(this, tr("Open file..."),
				settings.value("lastFile").toString(),
				selectedFilter, &selectedFilter);
	fileOpen(newFile);
}

void EMainWindow::fileOpen(QString absoluteFilePath)
{
	if(absoluteFilePath.isEmpty())
		return;
	EWidget *widget = getCentralWidget();
	if(!widget)
		return;
	QFile file(absoluteFilePath);
	if(!file.open(QIODevice::ReadOnly)){
		emit error(tr("Could not open file %1.").arg(file.fileName()));
		return;
	}
	QString msg;
	bool result = widget->load(&file, msg);
	file.close();
	if(!result){
		if(msg.isEmpty())
			msg = tr("Could not load file");
		emit error(msg);
		return;
	}
	QSettings settings;
	settings.setValue("lastFile", absoluteFilePath);
	addToRecentlyUsed(absoluteFilePath);
}

void EMainWindow::fileSave()
{
	EWidget *widget = getCentralWidget();
	if(!widget)
		return;
	
	QSettings settings;
	QString newFile = settings.value("lastFile").toString();
	fileDoSave(newFile);
}

void EMainWindow::fileSaveAs()
{
	EWidget *widget = getCentralWidget();
	if(!widget)
		return;
  QSettings settings;
  QString selectedFilter = widget->fileFilters(EngSaS::Save);
  QString newFile = EFileDialog::getSaveFileName(this, tr("Save as..."),
        settings.value("lastFile").toString(),
        selectedFilter, &selectedFilter);
  if(newFile.isEmpty())
    return;

	fileDoSave(newFile);
}

void EMainWindow::fileDoSave(QString file)
{
	QSettings settings;
	EWidget *widget = getCentralWidget();
	if(!widget)
		return;
	
	QFile qFile(file);
	QString msg;
  bool result = widget->save(&qFile, msg);
	if(!result){
		if(msg.isEmpty())
			msg = tr("Could not save data to file %1.").arg(qFile.fileName());
		emit error(msg);
		return;
	}
	settings.setValue("lastFile", qFile.fileName());
	addToRecentlyUsed(file);
}

void EMainWindow::filePrint()
{
	EWidget *widget = getCentralWidget();
	if(!widget)
		return;
	QString msg;
	widget->print(msg);
}

void EMainWindow::fileExit()
{
	close();
}

void EMainWindow::editCut()
{
	EWidget *widget = getCentralWidget();
	if(!widget)
		return;
	QString msg;
	widget->cut(msg);
}

void EMainWindow::editCopy()
{
	EWidget *widget = getCentralWidget();
	if(!widget)
		return;
	QString msg;
	widget->copy(msg);
}

void EMainWindow::editPaste()
{
	EWidget *widget = getCentralWidget();
	if(!widget)
		return;
	QString msg;
	widget->paste(msg);
}

void EMainWindow::editUndo()
{
	EWidget *widget = getCentralWidget();
	if(!widget)
		return;
	QString msg;
	widget->undo(msg);
}

void EMainWindow::editRedo()
{
	EWidget *widget = getCentralWidget();
	if(!widget)
		return;
	QString msg;
	widget->redo(msg);
}

void EMainWindow::viewReload()
{
	EWidget *widget = getCentralWidget();
	if(!widget)
		return;
	QString msg;
	widget->reload(msg);
}

void EMainWindow::helpHelp()
{
	helpHelp(QString());
}

void EMainWindow::helpHelp ( QString file )
{
  if(!helpCenter){
		helpCenter = new EHelpCenter(helpCenterSetup, this);
		connectESlots(helpCenter);
	}
	helpCenter->showDocumentation(file);
}

void EMainWindow::helpAbout()
{
	showAbout(QPixmap(), QApplication::applicationName(), QApplication::applicationVersion(), QUrl(), 0);
}

void EMainWindow::error(const QString& message)
{
	QMessageBox::critical ( this, QApplication::applicationName(), message );
}

void EMainWindow::warning(const QString& message)
{
	QMessageBox::warning ( this, QApplication::applicationName(), message );
}

void EMainWindow::information(const QString& message)
{
	statusbar->showMessage ( message );
}

void EMainWindow::reloadEnableActions()
{
	EWidget *widget = qobject_cast<EWidget*>(centralWidget());
	if(widget)
		enableActions(widget->enableActions());
}

void EMainWindow::addMenu(QMenu* newMenu, const QString& parent, QString before)
{
	if(!newMenu)
		return;

	if ( parent.isEmpty() ) {
		// addedMenu is a workaround, because QMenu::setParent isn't working
		QMenu *addedMenu = new QMenu(newMenu->title(), menubar);
		addedMenu->setObjectName(newMenu->objectName());
		addedMenu->setIcon(newMenu->icon());
		menubar->insertMenu(findBefore(menubar, before), addedMenu );
		delete newMenu;
// 		newMenu->setParent (menubar);
  }
  else{
		QList<QMenu*> parentMenus = menubar->findChildren<QMenu *> ( parent );
		QMenu *parentMenu = otherMenu;
		if( parentMenus.size() == 1 )
			parentMenu = parentMenus[0];

		// addedMenu is a workaroung, because QMenu::setParent isn't working
		QMenu *addedMenu = new QMenu(newMenu->title(), parentMenu);
		addedMenu->setObjectName(newMenu->objectName());
		addedMenu->setIcon(newMenu->icon());
		parentMenu->insertMenu(findBefore(parentMenu, before), addedMenu );
		delete newMenu;
// 		newMenu->setParent ( parentMenu );
	}

	controleMenuStati();
}

void EMainWindow::addAction(QAction* newAction, const QString& parent, QString before)
{
	if(!newAction)
		return;

	QList<QMenu*> parentMenus;
	QMenu *parentMenu = otherMenu;
	if ( !parent.isEmpty() )
		parentMenus = menubar->findChildren<QMenu *> ( parent );

	// found parent, so use it
  if ( parentMenus.size() == 1)
		parentMenu = parentMenus[0];

	parentMenu->insertAction ( findBefore(parentMenu, before), newAction );
	newAction->setParent ( parentMenu );

	controleMenuStati();
}

void EMainWindow::addToolBar(QToolBar* newToolBar, Qt::ToolBarArea area)
{
	QMainWindow::addToolBar(area, newToolBar);
}

void EMainWindow::addNewDockWidget(QDockWidget* widget, const Qt::DockWidgetArea& area)
{
	addDockWidget ( area, widget );
}

void EMainWindow::setCentralWidget(QWidget* centralWidget)
{
	QSettings settings;
	EWidget *widget = getCentralWidget();
	if(widget){
		settings.beginGroup("geometries");
		settings.setValue(widget->objectName() +"_geometry", saveGeometry());
		settings.setValue(widget->objectName() +"_state", saveState());
		settings.endGroup();
	}
	QMainWindow::setCentralWidget ( centralWidget );
	widget = getCentralWidget();
	if(!widget)
		return;
	enableActions(widget->enableActions());
	hide();
	settings.beginGroup("geometries");
	restoreGeometry(settings.value(widget->objectName() +"_geometry").toByteArray());
	restoreState(settings.value(widget->objectName() +"_state").toByteArray());
	settings.endGroup();
	show();
}

void EMainWindow::showHelp(const QString& page)
{
	helpHelp(page);
}

void EMainWindow::showStatusWidget(QWidget* widget)
{
	statusbar->addPermanentWidget(widget);
}

void EMainWindow::showSetupWizard(EngSaS::SetupWizardObjects objects,
																	bool msgOnNotNeeded,
																	QIcon logo,
																	QString applicationName)
{
	ESetupWizard wizard(this);
	connectESlots(&wizard);
	if(!logo.isNull())
		wizard.setLogo(logo);
	if(!applicationName.isEmpty())
		wizard.setApplicationName(applicationName);
	wizard.loadObjects(objects);
	if(wizard.isNeeded()){
		wizard.exec();
		return;
	}
	if(msgOnNotNeeded)
		QMessageBox::information(this,
														 QApplication::applicationName(),
														 tr("Everything is configured properly and the setup wizard isn't needed.")
														 );
}

void EMainWindow::showSettings(ESettingsModel* model)
{
	if(!model){
		information(tr("There are no settings available"));
		return;
	}
	model->addItem(ELibrary::settingsModelItem());
	ESettingsDialog dialog(model, this);
	connectESlots(&dialog);
	dialog.exec();
}

void EMainWindow::showDependencies(EDependencyModel* model)
{
	if(!model){
		information(tr("No dependencies given"));
		return;
	}
	EDependenciesDialog dialog(model, this);
	connectESlots(&dialog);
	dialog.exec();
}

void EMainWindow::showAbout(QPixmap logo, QString name, QString version,
														QUrl url, EAboutModel *model)
{
	foreach(EAboutPerson *person, ELibrary::aboutPersons())
		model->addPerson(person);
	EAboutDialog dialog(logo, name, version, url, model, this);
	connectESlots(&dialog);
	dialog.exec();
}

void EMainWindow::setupHelpCenter(EHelpCenterSetup setup)
{
	helpCenterSetup = setup;
}

void EMainWindow::closeEvent(QCloseEvent* event)
{
	QSettings settings;
	settings.beginGroup("geometries");
  settings.setValue(objectName() +"_geometry", saveGeometry());
  settings.setValue(objectName() +"_state", saveState());
	settings.endGroup();
	
	QMainWindow::closeEvent(event);
}

void EMainWindow::restoreWindowState()
{
	QSettings settings;
	settings.beginGroup("geometries");
  restoreGeometry(settings.value(objectName() +"_geometry").toByteArray());
  restoreState(settings.value(objectName() +"_state").toByteArray());
	settings.endGroup();
}

EWidget* EMainWindow::getCentralWidget()
{
	return qobject_cast<EWidget*>(centralWidget());
}

void EMainWindow::controleMenuStati()
{
	QList<QMenu*> menus = menubar->findChildren<QMenu*>();
	foreach(QMenu *menu, menus){
		// process only top level menus
		// (childs of menubar). All others
		// are processed as childs of the specific parent menu
		if(menu->parent() == menubar)
			controleMenuStati(menu);
	}
}

void EMainWindow::controleMenuStati(QMenu *menu){
	// do not handle fileRecentlyUsedMenu,
	// because it's already handled enough
	if(menu == fileRecentlyUsedMenu)
		return;

	// check wether all childs are hidden
	bool allHidden = true;
	// check sub menus
	foreach(QMenu *subMenu, menu->findChildren<QMenu*>()){
		controleMenuStati(subMenu);
		if(subMenu->isVisible())
			allHidden = false;
	}
	// check contained actions, if all submenus are hidden.
	// Otherwise, if a submenu is visible, this menu must be
	// shown at all and we do not need to check the status of the actions.
	if(allHidden){
		foreach(QAction *action, menu->findChildren<QAction*>()){
			if(action != menu->menuAction() && action->isVisible())
				allHidden = false;
		}
	}
	if(allHidden)
		menu->menuAction()->setVisible(false);
	else
		menu->menuAction()->setVisible(true);
}

void EMainWindow::addToRecentlyUsed(QString absoluteFilePath)
{
	QSettings settings;
	QStringList recentlyUsed = settings.value("recentlyUsed").toStringList();
	if(recentlyUsed.contains(absoluteFilePath))
		recentlyUsed.removeAll(absoluteFilePath);
	recentlyUsed.prepend(absoluteFilePath);
	for(int i = 10; i < recentlyUsed.size(); i++)
		recentlyUsed.removeLast();
	settings.setValue("recentlyUsed", recentlyUsed);
	loadRecentlyUsed();
}

QAction* EMainWindow::findBefore(QObject* parent, QString name)
{
	if(!parent)
		return 0;
	
	QAction *before = parent->findChild<QAction*>(name);
	if(before)
		return before;

	QMenu *beforeMenu = parent->findChild<QMenu*>(name);
	if(beforeMenu)
		return beforeMenu->menuAction();
	return 0;
}

void EMainWindow::clearRecentlyUsed()
{
	QSettings settings;
	settings.setValue("recentlyUsed", QStringList());
	qDeleteAll(recentlyUsedActions);
	recentlyUsedActions.clear();
}

void EMainWindow::setupUi()
{
	menubar = menuBar();
	menubar->setObjectName("EMainWindowMenuBar");
	statusbar = statusBar();
	statusbar->setObjectName("EMainWindowStatusBar");

	// file menu
	fileMenu = menubar->addMenu(tr("File"));
	fileMenu->setObjectName("fileMenu");

	fileNewAction = new EAction("document-new", ":/libengsas/document-new.png", tr("New"), fileMenu, true);
	fileNewAction->setObjectName("fileNewAction");
	fileNewAction->setShortcut(QKeySequence("Ctrl+N"));
	connect(fileNewAction, 			SIGNAL(triggered()), this, SLOT(fileNew()));
	fileMenu->addAction(fileNewAction);
	
	fileOpenAction = new EAction("document-open", ":/libengsas/document-open.png", tr("Open"), fileMenu, true);
	fileOpenAction->setObjectName("fileOpenAction");
	fileOpenAction->setShortcut(QKeySequence("Ctrl+O"));
	connect(fileOpenAction, 		SIGNAL(triggered()), this, SLOT(fileOpen()));
	fileMenu->addAction(fileOpenAction);

	fileRecentlyUsedMenu = fileMenu->addMenu(tr("Recently Used"));
	fileRecentlyUsedMenu->setObjectName("fileRecentlyUsedMenu");
	recentylUsedSeperator = fileRecentlyUsedMenu->addSeparator();
	recentylUsedSeperator->setObjectName("fileRecentlyUsedSeperator");
	EAction *clearRecentlyUsedAction = new EAction("edit-clear", ":/libengsas/edit-clear.png", tr("Clear"), fileMenu, true);
	fileRecentlyUsedMenu->setObjectName("clearRecentlyUsedAction");
	fileRecentlyUsedMenu->addAction(clearRecentlyUsedAction);
	connect(clearRecentlyUsedAction, SIGNAL(triggered()), this, SLOT(clearRecentlyUsed()));

	fileSaveAction = new EAction("document-save", ":/libengsas/document-save.png", tr("Save"), fileMenu, true);
	fileSaveAction->setObjectName("fileSaveAction");
	fileSaveAction->setShortcut(QKeySequence("Ctrl+S"));
	connect(fileSaveAction, 			SIGNAL(triggered()), this, SLOT(fileSave()));
	fileMenu->addAction(fileSaveAction);

	fileSaveAsAction = new EAction("document-save-as", ":/libengsas/document-save-as.png", tr("Save as"), fileMenu, true);
	fileSaveAsAction->setObjectName("fileSaveAsAction");
	connect(fileSaveAsAction, 			SIGNAL(triggered()), this, SLOT(fileSaveAs()));
	fileMenu->addAction(fileSaveAsAction);

	filePrintAction = new EAction("document-pring", ":/libengsas/document-print.png", tr("Print"), fileMenu, true);
	filePrintAction->setObjectName("filePrintAction");
	filePrintAction->setShortcut(QKeySequence("Ctrl+P"));
	connect(filePrintAction, 			SIGNAL(triggered()), this, SLOT(filePrint()));
	fileMenu->addAction(filePrintAction);

	QAction *sep = fileMenu->addSeparator();
	sep->setObjectName("fileMenuSeperatorBeforeExit");
	fileExitAction = new EAction("application-exit", ":/libengsas/application-exit.png", tr("Exit"), fileMenu, true);
	fileExitAction->setObjectName("fileExitAction");
	connect(fileExitAction, 			SIGNAL(triggered()), this, SLOT(fileExit()));
	fileMenu->addSeparator();
	fileMenu->addAction(fileExitAction);

	// edit menu
	editMenu = menubar->addMenu(tr("Edit"));
	editMenu->setObjectName("editMenu");
	editUndoAction = new EAction("edit-undo", ":/libengsas/edit-undo.png", tr("Undo"), editMenu, true);
	editUndoAction->setObjectName("editUndoAction");
	editUndoAction->setShortcut(QKeySequence("Ctrl+Z"));
	connect(editUndoAction, 			SIGNAL(triggered()), this, SLOT(editUndo()));
	editMenu->addAction(editUndoAction);

	editRedoAction = new EAction("edit-redo", ":/libengsas/edit-redo.png", tr("Redo"), editMenu, true);
	editRedoAction->setObjectName("editRedoAction");
	editRedoAction->setShortcut(QKeySequence("Ctrl+Shift+Z"));
	connect(editRedoAction, 			SIGNAL(triggered()), this, SLOT(editRedo()));
	editMenu->addAction(editRedoAction);

	editCutAction = new EAction("edit-cut", ":/libengsas/edit-cut.png", tr("Cut"), editMenu, true);
	editCutAction->setObjectName("editCutAction");
	editCutAction->setShortcut(QKeySequence("Ctrl+X"));
	connect(editCutAction, 			SIGNAL(triggered()), this, SLOT(editCut()));
	editMenu->addAction(editCutAction);

	editCopyAction = new EAction("edit-copy", ":/libengsas/edit-copy.png", tr("Copy"), editMenu, true);
	editCopyAction->setObjectName("editCopyAction");
	editCopyAction->setShortcut(QKeySequence("Ctrl+C"));
	connect(editCopyAction, 			SIGNAL(triggered()), this, SLOT(editCopy()));
	editMenu->addAction(editCopyAction);

	editPasteAction = new EAction("edot-paste", ":/libengsas/edit-paste.png", tr("Paste"), editMenu, true);
	editPasteAction->setObjectName("editPasteAction");
	editPasteAction->setShortcut(QKeySequence("Ctrl+V"));
	connect(editPasteAction, 			SIGNAL(triggered()), this, SLOT(editPaste()));
	editMenu->addAction(editPasteAction);

	sep = editMenu->addSeparator();
	sep->setObjectName("editMenuSeperatorBeforeSetupWizard");
	editSetupWizardAction = new EAction(tr("Setup Wizard"), editMenu);
	editSetupWizardAction->setObjectName("editSetupWizardAction");
	connect ( editSetupWizardAction, SIGNAL ( triggered() ), this, SLOT( editSetupWizard()));
	editMenu->addAction(editSetupWizardAction);
	
	editSettingsAction = new EAction("package-settings", ":/libengsas/configure.png", tr("Settings"), editMenu, true);
	editSettingsAction->setObjectName("editSettingsAction");
	connect(editSettingsAction, 			SIGNAL(triggered()), this, SLOT(editSettings()));
	editMenu->addAction(editSettingsAction);

	// view Menu
	viewMenu = menubar->addMenu(tr("View"));
	viewMenu->setObjectName("viewMenu");
	viewReloadAction = new EAction("view-refresh", ":/libengsas/view-reload.png", tr("Reload"), viewMenu, true);
	viewReloadAction->setObjectName("viewReloadAction");
	viewReloadAction->setShortcut(QKeySequence("F5"));
	connect(viewReloadAction, 			SIGNAL(triggered()), this, SLOT(viewReload()));
	viewMenu->addAction(viewReloadAction);

	// other menu
	otherMenu = menubar->addMenu(tr("Other"));
	otherMenu->setObjectName("otherMenu");
	
	// help menu
	helpMenu = menubar->addMenu(tr("Help"));
	helpMenu->setObjectName("helpMenu");
	helpHelpAction = new EAction("help-contents", ":/libengsas/help.png", tr("Help"), helpMenu, true);
	helpHelpAction->setObjectName("helpHelpAction");
	helpHelpAction->setShortcut(QKeySequence("F1"));
	connect(helpHelpAction, 			SIGNAL(triggered()), this, SLOT(helpHelp()));
	helpMenu->addAction(helpHelpAction);

	helpDependenciesAction = new EAction("", "", tr("About used software"), helpMenu, true);
	helpDependenciesAction->setObjectName("helpDependenciesAction");
	connect(helpDependenciesAction, SIGNAL(triggered()), this, SLOT(helpDependencies()));
	helpMenu->addAction(helpDependenciesAction);

	helpAboutAction = new EAction("help-about", ":/libengsas/help-about.png", tr("About"), helpMenu, true);
	helpAboutAction->setObjectName("helpAboutAction");
	connect(helpAboutAction, 			SIGNAL(triggered()), this, SLOT(helpAbout()));
	helpMenu->addAction(helpAboutAction);
	
	enableActions(EngSaS::AvailableActions());

	controleMenuStati();
	loadRecentlyUsed();
}

void EMainWindow::loadRecentlyUsed()
{
	QSettings settings;
	QStringList recentlyUsed = settings.value("recentlyUsed").toStringList();
	qDeleteAll(recentlyUsedActions);
	recentlyUsedActions.clear();
	foreach(QString file, recentlyUsed){
		QAction *action = new QAction(file, fileRecentlyUsedMenu);
		fileRecentlyUsedMenu->insertAction(recentylUsedSeperator, action);
		connect(action, SIGNAL(triggered()), recentlyUsedMapper, SLOT(map()));
		recentlyUsedMapper->setMapping(action, file);
		recentlyUsedActions << action;
	}
}
