/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "esetupwizard.h"
#include "esetupwizard.moc"
#include "ui_esetupwizard.h"

#include "esetupwizardwidget.h"
#include "edbupdatewidget.h"
#include "esetupwizarddbdump.h"

#include <QMessageBox>
#include <QApplication>

ESetupWizard::ESetupWizard(QWidget *parent, Qt::WindowFlags fl )
 : EDialog ( "eSetupWizard", parent, fl ), ui(new Ui::ESetupWizard)
{
	ui->setupUi ( this );
	setWindowTitle("SetupWizard");
	setLogo(QApplication::windowIcon());
	setApplicationName(QApplication::applicationName());
  needed = true;
	doReload = false;

	setHelp("keyword:Setupwizard", ui->helpButton);
	
  firstTitle = ui->titleLabel->text();

  connect ( ui->stackedWidget, SIGNAL ( currentChanged ( int ) ), this, SLOT ( setProgress() ) );
  connect ( ui->stackedWidget, SIGNAL ( widgetRemoved ( int ) ), this, SLOT ( setProgress() ) );

  connect ( ui->cancelButton, SIGNAL ( clicked() ), this, SLOT ( reject() ) );
  connect ( ui->nextButton, SIGNAL ( clicked() ), this, SLOT ( next() ) );
  connect ( ui->backButton, SIGNAL ( clicked() ), this, SLOT ( back() ) );
}

ESetupWizard::~ESetupWizard()
{
	foreach(ESetupWizardObject *object, objects)
		unloadObject(object);
	foreach(ESetupWizardObject *object, autoAddedObjects)
		unloadObject(object);
}

void ESetupWizard::setLogo(QIcon logo)
{
	if(logo.isNull())
		return;
	QPixmap pixmap = logo.pixmap(128, 128);
	ui->logoLabel->setMinimumSize(pixmap.size());
	ui->logoLabel->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	ui->logoLabel->setPixmap(pixmap);
}

void ESetupWizard::setApplicationName(QString name)
{
	ui->titleLabel->setText(tr("Welcome to the setup wizard of %1").arg(name));
}

bool ESetupWizard::isNeeded()
{
  return needed;
}

void ESetupWizard::reload()
{
	doReload = false;
	ui->stackedWidget->setCurrentIndex(0);
	foreach(ESetupWizardObject *object, objects){
		object->reloadSettings();
		if(!object->needed())
			unloadObject(object);
	}

	if(objects.size() < 1)
		accept();
	
	ui->stackedWidget->setCurrentIndex(1);
}

void ESetupWizard::loadObjects(EngSaS::SetupWizardObjects objects)
{
	this->objects = objects;
	loadObjects();
}

int ESetupWizard::exec()
{
  if ( isNeeded() )
    return QDialog::exec();

  return QDialog::Accepted;
}

void ESetupWizard::setProgress()
{
  int index = ui->stackedWidget->currentIndex();
  ui->progressBar->setRange ( 0, ui->stackedWidget->count() );
  ui->progressBar->setValue ( index );

  // disable and enable back and next button

  if ( index == 0 ) {
    ui->backButton->setEnabled ( false );
    ui->titleLabel->setText ( firstTitle );
  }

  else {
    ui->backButton->setEnabled ( true );
    ESetupWizardWidget *widget = qobject_cast<ESetupWizardWidget*> ( ui->stackedWidget->currentWidget() );

    if ( widget )
      ui->titleLabel->setText ( widget->name() );
  }

  if ( index >=  ui->stackedWidget->count() - 1 ) {
    ui->nextButton->setText ( tr ( "&Finish" ) );
  }

  else {
    ui->nextButton->setText ( tr ( "&Next" ) );
  }
}

void ESetupWizard::next()
{
	int index = ui->stackedWidget->currentIndex();
	if(index != 0){
		// the first widget is no setupwizardwidget
	  // save settings of last widget
		ESetupWizardWidget *widget = qobject_cast<ESetupWizardWidget*> ( ui->stackedWidget->currentWidget() );
		if ( !widget ){
			emit error ( tr ( "Could not save settings" ) );
			return;
		}
		
		if ( !widget->wizardObject() ){
			emit error ( tr ( "Could not save settings" ) );
			return;
		}
			
		if ( !widget->wizardObject()->finish ( widget ) ) {
			// only emit error if we don't have to reload,
			// because the function don't return something, because the objects 
			// are deleted before they can return a value
			emit error ( tr ( "Could not save settings" ) );
			return;
		}
		if(doReload){
			reload();
			return;
		}
	}  

  if ( index >=  ui->stackedWidget->count() - 1 ) {
    // finish setup
    accept();
  }

  else {
    ui->stackedWidget->setCurrentIndex ( index + 1 );
  }
}

void ESetupWizard::back()
{
  if ( ui->stackedWidget->currentIndex() > 0 )
    ui->stackedWidget->setCurrentIndex ( ui->stackedWidget->currentIndex() - 1 );
}

void ESetupWizard::markReload()
{
	doReload = true;
}

void ESetupWizard::loadObjects()
{
	foreach(ESetupWizardObject *object, objects)
		if(object)
			addObjectsWidgets(object);

	// if we have not added any SetupWizardWidgets's, a setup is not needed
  if ( ui->stackedWidget->count() == 1 ){
    needed = false;
		accept();
	}

	// if one SetupWizardDB is present, insert DBDumpObject as first object
	QList<ESetupWizardWidget*> widgets;
	bool insertBackup = false;
	for(int i = 0; i < ui->stackedWidget->count(); i++){
		ESetupWizardWidget *widget = qobject_cast<ESetupWizardWidget*>(ui->stackedWidget->widget(i));
		if(!widget)
			continue;

		widgets << widget;

		if(!insertBackup){
			EDBUpdateWidget *dbWidget = qobject_cast<EDBUpdateWidget*>(ui->stackedWidget->widget(i));
			if(dbWidget)
				insertBackup = true;
		}
	}
	if(insertBackup){
		ESetupWizardDBDump *dbDump = new ESetupWizardDBDump(this);
		if(dbDump->needed()){
			// the insert widget should be the first one
			foreach(ESetupWizardWidget *widget, widgets)
				ui->stackedWidget->removeWidget(widget);

			addObjectsWidgets(dbDump);
			autoAddedObjects << dbDump;
			foreach(ESetupWizardWidget *widget, widgets)
				ui->stackedWidget->addWidget(widget);
		}
	}

  // at last set the progress to start
  setProgress();
}

void ESetupWizard::addObjectsWidgets ( ESetupWizardObject *object )
{
	object->setParent(this);
  if ( !object )
    return;

  if ( !object->needed() )
    return;

  QList<ESetupWizardWidget*> widgets = object->wizardWidgets();

  if ( widgets.size() == 0 ) {
    emit error ( tr ( "Could not get needed Widget! %1" ).arg ( object->title() ) );
    return;
  }

	foreach ( EWidget *widget, widgets ) {
		connectByPass(widget);
		connect ( widget, SIGNAL ( reloadSettingsRequest()), this, SLOT( markReload()));
    ui->stackedWidget->addWidget ( widget );
  }
}

void ESetupWizard::unloadObject(ESetupWizardObject *object)
{
	if(!object)
		return;
	
	ui->stackedWidget->setCurrentIndex(0);

	// getting all related widgets
	QList<ESetupWizardWidget*> widgets = object->wizardWidgets();
	foreach(ESetupWizardWidget *widget, widgets)
		ui->stackedWidget->removeWidget(widget);
	objects.removeAll(object);
	delete object;
}
