/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "esqlfield.h"
#include "esqlfield.moc"

ESqlField::ESqlField(QObject *parent)
	: EObject(parent)
{
	Q_UNUSED(parent);
    init();
}

ESqlField::ESqlField ( QString myName, ESqlField::Type myType, bool notNull, bool autoIncrement, QObject* parent )
	: EObject(parent)
{
	Q_UNUSED(parent);
    init();
    fieldName = myName;
    setDataType ( myType );
    nn = notNull;
    ai = autoIncrement;
}


void ESqlField::init()
{
    fieldType = ESqlField::NoType;
    pk = false;
    ai = false;
    nn = false;
    zf = false;
    fieldUnsigned = false;
    binary = false;
    fieldLength = 0;

    convFieldTypes.insert ( ESqlField::Integer, "INT" );
    convFieldTypes.insert ( ESqlField::Boolean, "" );
    convFieldTypes.insert ( ESqlField::Text, "TEXT" );
    convFieldTypes.insert ( ESqlField::Varchar, "VARCHAR" );
    convFieldTypes.insert ( ESqlField::Date, "DATE" );
		convFieldTypes.insert ( ESqlField::Time, "TIME" );
    convFieldTypes.insert ( ESqlField::Tinyint, "TINYINT" );
    convFieldTypes.insert ( ESqlField::Double, "DOUBLE" );
    convFieldTypes.insert ( ESqlField::Timestamp, "TIMESTAMP" );
    convFieldTypes.insert ( ESqlField::Blob, "BLOB" );
}

bool ESqlField::isValid() const
{
    if ( ( fieldName.isEmpty() ) || ( fieldType == ESqlField::NoType ) ) {
        return false;
    }

    if ( fieldType == ESqlField::Varchar ) {
        if ( fieldLength < 1 )
            return false;
    }

    if ( ( nn ) && ( zf ) ) {
        // zerofill and not null can not be used at same time
        return false;
    }

    return true;
}

QString ESqlField::name() const
{
    return fieldName;
}

void ESqlField::setName ( QString myName )
{
    fieldName = myName;
}

bool ESqlField::isPrimaryKey() const
{
    return pk;
}

void ESqlField::setIsPrimaryKey ( bool status )
{
    pk = status;
}

bool ESqlField::autoIncrement() const
{
    return ai;
}

void ESqlField::setAutoIncrement ( bool status )
{
    ai = status;
}

bool ESqlField::notNull() const
{
    return nn;
}

void ESqlField::setNotNull ( bool status )
{
    nn = status;
}

QVariant ESqlField::defaultValue() const
{
	bool check;
	fieldDefValue.toInt(&check);
	if ( check )
			return fieldDefValue;
	if(dataType() == ESqlField::Timestamp)
		return fieldDefValue;

   return QVariant();
}

void ESqlField::setDefaultValue ( QVariant value )
{
    fieldDefValue = value;
}

ESqlField::Type ESqlField::dataType() const
{
    return fieldType;
}

void ESqlField::setDataType ( ESqlField::Type myType )
{
    if ( myType == ESqlField::Boolean ) {
        fieldType = ESqlField::Tinyint;
        fieldLength = 1;
        return;
    }

    fieldType = myType;
}

unsigned int ESqlField::length() const
{
    return fieldLength;
}

void ESqlField::setLength ( unsigned int len )
{
    fieldLength = len;
}

bool ESqlField::isUnsigned() const
{
    return fieldUnsigned;
}

void ESqlField::setIsUnsigned ( bool status )
{
    fieldUnsigned = status;
}

bool ESqlField::isBinary() const
{
    return binary;
}

void ESqlField::setIsBinary ( bool status )
{
    binary = status;
}

bool ESqlField::zeroFill() const
{
    return zf;
}

void ESqlField::setZeroFill ( bool status )
{
    zf = status;
}

QString ESqlField::sqlFieldType() const
{
    QString result;

    if ( fieldType == NoType )
        return result;

    result = convFieldTypes.value ( fieldType );

    if ( fieldLength > 0 ) {
        result += "(" + QString::number ( fieldLength ) + ")";
    }

    return result;
}

QString ESqlField::createStatement(EngSaS::SQL::DatabaseType type)
{
  if ( !isValid() ) {
    emit error ( tr ( "SQL Scheme is not valid!\nField: %1" )
                 .arg ( name() )
               );
    return QString();
  }

  QString result = " `" + name() + "` " + sqlFieldType();

  if ( isUnsigned() ) {
    if ( type != EngSaS::SQL::SQLite )
      result += " UNSIGNED ";
  }

  if ( zeroFill() ) {
    result += " ZEROFILL ";
  }

  if ( notNull() ) {
    result += " NOT NULL ";
  }

  if ( autoIncrement() ) {
    if ( type == EngSaS::SQL::SQLite ) {
// 				newQuery += " AUTOINCREMENT ";
    } else {
      result += " AUTO_INCREMENT ";
    }
  }

  if ( !defaultValue().toString().isEmpty() ) {
		result += " DEFAULT '" + defaultValue().toString() + "' ";
  }
  return result;
}

ESqlField* ESqlField::copy() const
{
	ESqlField *newField = new ESqlField(name(), fieldType, notNull(), autoIncrement());
	newField->setIsPrimaryKey(isPrimaryKey());
	newField->setDefaultValue(defaultValue());
	newField->setLength(length());
	newField->setIsUnsigned(isUnsigned());
  newField->setIsBinary(isBinary());
	newField->setZeroFill(zeroFill());
	
	return newField;
}

bool ESqlField::operator==(const ESqlField& field) const
{
	if(field.name() == name())
		return true;
	
	return false;
}

bool ESqlField::operator!=(const ESqlField& field) const
{
	return !operator==(field);
}
