/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "esqlinit.h"
#include "esqlinit.moc"

#include "esqllogin.h"

#include <QSqlError>
#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlDriver>
#include <QSqlField>
#include <QSettings>
#include <QRegExp>
#include <QApplication>

ESqlInit::ESqlInit ( QObject *parent,
							QString username, QString host,
							QString database, QString driver,
							QString password )
	: EObject ( parent )
{
  transactions = false;
  dbType = EngSaS::SQL::Other;
  neededPrivs << ESqlInit::SELECT;
  versionTable = "dbmoduleversions";
	currentVersion = 0;
	actVersion = 0;
	
	this->username = username;
	this->host = host;
	dbName = database;
	this->password = password;
	this->driver = driver;

	if(dynamic_cast<QApplication*>(qApp))
		gui = true;
}

ESqlInit::~ESqlInit()
{
  qDeleteAll ( tables );
}

ESqlInit::Status ESqlInit::login ( bool loginOnly, bool runUpdate )
{
	return login(loginOnly, runUpdate, true);
}

ESqlInit::Status ESqlInit::loginGuiless ( bool loginOnly, bool runUpdate )
{
	return login(loginOnly, runUpdate, false);
}

QString ESqlInit::label() const
{
	return thisName;
}

int ESqlInit::schemaVersion() const
{
	return currentVersion;
}

QString ESqlInit::databaseName() const
{
	return dbName;
}

void ESqlInit::addInformation(QString msg)
{
	emit information(msg);
}

void ESqlInit::reloadSettings()
{
	ESqlLogin wLogin(gui, this);
	connectByPass(&wLogin);
	wLogin.reloadSettings();
	db = ESqlDatabase::database();
	if(!db.databaseName().isEmpty())
		dbName = db.databaseName();
	username = db.userName();
	host = db.hostName();
	password = QString();
}

bool ESqlInit::runUpdateRoutines ( int fromVersion, QString& errorString, int toVersion )
{
	// inital setup should be all donee correctly without update routines
	if(fromVersion == 0)
		return true;

	emit neededSteps(updateRoutines.values().count());
	// since QMap is sorted alread, we start with lowest version as first
	foreach(int funcToVersion, updateRoutines.uniqueKeys()){
		bool (*func)(ESqlInit*, QList<ESqlTable*>, QString &);
		
		// we do not update to newer version than we should
		if((funcToVersion > toVersion) && (toVersion != 0))
			return true;
		
		if(fromVersion < funcToVersion){
			func = updateRoutines.value(funcToVersion);
			if(!func(this, tables, errorString)){
				emit information(tr("  Update function to version %1 failed.").arg(funcToVersion));
				return false;
			}
			
			fromVersion = funcToVersion;
			emit information(tr("  Ran update function to version %1.").arg(fromVersion));
			emit finishedStep();
			QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
		}
	}

	return true;
}

void ESqlInit::addUpdateRoutine(int toVersion, bool (*functionPointer)(ESqlInit*, QList<ESqlTable*>, QString&) )
{
	updateRoutines.insert(toVersion, functionPointer);
}

ESqlTable *ESqlInit::addTable ( QString name, bool appendFieldId, bool addPrefix )
{
	if(name.isEmpty())
		return NULL;
	if(addPrefix)
		name = QString("%1_%2").arg(thisName).arg(name);
  ESqlTable *result = new ESqlTable ( name );
  tables << result;
	if(appendFieldId)
		result->appendFieldId();
  return result;
}

void ESqlInit::addView(QString name, QString selectStatement, bool addPrefix)
{
	if(name.isEmpty())
		return;
	if(selectStatement.isEmpty())
		return;
	if(addPrefix)
		name = QString("%1_%2").arg(thisName).arg(name);
	QStringList view;
	view << name;
	view << selectStatement;
	views << view;
}

void ESqlInit::setup(QString name, QString label, int version)
{
	thisName = name;
	moduleLabel = label;
	currentVersion = version;
}

bool ESqlInit::updateDb()
{
  emit actualStep ( tr ( "Update Database" ) );
  emit information ( tr ( "Database needs to be updated!" ) );
  QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);

  if ( !doUpdate() ) {
//     emit warning ( tr ( "Database cannot be updated to current Version" ) );
    return false;
  }

  emit information ( tr ( "Database is updated" ) );

  return true;
}

ESqlInit::Status ESqlInit::login ( bool loginOnly, bool runUpdate, bool enableGui )
{
	// only allow the change of gui value, if a QApplication allows GUI classes
	if(gui)
		gui = enableGui;

	emit neededSteps ( 0 );
	emit actualStep ( tr ( "Connect to database" ) );
	QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);

	db = ESqlDatabase::database();
	if ( !db.open() ) {
		ESqlLogin wlogin(gui, this);
		if ( !wlogin.loadAndCheckSettings(username, host, dbName, driver, password) )
				return BadSettings;

		connectByPass ( &wlogin );
		if ( wlogin.exec() != QDialog::Accepted )
			return LoginFailed;
	}
	if( loginOnly )
		return LoggedIn;

	db = ESqlDatabase::database();
	// load settings from database object
	QSettings settings;
	settings.beginGroup ( "Database" );
	if(driver.isEmpty()){
		if(!db.driverName().isEmpty()){
			driver = db.driverName();
		}
		else{
			driver = settings.value ( "type" ).toString();
			if(driver.isEmpty()){
				emit error(tr("No SQL-Driver given!"));
				return BadSettings;
			}
		}
	}
	if(driver.contains ( "SQLITE" ) )
		dbType = EngSaS::SQL::SQLite;
	if ( db.driver()->hasFeature ( QSqlDriver::Transactions ) ) {
		transactions = true;
	}
	if(dbName.isEmpty()){
		if(!db.databaseName().isEmpty()){
			dbName = db.databaseName();
		}
		else{
			dbName = settings.value ( "database" ).toString();
			if(dbName.isEmpty()){
				emit error(tr("No Database given!"));
				return BadSettings;
			}
		}
	}
	username = db.userName();
	host = db.hostName();
	settings.endGroup();

	emit information ( tr ( "Checking existence of Database" ) );
	QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
	if ( db.databaseName().isEmpty() ) {
		// we need an update or if we should update, we must create
		// the database
		if ( !runUpdate )
			return UpdateNeeded;

		QSqlQuery query;
		if ( !query.exec ( QString ( "CREATE DATABASE `%1`" ).arg ( dbName ) ) ) {
			emit error ( tr ( "Could not create database!" ) );
			emit information ( QString ( "%1\n%2" ).arg ( query.lastError().text() ).arg ( query.lastQuery() ) );
			return UpdateFailed;
		}
		// we've created the database, now we must reconnect to use the new database
		db.close();
		db.setDatabaseName ( dbName );
		if ( !db.open() ) {
			emit error ( tr ( "Could not create database!" ) );
			emit information ( QString ( "%1\n%2" ).arg ( query.lastError().text() ).arg ( query.lastQuery() ) );
			return UpdateFailed;
		}
	}

	bool result = updateNeeded();
	if ( !runUpdate )
		db.close();
	if ( !result ) {
		// it is no update needed
		// so if we should update, we can return true here, because it's all nice
		// if we should check wether updates are needed, we must return false, because no update is needed
		return NoUpdateNeeded;
	}

	// so below an update is needed
	if ( !runUpdate )
		return UpdateNeeded;

	// below an update is needed and should be performed

	// check if a login with adminrights is needed
	neededPrivs << ESqlInit::INSERT << ESqlInit::UPDATE << ESqlInit::CREATE << ESqlInit::ALTER;

	// login with admin permission
// 	SqlLogin wLogin(tr("Administration rights required"), "admin");
// 	if(!wLogin.loadAndCheckSettings())
// 		return false;
// 	connectByPass(&wLogin);
// 	if(wLogin.exec() != QDialog::Accepted)
// 		return false;

	result = updateDb();

	db.close();

	if(result)
		return NoUpdateNeeded;
	return UpdateFailed;
}

bool ESqlInit::updateNeeded()
{
	init();
  emit actualStep ( tr ( "Checkin if updates are needed" ) );
  QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);

  // check if the versions table is present
  QStringList dbTables = db.tables ( QSql::AllTables );

  if ( dbTables.indexOf ( versionTable ) == -1 ) {
    // table is not there, so a update at least to version one is needed!
    emit information ( tr ( "%1 is not present, but is needed!" ).arg ( versionTable ) );
    return true;
  }

  // check current version
  QSqlQuery query ( "SELECT version FROM dbmoduleversions WHERE (module='" + thisName + "')" );

  if ( !query.first() ) {
    emit information ( tr ( "Cannot select version of %1!\n%2" )
                       .arg ( thisName )
                       .arg ( query.lastError().text() ) );
    return true;
  }

  actVersion = query.value ( 0 ).toUInt();

  if ( actVersion < currentVersion ) {
    emit information ( tr ( "This module-version requires a newer database scheme\nNeeded: %1\nThere: %2" )
                       .arg ( currentVersion )
                       .arg ( actVersion ) );
    return true;
  }

  else if ( actVersion > currentVersion ) {
    emit error ( tr ( "This modul-version requires an older database, so upgrade this module!" ) );
    return false;
  }

  return false;
}

bool ESqlInit::doUpdate()
{
	bool transactions = db.driver()->hasFeature(QSqlDriver::Transactions);
	if(transactions)
		db.transaction();

  emit neededSteps ( tables.size() + views.size() );
  // updating tables
  emit actualStep ( tr ( "Updateting tables" ) );
  QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
  bool result = true;
  QStringList failedTables;
  foreach ( ESqlTable *table, tables ) {
    QString errorString;
    if ( !compareTables ( table, errorString, true ) ) {
      result = false;
      failedTables.append ( "  "+ table->name() + ": \n    " + errorString +"\n" );
    }
    emit finishedStep();
    QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
  }

  if ( !result ) {
    emit error ( tr ( "Unable to update all tables!\nFailed Tables:\n%1" ).arg ( failedTables.join ( "\n" ) ) );
		if(transactions)
			db.rollback();
    return false;
  }

  // Updating views
  emit actualStep ( tr ( "Updateting views" ) );
  QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
  QStringList failedViews;
  result = true;
  foreach ( QStringList view, views ) {
    QString errorString;
    if ( !compareViews ( view, errorString, true ) ) {
      result = false;
      failedViews.append ( view[0] + ": \n" + errorString );
    }
    emit finishedStep();
    QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
  }
  if ( !result ) {
    emit error ( tr ( "Unable to update all Views!\nFailed Views:\n%1" ).arg ( failedViews.join ( "\n" ) ) );
		if(transactions)
			db.rollback();
    return false;
  }
  
  if(actVersion > 0){
		emit actualStep( tr( "Run past update process" ) );
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
		QString errorString;
		if( !runUpdateRoutines(actVersion, errorString, currentVersion) ) {
			emit warning ( tr ( "Database cannot be updated to current Version\n%1" ).arg(errorString) );
			if(transactions)
				db.rollback();
			return false;
		}
		emit finishedStep();
	}

  // set the currentVersion at the table dbmodulesversion
  emit actualStep ( tr ( "Finish database update" ) );

  QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);

  QSqlQuery query ( "SELECT module FROM " + versionTable + " WHERE (module='" + thisName + "')" );

  QString versionQuery;

  if ( !query.first() ) {
    // we must enter the entry
    versionQuery = "INSERT INTO " + versionTable + " (module, moduleLabel, version) VALUES ('" + thisName + "', '" + moduleLabel + "', '" + QString::number ( currentVersion ) + "')";
  }

  else {
    versionQuery = "UPDATE " + versionTable + " SET version=" + QString::number ( currentVersion ) + " WHERE (module='" + thisName + "')";
  }

  if ( !query.exec ( versionQuery ) ) {
    emit error ( tr ( "Cannot update table %1 to current version!\nQuery:%2\nError:%3" )
                 .arg ( versionTable )
                 .arg ( versionQuery )
                 .arg ( query.lastError().text() ) );
		if(transactions)
			db.rollback();
    return false;
	}
	
	if(transactions)
		db.commit();
	
  return true;
}

bool ESqlInit::compareTables ( ESqlTable* table, QString& errorString, bool create )
{
	bool fieldsMustBeDeleted = false;
	errorString = QString();

	QList<ESqlField*> fields;
  QSqlRecord content = db.record ( table->name() );
  QSqlQuery query;
  if ( content.isEmpty() ) {
    if ( !create ) {
      return false;
    }

    else {
      // update db
      if ( !query.exec ( table->createStatement ( dbName, dbType) ) ) {
        errorString.append ( query.lastError().text() +"\n"+ query.lastQuery() );
        return false;
      }
    }
  }
  else {
    // check if we must alter table

		// 1. check fields
		QStringList newFieldNames;
    fields = table->fields();
    QSqlRecord dbFields = db.record ( table->name() );
    foreach ( ESqlField *field, fields ) {
			// The other way round (deleting fields) isn't done here,
			// because you may want to do something with the deleted data,
			// before they are deleted. Therefore delete fields in an update routine.
			// But to avoid errors in this case while adding default values (point 3),
			// they are omitted too. This means, in your update function, you must also
			// insert new default values, if they are any.
      if ( !dbFields.contains ( field->name() ) ) {
        if ( !create )
          return false;
        if ( !query.exec ( QString ( "ALTER TABLE %1 ADD %2" )
                           .arg ( table->name() )
                           .arg ( field->createStatement ( dbType) )
                         ) ) {
          errorString.append ( query.lastError().text() +"\n"+ query.lastQuery() );
          return false;
        }
      }
			// needed to check, if a field should be deleted
			newFieldNames << field->name();
    }
		// check wether a field is into the database, but not in the schema anymore (deletion case)
		for(int i = 0; i < dbFields.count(); i++){
			if(!newFieldNames.contains(dbFields.field(i).name())){
				fieldsMustBeDeleted = true;
				break;
			}
		}

    // 2. check foreingkeys
		// getting the actual SQL statement to create the table
		QSqlQuery query;
		if(!query.exec(QString("SHOW CREATE TABLE %1").arg(table->name()))){
			errorString.append ( query.lastError().text() +"\n"+ query.lastQuery() );
			if(create)
				return false;
		}
		if(!query.first()){
			errorString.append ( query.lastError().text() +"\n"+ query.lastQuery() );
			if(create)
				return false;
		}
		// extracting foreignkey statements
		QStringList createTableStatement = query.record().value("Create Table").toString().split("\n");
		QStringList foreignKeyList;
		// map indexed columns to index names
		QMultiMap<QString, QString> indexes;
		QRegExp key("[\\s]*KEY `([^`]+)` \\(`([^`]+)`");
		foreach(QString statement, createTableStatement){
			if(statement.contains("CONSTRAINT")){
				statement = statement.trimmed();
				if(statement.right(1) == ",")
					statement = statement.left(statement.length() - 1);
				if(!statement.isEmpty())
					foreignKeyList.append(statement);
			}
			else if(key.indexIn(statement) > -1){
				indexes.insert(key.cap(2), key.cap(1));
			}
		}
		// comparing foreignkeys in database and local stored
		QStringList localKeys = table->sqlConstraintsString().split(",\n");
		for(int i = 0; i< localKeys.size(); i++){
			localKeys[i] = localKeys[i].trimmed();
			localKeys[i] = localKeys[i].replace ( QRegExp ( "(%db%)" ), dbName );
			if(localKeys[i].isEmpty())
				localKeys.removeAt(i);
		}
		// check wether foreignkeys should be dropped
		foreach(QString foreignKey, foreignKeyList){
			if(!localKeys.contains(foreignKey)){
				// key is in database, but shouldn't be, remove it
				QRegExp symbol("CONSTRAINT `([\\w]+)` FOREIGN KEY");
				if(symbol.indexIn(foreignKey) == -1){
           errorString.append ( tr("Could not find foreign key symbol to drop it from database\n%1").arg(foreignKey) );
					if(create)
						return false;
				}
				QString fkSymbol = symbol.cap(1);
				if(fkSymbol.isEmpty()){
					errorString.append ( tr("Could not find foreign key symbol to drop it from database\n%1").arg(foreignKey) );
					if(create)
						return false;
				}

				if(!query.exec(QString("ALTER TABLE %1 DROP FOREIGN KEY %2").arg(table->name()).arg(fkSymbol))){
					errorString.append ( query.lastError().text() + "\n" + query.lastQuery() );
					if(create)
						return false;
				}
			}
		}
		// check wether foreignkeys should be added
		foreach(QString foreignKey, localKeys){
			if(!foreignKeyList.contains(foreignKey)){
				// key is not in database, but it should be
				// but before we can add the key, we must add an index manually
				QRegExp reg("CONSTRAINT[\\s]+`([^`]+)`[\\s]+FOREIGN[\\s]+KEY[\\s]+\\(`([^`]+)`\\)[\\s]+REFERENCES");
				if(reg.indexIn(foreignKey.replace("\n", "")) < 0){
					errorString.append (tr("Could not analyze missing foreign key to add index at %1")
						.arg(table->name())
					);
					if(create)
						return false;
				}
				QString indexName = reg.cap(1);
				QString indexColumn = reg.cap(2);
				// check wether index exists and if not
				// create it
				if(!indexes.uniqueKeys().contains(indexColumn)){
					QString indexQuery = QString("CREATE INDEX %2 ON %1 (%3)")
						.arg(table->name())
						.arg(indexName)
						.arg(indexColumn);
					if(!query.exec(indexQuery)){
						errorString.append ( tr("Could not create index to add foreignkey at %1.%2\n%3")
							.arg(table->name())
							.arg(indexName)
							.arg(query.lastError().text())
						);
						if(create)
							return false;
					}
				}
				query.exec("SET foreign_key_checks = 0");
				if(!query.exec(QString("ALTER TABLE %1 ADD %2").arg(table->name()).arg(foreignKey))){
					errorString.append ( query.lastError().text() + "\n" + query.lastQuery()  );
					query.exec("SET foreign_key_checks = 1");
					if(create)
						return false;
				}
				query.exec("SET foreign_key_checks = 1");
			}
		}
  }

  // 3. add default values
	if(!fieldsMustBeDeleted){
		QList<QStringList> values = table->values();
		fields = table->fields();
		// creating the insert field list
		QStringList fieldList, whereFieldList;
		foreach ( ESqlField *field, fields ) {
			if ( field->autoIncrement() )
				continue;

			fieldList.append ( field->name() );
			whereFieldList.append ( field->name() +"=:"+ field->name() );
		}

		foreach ( QStringList valueList, values ) {
			if ( valueList.size() != fields.size() )
				continue;

			// prepare the insert query and the select query (used to
			// check wether the value is already present at the database)
			QSqlQuery insertQuery;
			insertQuery.prepare ( QString ( "INSERT INTO %1 (%2) VALUES(%3)" )
														.arg ( table->name() )
														.arg ( fieldList.join ( ", " ) )
														.arg ( ":"+ fieldList.join ( ", :" ) )
													);
			QSqlQuery selectQuery;
			selectQuery.prepare ( QString ( "SELECT id FROM %1 WHERE %2" )
														.arg ( table->name() )
														.arg ( whereFieldList.join ( " AND " ) )
													);

			for ( int i = 0; i < valueList.size(); i++ ) {
				// we must use fields here, because fieldList is without autoIncremented fields
				// and valueList is with autoIncremented fields
				// But the autoIncremented fields won't be added, because the identifier
				// does not exist in the prepared query
				insertQuery.bindValue ( ":"+fields[i]->name(), valueList[i] );
				selectQuery.bindValue ( ":"+fields[i]->name(), valueList[i] );
			}

			if ( selectQuery.exec () )
				if ( selectQuery.size() < 1 )
					if ( !insertQuery.exec ( ) )
						errorString.append ( tr ( "Cannot add value to %1 \n %2." ).arg ( table->name() ).arg ( query.lastError().text() ) );
		}
	}

  if ( errorString.isEmpty() )
    return true;

  return false;
}

bool ESqlInit::compareViews ( QStringList view, QString& errorString, bool create )
{
  if ( view.size() != 2 ) {
    errorString.append ( tr ("View failed. Not enough fields in QStringList") );
    return false;
  }

  QSqlRecord content = db.record ( view[0] );

	// if we are running updates, we say always update is needed,
	// because we don't know differences between old and new one, so we drop and create again on every update
	if ( !create ) {
		return true;
	}
  else {
		// update db
		QSqlQuery query;
		if(!content.isEmpty()){
			// view exists already, so drop it before inserting again
			if(!query.exec( QString("DROP VIEW %1").arg(view[0]))){
				errorString.append ( tr ("View failed \n") + query.lastError().text() + "\n" + query.lastQuery() );
        return false;
			}
		}

		if ( !query.exec ( createViewStatement ( view ) ) ) {
			errorString.append ( tr ("View failed \n") + query.lastError().text() + "\n" + query.lastQuery() );
			return false;
		}
  }

  return true;
}

QString ESqlInit::createViewStatement ( QStringList view )
{
  if ( view.size() < 2 ) {
    return QString();
  }

  QString result = QString ( "CREATE OR REPLACE VIEW `%1`.`%2` AS %3" )

                   .arg ( dbName )
                   .arg ( view[0] )
                   .arg ( view[1] );

  if ( dbType == EngSaS::SQL::SQLite )
    result = QString ( "CREATE OR REPLACE VIEW `%1` AS %2" )
             .arg ( view[0] )
             .arg ( view[1] );

  return result;
}

void ESqlInit::init()
{
	tables.clear();
	updateRoutines.clear();
	
	// add the database module table
	// to manage versions of modules
	ESqlTable *table =  addTable ( "dbmoduleversions", false, false );

  ESqlField *field = table->appendField ( "module", ESqlField::Varchar, true );
  field->setLength ( 45 );
  table->addIndex ( field );

  field = table->appendField ( "moduleLabel", ESqlField::Varchar, true );
  field->setLength ( 100 );

  field = table->appendField ( "version", ESqlField::Integer );
  field->setIsUnsigned ( true );
  field->setZeroFill ( true );
	
	// add specific tables
  scheme();
}
