/*
		Copyright (C) 2013 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

		This library is free software; you can redistribute it and/or
		modify it under the terms of the GNU Lesser General Public
		License as published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option) any later version.

		This library is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

		You should have received a copy of the GNU Lesser General Public
		License along with this library; if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
		MA 02110-1301  USA.
*/

#include "esqllogin.h"
#include "esqllogin.moc"

#include "elibconfig.h"

#include "esqldatabase.h"
#include "edbsettingswidget.h"
#include "esqllogindialog.h"

#include <QSettings>
#include <QSqlError>

#ifdef BINDINGS_KDE
	#include <QApplication>
	#include <KWallet/Wallet>
#endif

ESqlLogin::ESqlLogin(QObject *parent)
	: EObject(parent)
{
	init(true);
}

ESqlLogin::ESqlLogin(bool enableGui, QObject *parent)
	: EObject(parent)
{
	init(enableGui);
}

ESqlLogin::ESqlLogin(QString title, QString newConnName, bool enableGui, QObject *parent)
{
	init(enableGui, newConnName);
	dialogTitle = title;
}

QStringList ESqlLogin::data()
{
	QStringList result;
	result.append ( username );
	result.append ( password );
	return result;
}

void ESqlLogin::reloadSettings()
{
	ESqlDatabase db = ESqlDatabase::database(connName);
	loadAndCheckSettings(db.userName(), QString(), QString(), QString(), db.password());
	exec();
}

bool ESqlLogin::loadAndCheckSettings(QString username, QString host, QString database,
																		 QString driver, QString password)
{
	if(!username.isEmpty()){
		this->username = username;
#ifdef BINDINGS_KDE
		if(gui){
			// try to set password from kwallet the user
			if(wallet)
				if(wallet->isOpen())
					wallet->readPassword(username, this->password);
		}
#endif
	}

	if(!password.isEmpty())
		this->password = password;

	if(!host.isEmpty())
		this->host = host;
	else
		this->host = EDBSettingsWidget::host();

	if(!driver.isEmpty())
		this->driver = driver;
	else
		this->driver = EDBSettingsWidget::driver();

	if(!database.isEmpty())
		this->database = database;
	else
		this->database = EDBSettingsWidget::database();

	useSSL = EDBSettingsWidget::ssl();

	return checkDBSettingsPrivate();

	// hint to implement ssl usage with QMYSQL driver
	//	db.setConnectOptions("CLIENT_SSL=1;CLIENT_IGNORE_SPACE=1");
	//  QVariant v = db.driver()->handle();
	//  if (v.isValid() && qstrcmp(v.typeName(), "MYSQL*")==0)
	//  {
	//  MYSQL *handle = static_cast<MYSQL *>(v.data());
	//  if (handle != NULL)
	//  {
	//  mysql_ssl_set(handle, "./ssl/client-key.pem",
	//  "./ssl/client-cert.pem", "./ssl/cacert.pem",
	//  NULL, NULL);
	//  }
	//  }
	//  db.open();
}

bool ESqlLogin::checkDBSettings()
{
	ESqlLogin login;
	login.loadAndCheckSettings();
	return login.checkDBSettingsPrivate();
}


ESqlLogin::Result ESqlLogin::login()
{
	// try to get already established connections
	ESqlDatabase checkDB;

	if ( connName.isEmpty() )
		checkDB = ESqlDatabase::database();
	else
		checkDB = ESqlDatabase::database ( connName );

	if ( checkDB.open() ){
		checkDB.close();
		return ESqlLogin::LoggedIn;
	}

	// if we have database like SQLite where no login is needed, we doesn't show the
	// login screen, but connect and return accepted
	ESqlLogin::Result result = doLogin();
	if((result == ESqlLogin::LoggedIn) || (result == ESqlLogin::Canceled) || !gui)
		return result;
	return ESqlLogin::Failed;
}

int ESqlLogin::exec(){
	ESqlLogin::Result result = doLogin();
	if(result != ESqlLogin::LoggedIn)
		return 0;
	return 1;
}

ESqlLogin::Result ESqlLogin::doLogin()
{
	if ( !driver.contains ( "SQLITE" ) ) {
		if ( username.isEmpty() ) {
			if(!gui)
				return ESqlLogin::Failed;

			ESqlLoginDialog *dialog = createDialog();
			if(dialog->exec() != QDialog::Accepted)
				return ESqlLogin::Canceled;
			username = dialog->username();
			password = dialog->password();
			delete dialog;
			return doLogin();
		}

		if ( password.isEmpty() ) {
			if(!gui)
				return ESqlLogin::Failed;

			ESqlLoginDialog *dialog = createDialog();
			dialog->displayError ( tr ( "Please enter your password" ) );
			if(dialog->exec() != QDialog::Accepted)
				return ESqlLogin::Canceled;

			username = dialog->username();
			password = dialog->password();
			delete dialog;
			return doLogin();
		}
	}

	ESqlDatabase db;
	if ( connName.isEmpty() )
		db = ESqlDatabase::database();
	else
		db = ESqlDatabase::database ( connName );

	if ( !connName.isEmpty() ) {
		if ( db.connectionName() != connName ) {
			if ( db.connectionNames().contains ( connName ) ) {
				ESqlDatabase::removeDatabase ( connName );
			}

			db = ESqlDatabase::addDatabase ( driver, connName );
		}
	}
	else {
		if ( !db.isValid() )
			db = ESqlDatabase::addDatabase ( driver );

		if ( db.driverName() != driver ) {
			ESqlDatabase::removeDatabase ( QLatin1String ( "defaultConnection" ) );
			db = ESqlDatabase::addDatabase ( driver );
		}
	}

	if(useSSL){
		emit error("using ssl");
		db.setConnectOptions("CLIENT_SSL=1");
	}
	db.setDatabaseName("");
	if(!withoutDatabase)
		db.setDatabaseName ( database );

	if ( !driver.contains ( "SQLITE" ) ) {
		db.setHostName ( host );
		db.setUserName ( username );
		db.setPassword ( password );
	}

	if ( !db.open() ) {
		if(!withoutDatabase){
			// may be the database does not exists,
			// but SqlInit creates it, so try to connect without database
			withoutDatabase = true;
			return doLogin();
		}
		else{
			// both tries (with and wihtout database) failed,
			// so it must be a wrong username and password.
			// Therefore we set withoutDatabase back to initial value (false)
			withoutDatabase = false;
		}
		// we already tried without database and it doesn't work, so we give up
		if(!gui)
			return ESqlLogin::Failed;
		password = "";
		ESqlLoginDialog *dialog = createDialog();
		dialog->displayError ( tr ( "Can not connect to database:\n%1" ).arg ( db.lastError().text() ) );
		if(dialog->exec() != QDialog::Accepted)
			return ESqlLogin::Canceled;
		username = dialog->username();
		password = dialog->password();
		return doLogin();
	}

#ifdef BINDINGS_KDE
	if((!username.isEmpty()) && (!password.isEmpty()))
		if(gui && wallet)
			if(wallet->isOpen())
				wallet->writePassword(username, password);
#endif

	return ESqlLogin::LoggedIn;
}

void ESqlLogin::showSettingsDialog()
{
	emit showSettings();
}

void ESqlLogin::init(bool enableGui, QString connectionName)
{
	gui = enableGui;
	connName = connectionName;
	useSSL = false;
	withoutDatabase = false;

#ifdef BINDINGS_KDE
	wallet = NULL;
	if(gui){
		wallet = KWallet::Wallet::openWallet(KWallet::Wallet::NetworkWallet(), NULL);
		if(wallet){
			QSettings settings;
			QString myFolder = settings.value("appNameShort", QApplication::applicationName()).toString();
			if(!wallet->hasFolder(myFolder)){
				wallet->createFolder(myFolder);
			}
			if(wallet->setFolder(myFolder) != 0){
				wallet->closeWallet(myFolder, true);
			}
		}
	}
#endif

	if ( !loadAndCheckSettings() ) {
		emit showSettings();
	}
}

ESqlLoginDialog* ESqlLogin::createDialog() const
{
	ESqlLoginDialog *loginDialog = new ESqlLoginDialog(dialogTitle);
	loginDialog->setUsername(username);
	loginDialog->setPassword(password);
	return loginDialog;
}

bool ESqlLogin::checkDBSettingsPrivate()
{
	if(!QSqlDatabase::drivers().contains(driver))
		return false;
	if ( ( driver.contains ( "SQLITE" ) ) && ( !database.isEmpty() ) )
		return true;
	else if ( !host.isEmpty() && !driver.isEmpty() && !database.isEmpty() )
		return true;

	return false;
}
