/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "esqltable.h"
#include "esqltable.moc"

ESqlTable::ESqlTable ( QString name )
{
  setName(name);
  fkActionMap.insert ( ESqlTable::NoAction, "NO ACTION" );
  fkActionMap.insert ( ESqlTable::SetNull, "SET NULL" );
  fkActionMap.insert ( ESqlTable::Cascade, "CASCADE" );
  fkActionMap.insert ( ESqlTable::Restrict, "RESTRICT" );
}

ESqlTable::~ESqlTable()
{
	qDeleteAll(myFields);
}

bool ESqlTable::isValid()
{
  if ( tableName.isEmpty() )
    return false;
  
  if(fields().size() < 1)
		return false;

	foreach(ESqlField *field, fields())
		if(!field->isValid())
			return false;

  return true;
}

QString ESqlTable::name()
{
  // seems to be, that the driver makes all names lower names
  return tableName.toLower();
}

void ESqlTable::setName ( QString name )
{
  tableName = name;
}

QList<ESqlField*> ESqlTable::fields()
{
  return myFields;
}

void ESqlTable::appendField ( ESqlField *append )
{
	if(!append)
		return;
	connectByPass(append);
  myFields.append ( append );
}

ESqlField * ESqlTable::appendField ( QString name, ESqlField::Type type, bool notNull, bool autoIncrement )
{
  ESqlField *result = new ESqlField ( name, type, notNull, autoIncrement );
  appendField ( result );
  return result;
}

void ESqlTable::appendFieldId ( QString name)
{
	// id standard field
  // notNull and autoIncrement are set to true via constructor call
  // unsigned and primary key is seted manually below
	ESqlField *fieldId = appendField ( name, ESqlField::Integer, true, true );
  fieldId->setIsPrimaryKey ( true );
  fieldId->setIsUnsigned ( true );
}

ESqlField * ESqlTable::appendFieldVarchar ( QString name, uint length, bool notNull )
{
	ESqlField *field = appendField(name, ESqlField::Varchar, notNull, false);
	field->setLength(length);
	return field;
}

bool ESqlTable::appendValues ( QStringList value )
{
  if ( value.size() != myFields.size() )
    return false;

  myValues.append ( value );

  return true;
}

QList<QStringList> ESqlTable::values()
{
  return myValues;
}

void ESqlTable::addForeignKey ( QString fieldName,
                               QString refTable,
                               QString refFieldName,
                               ESqlTable::FkAction del,
                               ESqlTable::FkAction update )
{
	if(fieldName.isEmpty())
		return;
	
	if(refTable.isEmpty())
		return;
	
	if(refFieldName.isEmpty())
		return;
	
  QStringList tmp;
  tmp.append ( fieldName );
  tmp.append ( refTable );
  tmp.append ( refFieldName );
  tmp.append ( fkActionMap.value ( del ) );
  tmp.append ( fkActionMap.value ( update ) );
  constraints.append ( tmp );
}

void ESqlTable::addForeignKey ( ESqlField* ourField, ESqlTable* refTable,
                         ESqlTable::FkAction del, ESqlTable::FkAction update )
{
	if(!ourField || !refTable)
		return;
	
	addForeignKey ( ourField->name(), refTable->name(), "id", del, update );
}

void ESqlTable::addForeignKey ( ESqlTable *refTable,
																QString newFieldName,
																ESqlTable::FkAction del,
																ESqlTable::FkAction update,
																QString idFieldName)
{
	if(!refTable)
		return;
	
	QString tablePart = refTable->name().right(refTable->name().length() - refTable->name().indexOf("_") - 1);
	
	// find id field
	QList<ESqlField*> refFields = refTable->fields();
	foreach(ESqlField *refField, refFields){
		if(refField->name() == idFieldName){
			if(newFieldName.isEmpty())
				newFieldName = tablePart + "Id";
			ESqlField *newField = refField->copy();
			newField->setName(newFieldName);
			newField->setIsPrimaryKey(false);
			newField->setAutoIncrement(false);
			appendField(newField);
			addForeignKey(newField, refTable, del, update);
			return;
		}
	}
}

QString ESqlTable::sqlConstraintsString ( EngSaS::SQL::DatabaseType type )
{
  QStringList results;
  foreach ( QStringList tmp, constraints ) {
    QString constraint;
    constraint  = " CONSTRAINT `";
    constraint += "fk_" + name() + "_" + tmp[0] + "`\n";
    constraint += "  FOREIGN KEY (`" + tmp[0] + "`)\n";

    if ( type == EngSaS::SQL::SQLite ) {
      constraint += "  REFERENCES " + tmp[1] + "(`" + tmp[2] + "`)\n";
    }

    else {
      constraint += "  REFERENCES `%db%`.`" + tmp[1] + "`(`" + tmp[2] + "`)\n";
    }

    constraint += "  ON DELETE " + tmp[3] + " ";

    constraint += "  ON UPDATE " + tmp[4];
    results.append ( constraint );
  }

  return results.join ( ",\n" );
}

void ESqlTable::addIndex ( QString fieldName, unsigned int length, Qt::SortOrder order )
{
  // find field
	foreach(ESqlField *field, fields()){
		if(field->name() == fieldName)
			return addIndex(field, length, order);
	}
}

void ESqlTable::addIndex(ESqlField* field, unsigned int length, Qt::SortOrder order)
{
	if(!field)
		return;
	
	QString orderString = "ASC";

	QString lengthString = "";
	if(length > 0){
		if(field->length() > 0){
			if(field->length() < length)
				length = field->length();
		}
		lengthString = QString("(%1)").arg(length);
	}
	
  if ( order == Qt::DescendingOrder )
    orderString = "DESC";

  myIndizes.append ( QString ( "INDEX `index_%1` (`%1`%2 %3)" )
                     .arg ( field->name() )
                     .arg ( lengthString )
                     .arg ( orderString )
                   );
}


QString ESqlTable::indizes ()
{
  return myIndizes.join ( ",\n " );
}

QString ESqlTable::createStatement(QString dbName, EngSaS::SQL::DatabaseType type)
{
	QString newQuery = QString ( "CREATE TABLE IF NOT EXISTS `%1`.`%2` (\n" )
											.arg ( dbName )
											.arg ( name() );

	if ( type == EngSaS::SQL::SQLite )
		newQuery = QString ( "CREATE TABLE IF NOT EXISTS `%1` (\n" )
								.arg ( name() );

	QString primaryKey;

	bool first = true;
	foreach ( ESqlField *field, fields() ) {
		QString fieldString = field->createStatement(type);
		if ( fieldString.isEmpty() )
			return QString();

		if ( first ) {
			first = false;
		} else {
			newQuery += ",\n";
		}
		newQuery += fieldString;

		if ( field->isPrimaryKey() ) {
			primaryKey += ", PRIMARY KEY(`" + field->name() + "`)";
		}
	}

	newQuery += primaryKey;

	// indizes

	if ( !indizes().isEmpty() )
		newQuery += ",\n " + indizes();

	// foreign keys constraints
	QString constraint  = sqlConstraintsString ( type );
	if ( !constraint.isEmpty() )
		newQuery += ",\n" + constraint.replace ( QRegExp ( "(%db%)" ), dbName );

	newQuery += " )";

	return newQuery;
}
