/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "eupdatenoticedialog.h"
#include "eupdatenoticedialog.moc"
#include "ui_eupdatenoticedialog.h"

#include "eopenfileexternal.h"
#include "version.h"
#include "elibrary.h"
#include "enetworkaccessmanager.h"

#include <QSettings>
#include <QNetworkReply>
#include <QNetworkRequest>
#include <QDomDocument>
#include <QDate>
#include <QApplication>

EUpdateNoticeDialog::EUpdateNoticeDialog(QIcon logo, QUrl url,
																				 QWidget* parent, Qt::WindowFlags flags)
	: EDialog(parent, flags), ui(new Ui::EUpdateNoticeDialog)
{
	this->appName = QApplication::applicationName();
	init(logo, url);
}

EUpdateNoticeDialog::EUpdateNoticeDialog(QString appName, QIcon logo, QUrl url,
																				 QWidget* parent, Qt::WindowFlags flags)
	: EDialog(parent, flags), ui(new Ui::EUpdateNoticeDialog)
{
	this->appName = appName;
	init(logo, url);
}


void EUpdateNoticeDialog::check(QString currentVersion)
{
	this->currentVersion = currentVersion;
	if(this->currentVersion.isEmpty())
		this->currentVersion = QApplication::applicationVersion();
	if(this->currentVersion.isEmpty())
		return;

	QSettings settings;
	QDate lastCheck = settings.value("updateCheck").toDate();
	if(!lastCheck.isNull() && lastCheck.daysTo(QDate::currentDate()) < 2){
		// get last available new version
		compare(settings.value("newAppVersion").toString());
		return;
	}
	
	ENetworkAccessManager *http = new ENetworkAccessManager(this);
	QNetworkRequest request(QUrl("http://ftp.engsas.de/versions.xml"));
	QNetworkReply *reply = http->get(request);
	connect(reply, SIGNAL(finished()), this, SLOT(replyFinished()));
}

void EUpdateNoticeDialog::openLink(QString url)
{
	EOpenFileExternal::openFile(url, "", this);
	accept();
}

void EUpdateNoticeDialog::dontShowAgain(int state)
{
	if(state == Qt::PartiallyChecked)
		return;
	
	QSettings settings;
	bool checked = false;
	if(state == Qt::Checked)
		checked = true;
	// since the dialog appears after setting the current available version,
	// we can use it here to store it as the version, for which no updates
	// are displayed anymore
	if(checked)
		settings.setValue("dontShowUpdateNotice", settings.value("newAppVersion").toString());
}

void EUpdateNoticeDialog::replyFinished()
{
	QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
	if(!reply)
		return;

#if defined(Q_OS_WIN32)
	QString platform = "win32";
# endif
#if defined(Q_OS_MACX)
	QString platform = "mac";
#endif
#if defined(Q_OS_LINUX)
	QString platform = "linux";
#endif
		
	if(reply->error() != QNetworkReply::NoError){
		deleteLater();
		return;
	}

	QDomDocument xmlDoc;
	xmlDoc.setContent(reply->readAll());
	reply->deleteLater();
	QDomElement elem = xmlDoc.firstChildElement(appName);
	if(elem.isNull()){
		emit information(tr("Could not get version information!"));
		deleteLater();
		return;
	}
	elem = elem.firstChildElement("platform");

	int major = 0;
	int minor = 0;
	int patch = 0;
	int rc = 0;
	while(!elem.isNull()){
		if(elem.attribute("name") == platform){
			elem = elem.firstChildElement("actualVersion");
			if(!elem.isNull()){
				major = elem.attribute("major", QString("0")).toInt();
				minor = elem.attribute("minor", QString("0")).toInt();
				patch = elem.attribute("patch", QString("0")).toInt();
				rc = elem.attribute("rc", QString("0")).toInt();
			}
		}
		elem = elem.nextSiblingElement("platform");
	}
	compare(Version(major, minor, patch, rc).toString());
	
	deleteLater();
}

void EUpdateNoticeDialog::compare(QString version)
{
	Version actVersion = Version::fromString(version);
	Version curVersion = Version::fromString(currentVersion);
	
	if(!curVersion.isValid()){
		emit information(tr("Could not determine current version!"));
		return;
	}
	if(!actVersion.isValid()){
		emit information(tr("Could not determine actual version!"));
		return;
	}

	QSettings settings;
	bool useUnstable = settings.value("noticeAboutUnstableReleases", false).toBool();
	if(actVersion == Version::fromString(settings.value("dontShowUpdateNotice").toString()))
		return;
	// save actual version
	settings.setValue("newAppVersion", actVersion.toString());
	bool updatePossible = false;
	// only show the notice, if the new version is a stable version (rc=0)
	// or the user has chosen to also install unstable versions
	if(actVersion > curVersion && (actVersion.rc() == 0 || useUnstable))
		updatePossible = true;
	settings.setValue("updateCheck", QDate::currentDate());

	if(updatePossible){
		ui->labelInformation->setText(
			"<h3>"
			+ tr("An Update is available!")
			+ "</h3>"
			+ "<table>"
			+ QString("  <tr><td>%1</td><td>%2</td></tr>")
				.arg(tr("New Version:"))
				.arg(actVersion.toString())
			+ QString("  <tr><td>%1</td><td>%2</td></tr>")
				.arg(tr("Your Version:"))
				.arg(curVersion.toString())
			+ "</table>"
			+ "<br>"
			+ tr("Please download new version at %1.")
				.arg(QString(" <a href=\"%1\">%1</a>")
					.arg(url.toString())
				)
		);
		exec();
	}
	else{
		emit information(tr("No update available!"));
	}
	close();
}

void EUpdateNoticeDialog::init(QIcon logo, QUrl url)
{
	setObjectName("eUpdateNoticeDialog");
	ui->setupUi(this);
	connect(ui->labelInformation, SIGNAL(linkActivated(QString)), this, SLOT(openLink(QString)));
	connect(ui->dontShowAgainEdit, SIGNAL(stateChanged(int)), this, SLOT(dontShowAgain(int)));
	connect(ui->closeButton, SIGNAL(clicked(bool)), this, SLOT(reject()));
	
	ui->headerLabel->setText(appName);
	
	this->url = url;
	if(this->url.isRelative())
		this->url = ELibrary::website().resolved(this->url);
	
	if(!logo.isNull()){
		QPixmap pixmap = logo.pixmap(128, 128);
		ui->logoLabel->setMinimumSize(pixmap.size());
		ui->logoLabel->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
		ui->logoLabel->setPixmap(pixmap);
	}

	loadGeometry();
}
