/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "exmlmodel.h"
#include "exmlmodel.moc"

#include "exmlitem.h"
#include "exmlparseritem.h"

#include <QXmlSchemaValidator>
#include <QXmlSchema>
#include <QTextCodec>
#include <QTextStream>

EXmlModel::EXmlModel(QUrl validationUrl, QObject* parent)
	: EAbstractItemModel(parent)
{
	qRegisterMetaType<EXmlItem*>();
	myRootItem = NULL;
	setValidationSchema(validationUrl);
}

EXmlModel::~EXmlModel()
{
	delete myRootItem;
	QList<EXmlParserItem*> parsersList = parsers.values();
	parsers.clear();
	qDeleteAll(parsersList);
}

bool EXmlModel::load(QDomElement element)
{
	// is a empty document loadable?
	// I assume, an empty document is
	// loaded successfully without doing anything
	if(element.isNull())
		return true;
	
	QStringList keys = parsers.uniqueKeys();
	if(!keys.contains(element.tagName())){
		emit error(tr("no parser found for %1\n"
									"Possible tags: %2")
									.arg(element.tagName())
									.arg(keys.join(", ")));
		return false;
	}

	emit layoutAboutToBeChanged();
	setRootItem(parsers.value(element.tagName())->load(element, 0, parsers));
	emit layoutChanged();
	if(!rootItem()){
		emit error(tr("Could not load model"));
		return false;
	}
	return true;
}

bool EXmlModel::load(QDomDocument document)
{
	return load(document.documentElement());
}

bool EXmlModel::load(QFile *file)
{
	if(!file)
		return false;
	bool closeFile = !file->isOpen();
	if(closeFile){
		if(!file->open(QIODevice::ReadOnly))
			return false;
	}
	QDomDocument document;
	bool result = document.setContent(file);
	if(closeFile)
		file->close();
	if(!result)
		return false;
	return load(document);
}

bool EXmlModel::load(QString file)
{
	if(file.isEmpty())
		return false;
	QFile qFile(file);
	return load(&qFile);
}

bool EXmlModel::load(QByteArray data)
{
	QDomDocument document;
	if(!document.setContent(data))
		return false;
	return load(document);
}

bool EXmlModel::save(QDomElement &useElem)
{
	if(!rootItem())
		return false;
	
	if(!prepareForSaving())
		return false;
		
	if(useElem.isNull())
		return false;
	if(!rootItem()->save(useElem) || useElem.isNull())
		return false;
	return true;
}

bool EXmlModel::save(QFile* file)
{
	if(!file)
		return false;
	bool closeFile = !file->isOpen();
	if(closeFile){
		if(!file->open(QIODevice::WriteOnly))
			return false;
	}
	
	// create empty doc in utf-8 format
	QDomDocument doc = save();
	if(doc.isNull()){
		if(closeFile)
			file->close();
		return false;
	}

	file->resize(0);
  QTextStream stream(file);
  stream.setCodec(QTextCodec::codecForName("UTF-8"));
  stream <<  doc.toString();
	if(closeFile)
		file->close();
	emit information(tr("Document saved to %1.").arg(file->fileName()));
	return true;
}

bool EXmlModel::save(QString file)
{
	if(file.isEmpty())
		return false;
	QFile qFile(file);
	return save(&qFile);
}

QDomDocument EXmlModel::save()
{
	QDomDocument doc;
	QDomProcessingInstruction header = doc.createProcessingInstruction(
		"xml", "version='1.0' encoding='UTF-8'"
	);
  doc.appendChild(header);
	QDomElement result = doc.createElement("tmptag");
	doc.appendChild(result);
	if(!save(result) || result.isNull())
		return QDomDocument();
	
	return doc;
}

bool EXmlModel::isValidXML()
{
	if(!validationUrl.isValid()){
		emit error(tr("No valid schema url given.\n%1").arg(validationUrl.toString()));
		return false;
	}
	
	QUrl schemaUrl(validationUrl);
	QXmlSchema schema;
	schema.load(schemaUrl);
	if (!schema.isValid()){
		emit error(tr("Schema is not valid.\nSchema: %1").arg(validationUrl.toString()));
		return false;
	}
	
	
	QDomDocument doc = save();
	
	QXmlSchemaValidator validator(schema);
	if (!validator.validate(doc.toByteArray())){
		emit error("Instance document is invalid.");
		return false;
	}

	emit information("Document is valid.");
	return true;
}

EXmlItem* EXmlModel::itemFromIndex(QModelIndex index) const
{
	if(!index.isValid())
		return 0;
	return static_cast<EXmlItem*>(index.internalPointer());
}

QModelIndex EXmlModel::indexFromItem(EXmlItem* item) const
{
	if(!item)
		return QModelIndex();
	
	if(item == rootItem())
		return index(0, 0);
	
	return indexFromItem(item, rootItem());
}

bool EXmlModel::removeItem(QModelIndex index)
{
	if(!index.isValid())
		return false;

	EXmlItem *item = itemFromIndex(index);
	EXmlItem *parent = item->parent();
	if(!parent){
		// the only item without a parent is the root item
		setRootItem(NULL);
		delete item;
		return true;
	}
	beginRemoveRows(this->parent(index), index.row(), index.row());
	bool result = parent->removeChild(index.row());
	endRemoveRows();
	return result;
}

bool EXmlModel::removeItem(EXmlItem *item)
{
	return removeItem(indexFromItem(item));
}

void EXmlModel::setRootItem(EXmlItem* item)
{
	bool reset = false;
	if(rootItem()){
		reset = true;
		beginResetModel();
		delete myRootItem;
		myRootItem = NULL;
	}
	if(item){
		if(!reset)
			emit layoutAboutToBeChanged();
		myRootItem = item;
		connect(myRootItem, SIGNAL(layoutAboutToBeChanged()), this, SIGNAL(layoutAboutToBeChanged()));
		connect(myRootItem, SIGNAL(layoutChanged()), this, SIGNAL(layoutChanged()));
		connect(myRootItem, SIGNAL(changed(EXmlItem*)), this, SLOT(itemChanged(EXmlItem*)));
		if(!reset)
			emit layoutChanged();
	}
	if(reset)
		endResetModel();
}

EXmlItem *EXmlModel::rootItem() const
{
	return myRootItem;
}

QVariant EXmlModel::data(const QModelIndex& index, int role) const
{
	EXmlItem *item = itemFromIndex(index);
	if(!item)
		return QVariant();
	
	return item->data(index.column(), role);
}

int EXmlModel::columnCount(const QModelIndex& parent) const
{
	Q_UNUSED(parent);
	return headerList.size();
}

int EXmlModel::rowCount(const QModelIndex& parent) const
{
	// catch special case root item
	if(!rootItem())
		return 0;
	if(parent == QModelIndex())
		return 1;
	if(parent.column() != 0)
		return 0;
	EXmlItem *item = itemFromIndex(parent);
	if(!item)
		return 0;
	return item->rowCount();
}

QModelIndex EXmlModel::parent(const QModelIndex& child) const
{
	EXmlItem *item = itemFromIndex(child);
	if(!item)
		return QModelIndex();
	EXmlItem *parent = item->parent();
	if(!parent)
		return QModelIndex();
	EXmlItem *grandParent = parent->parent();
	if(!grandParent){
		if(!rootItem())
			return QModelIndex();

		return createIndex(0, 0, rootItem());
	}
	
	int row = grandParent->row(parent);
	if(row < 0)
		return QModelIndex();
	return createIndex(row, 0, parent);
}

QModelIndex EXmlModel::index(int row, int column, const QModelIndex& parent) const
{
	if(row < 0)
		return QModelIndex();

	if(row >= rowCount(parent))
		return QModelIndex();

	if(column < 0)
		return QModelIndex();

	if(column >= columnCount(parent))
		return QModelIndex();


  EXmlItem *parentItem = itemFromIndex ( parent );

  if ( !parentItem ){
		if(!rootItem())
			return QModelIndex();
		
		return createIndex(row, column, rootItem());
	}

  return createIndex ( row, column, parentItem->child(row) );
}

QVariant EXmlModel::headerData(int section, Qt::Orientation orientation, int role) const
{
	if(role != Qt::DisplayRole)
		return QVariant();
	
	if(section < 0)
		return QVariant();
	
	if(orientation == Qt::Vertical)
		return section + 1;
	
	if(section > headerList.size())
		return QVariant();
	
	return headerList[section];
}

bool EXmlModel::registerParser(EXmlItem *(*createCallback)(EXmlModel*))
{
	if(!createCallback)
		return false;
	
	EXmlItem *xmlItem = createCallback(this);
	if(!xmlItem)
		return false;
	QString tagName = xmlItem->tagName().toLower();
	delete xmlItem;
	if(tagName.isEmpty())
		return false;
	EXmlParserItem *item = new EXmlParserItem(tagName, createCallback, this);
	if(!item)
		return false;
	parsers.insert(item->tagName(), item);
	return true;
}

void EXmlModel::setValidationSchema(QUrl url)
{
	validationUrl = url;
}

void EXmlModel::itemChanged(EXmlItem *item)
{
	if(!item)
		return;

	QModelIndex itemIndex = indexFromItem(item);
	if(!itemIndex.isValid())
		return;

	QModelIndex end = index(itemIndex.row(), columnCount(itemIndex) - 1, parent(itemIndex));
	if(!end.isValid()){
		emit dataChanged(itemIndex, itemIndex);
		return;
	}
	emit dataChanged(itemIndex, end);
}

QModelIndex EXmlModel::indexFromItem(EXmlItem* find, EXmlItem* parent) const
{
	if(!find)
		return QModelIndex();
	if(!parent)
		return QModelIndex();
	
	// check whether current parent contains the searched item
	// and return a valid QModelIndex, if it is the case
	int row = parent->row(find);
	if(row > -1)
		return createIndex(row, 0, find);
	
	// search through all childs
	for(int i = 0; i < parent->rowCount(); i++){
		QModelIndex result = indexFromItem(find, parent->child(i));
		if(result.isValid())
			return result;
	}
	return QModelIndex();
}
