/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "emimetype.h"

#include "emimeprogram.h"
#include "esettings.h"

#include <QString>
#include <QFileInfo>
#include <QStringList>

EMimeType::EMimeType ( QString newName )
	: EMimeModelItem()
{
  myDefaultProgram = 0;
  setName ( newName );
}

EMimeType::~EMimeType()
{
	qDeleteAll(myPossiblePrograms);
}

void EMimeType::setName ( QString newName )
{
  myName = newName;
}

QString EMimeType::name() const
{
  return myName;
}

void EMimeType::setSuffix ( QString suffix )
{
  mySuffix = suffix;
	if(!suffix.isEmpty())
		setPrefix(QString());
}

QString EMimeType::suffix() const
{
  return mySuffix;
}

void EMimeType::setPrefix ( QString prefix )
{
  myPrefix = prefix;
	if(!prefix.isEmpty())
		setSuffix(QString());
}

QString EMimeType::prefix() const
{
  return myPrefix;
}

void EMimeType::setDefaultProgram ( EMimeProgram *newProgram )
{
  myDefaultProgram = newProgram;
	if(!newProgram)
		return;

	bool found = false;
	foreach(EMimeProgram *app, possiblePrograms()){
		if(app->completePath() == newProgram->completePath()){
			found = true;
			break;
		}
	}
	if(found)
		return;
	// because it is not listed as possible app, we add it
	setPossibleProgram(newProgram);
}

void EMimeType::setDefaultProgram(QString newProgram)
{
	setDefaultProgram( new EMimeProgram(newProgram) );
}

EMimeProgram *EMimeType::defaultProgram() const
{
  return myDefaultProgram;
}

void EMimeType::setPossibleProgram ( QString possibleProgram )
{
  setPossibleProgram ( QFileInfo ( possibleProgram ) );
}

void EMimeType::setPossibleProgram ( QFileInfo possibleProgram )
{
  EMimeProgram *posApp = new EMimeProgram ( possibleProgram.absoluteFilePath(), this );
  posApp->setProgram ( possibleProgram );
  setPossibleProgram ( posApp );
}

void EMimeType::setPossibleProgram ( EMimeProgram *possibleProgram )
{
	if(!possibleProgram)
		return;
  possibleProgram->setParent ( this );
  myPossiblePrograms << possibleProgram;
}

QList<EMimeProgram*> EMimeType::possiblePrograms() const
{
  return myPossiblePrograms;
}

int EMimeType::rowCount() const
{
  return myPossiblePrograms.size();
}

bool EMimeType::saveMimeType()
{
  ESettings settings;
  settings.beginGroup ( group() );

  // saving this mimetype
  QStringList save;
	save.append ( suffix() );
	save.append ( prefix() );

  if ( defaultProgram() )
    save.append ( defaultProgram()->completePath() );
	else
    save.append ( "" );

  QStringList possibleApps;

  foreach ( EMimeProgram *app, possiblePrograms() ) {
    possibleApps.append ( app->completePath() );
  }

  save.append ( possibleApps.join ( "?" ) );
  settings.setValue ( name(), save );

  // appending this mimetype to the saved list
  QStringList savedTypes = settings.value ( "mimeTypes" ).toStringList();

  if ( savedTypes.indexOf ( name() ) == -1 ) {
    savedTypes.append ( name() );
    settings.setValue ( "mimeTypes", savedTypes );
  }

  settings.endGroup();

  return true;
}

bool EMimeType::loadMimeType ( QString newName )
{
  if ( !newName.isEmpty() )
    setName ( newName );

  if ( name().isEmpty() )
    return false;

  ESettings settings;

  settings.beginGroup ( group() );

  // looking up list of stored mimeTypes
  QStringList savedTypes = settings.value ( "mimeTypes" ).toStringList();

  if ( savedTypes.indexOf ( name() ) == -1 ) {
    settings.endGroup();
    return false;
  }

  // it's in the list, so we can get data out of the settings
  QStringList savedData = settings.value ( name() ).toStringList();

  settings.endGroup();

  // it must be 3, because [0] - is suffix, [1] - is defaultProgram
  // [2] - are possible Programs joined with ?
  if ( savedData.size() != 4 )
    return false;

  setSuffix ( savedData[0] );
	setPrefix ( savedData[1] );

	// possible apps must be loaded before default app
	// to avoid double insert of default app
	QStringList possibleApps = savedData[3].split ( "?" );
  foreach ( QString posApp, possibleApps ) {
    setPossibleProgram ( posApp );
  }
  
	setDefaultProgram ( savedData[2] );

  return true;
}

bool EMimeType::operator== ( const EMimeType &compareType ) const
{
  if ( name() == compareType.name() )
    return true;

  return false;
}

bool EMimeType::operator!=(const EMimeType& compareType) const
{
	return !operator==(compareType);
}

