/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "emimetypemodel.h"
#include "emimetypemodel.moc"

#include "emimetype.h"
#include "emimemodelitem.h"
#include "emimeprogram.h"
#include "esettings.h"
#include "eopenfileexternal.h"

#include <QFileSystemModel>

#include <QDebug>

EMimeTypeModel::EMimeTypeModel ( QObject *parent )
	: EAbstractItemModel ( parent )
{
	Q_UNUSED(parent);
}

EMimeTypeModel::~EMimeTypeModel()
{
	emit layoutAboutToBeChanged();
	qDeleteAll(mimeTypes);
	emit layoutChanged();
}

void EMimeTypeModel::addMimeType ( EMimeType *mimeType )
{
  emit layoutAboutToBeChanged();
  mimeTypes.append ( mimeType );
  emit layoutChanged();
}

EMimeType *EMimeTypeModel::getMimeType ( QString mimeType )
{
  for ( int i = 0; i < rowCount(); i++ ) {
    if ( mimeTypes[i]->name() == mimeType )
      return mimeTypes[i];
  }

  return 0;

//  EMimeType *tmp = new EMimeType(mimeType);
//  int index = mimeTypes.indexOf(tmp);
//  delete tmp;
//  if(index == -1)
//    return 0;
//  return mimeTypes[index];
}

EMimeType *EMimeTypeModel::getMimeType ( int row )
{
  if ( ( row < 0 ) || ( row >= mimeTypes.size() ) )
    return 0;

  return mimeTypes[row];
}

QModelIndex EMimeTypeModel::index ( int row, int column, const QModelIndex &parent ) const
{
  if ( !parent.isValid() ) {
    // a mimetype is requested
    if ( ( 0 > row ) || ( row >= mimeTypes.size() ) )
      return QModelIndex();

    if ( ( 0 > column ) || ( column >= columnCount ( parent ) ) )
      return QModelIndex();

    return createIndex ( row, column, mimeTypes[row] );
  }

  if ( parent.column() != 0 )
    return QModelIndex();

  if ( ( 0 > row ) || ( row >= rowCount ( parent ) ) )
    return QModelIndex();

  EMimeType *parentItem = mimeTypes[parent.row() ];

  if ( ( 0 > row ) || ( row >= rowCount ( parent ) ) )
    return QModelIndex();

  if ( ( 0 > column ) || ( column >= columnCount ( parent ) ) )
    return QModelIndex();

  return createIndex ( row, column, parentItem->possiblePrograms() [row] );
}

QModelIndex EMimeTypeModel::parent ( const QModelIndex &index ) const
{
  // if it is a mimeType, the parent is the root node with invalid index QModelIndex()
  EMimeModelItem *item = elementFromIndex ( index );

  if ( !item )
    return QModelIndex();

  EMimeType *mimeType = dynamic_cast<EMimeType*> ( item );

  if ( mimeType )
    return QModelIndex();

  // the application should know it's parent
  EMimeProgram *application = dynamic_cast<EMimeProgram*> ( item );

  if ( application ) {
    EMimeType *parentItem = application->parent();

    if ( !parentItem )
      return QModelIndex();

    // find parent in mimeTypes-List
    int index = mimeTypes.indexOf ( parentItem );

    if ( index == -1 )
      return QModelIndex();

    return createIndex ( index, 0, parentItem );
  }

  return QModelIndex();
}

int EMimeTypeModel::rowCount ( const QModelIndex &parent ) const
{
  EMimeModelItem *item = elementFromIndex ( parent );

  if ( !item )
    return mimeTypes.size();

  EMimeType *mimeType = dynamic_cast<EMimeType*> ( item );

  if ( !mimeType )
    return 0;

  return mimeType->rowCount();
}

int EMimeTypeModel::columnCount ( const QModelIndex &parent ) const
{
  EMimeModelItem *item = elementFromIndex ( parent );

  if ( !item )
    return 4;

  EMimeType *mimeType = dynamic_cast<EMimeType*> ( item );

  if ( !mimeType )
    return 0;

  if ( mimeType->rowCount() > 0 )
    return 1;

  return 0;
}

QVariant EMimeTypeModel::data ( const QModelIndex &index, int role ) const
{
  if ( role != Qt::DisplayRole )
    return QVariant();

  EMimeModelItem *item = elementFromIndex ( index );

  if ( !item )
    return QVariant();

  EMimeType *mimeType = dynamic_cast<EMimeType*> ( item );

  if ( mimeType ) {
    switch ( index.column() ) {
      case 0:
        return mimeType->name();
      case 1:
        return mimeType->suffix();
      case 2:
	return mimeType->prefix();
      case 3:
        if ( mimeType->defaultProgram() )
          return mimeType->defaultProgram()->file().absoluteFilePath();

      default:
        return QVariant();
    }
  }

  EMimeProgram *appItem = dynamic_cast<EMimeProgram*> ( item );

  if ( appItem ) {
    return appItem->file().fileName();
  }

  return QVariant();
}

bool EMimeTypeModel::hasChildren ( const QModelIndex &parent ) const
{
  EMimeModelItem *item = elementFromIndex ( parent );

  if ( !item ) {
    if ( rowCount ( QModelIndex() ) > 0 )
      return true;

    return false;
  }

  EMimeType *mimeItem = dynamic_cast<EMimeType*> ( item );

  if ( mimeItem ) {
    if ( ( rowCount ( parent ) > 0 ) && ( parent.column() == 0 ) )
      return true;

    return false;
  }

  return false;
}

QVariant EMimeTypeModel::headerData ( int section, Qt::Orientation orientation, int role ) const
{
  if ( role != Qt::DisplayRole )
    return QVariant();

  if ( orientation == Qt::Horizontal ) {
    switch ( section ) {
      case 0:
        return tr ( "Mime Type" );
      case 1:
        return tr ( "Suffix" );
      case 2:
        return tr ( "Prefix" );
      case 3:
        return tr ( "Default Program" );
      default:
        return QVariant();
    }
  }

  return QVariant();
}

EMimeModelItem* EMimeTypeModel::elementFromIndex ( const QModelIndex &index ) const
{
  if ( index.isValid() ) {
    return static_cast<EMimeModelItem *> ( index.internalPointer() );

  } else {
    return 0;
  }
}

bool EMimeTypeModel::loadModel()
{
  EMimeType *item = 0;

  ESettings settings;
  settings.beginGroup ( EMimeType::group() );
  QStringList savedTypes = settings.value ( "mimeTypes" ).toStringList();
  foreach ( QString type, savedTypes ) {
    item = new EMimeType ( type );

    if ( item->loadMimeType() )
      addMimeType ( item );
  }

  settings.endGroup();

  QFileSystemModel fileModel(this);
  QString rootPath = "/";
  #if defined(Q_OS_WIN32)
    rootPath = "c:/";
  #endif
  
  fileModel.setRootPath(rootPath);
  
  // adding default mime types
  item = getMimeType ( "application/pdf" );

  if ( !item ) {
    item = new EMimeType ( "application/pdf" );
    item->setSuffix ( "pdf" );
    QStringList filters;
#if defined(Q_OS_WIN32)
    filters << "AcroRd32.exe";
    filters << "Acrobat.exe"; 
    // add adobe readers
    item->setPossibleProgram ( new EMimeProgram ( "Adobe/Reader 8.0/Reader/AcroRd32.exe", item ) );
    item->setPossibleProgram ( new EMimeProgram ( "Adobe/Reader 9.0/Reader/AcroRd32.exe", item ) );
    // add adobe acrobats
    item->setPossibleProgram ( new EMimeProgram ( "Adobe/Acrobat 6.0/Acrobat/Acrobat.exe", item ) );
#else
    filters << "xpdf";
    filters << "okular";
    filters << "acroread";
    filters << "evince";
    filters << "skim";
    filters << "preview";
    item->setPossibleProgram ( new EMimeProgram ( "xpdf", item ) );
    item->setPossibleProgram ( new EMimeProgram ( "okular", item ) );
    item->setPossibleProgram ( new EMimeProgram ( "acroread", item ) );
    item->setPossibleProgram ( new EMimeProgram ( "evince", item ) );
    item->setPossibleProgram ( new EMimeProgram ( "skim", item ) );
    item->setPossibleProgram ( new EMimeProgram ( "preview", item ) ); // MAC OS X
#endif
    fileModel.setNameFilters(filters);
    
    for(int i = 0; i < fileModel.rowCount(); i++){
      QFileInfo info = fileModel.fileInfo(fileModel.index(i, 0));
      qDebug() << QString("Row %1: %2").arg(i).arg(info.absoluteFilePath());
    }
    
    addMimeType ( item );
  }

  // adding default mime types
  item = getMimeType ( "application/http" );

  if ( !item ) {
    item = new EMimeType ( "application/http" );
    item->setPrefix( "http" );
#if defined(Q_OS_WIN32)
//     // add adobe readers
//     item->setPossibleProgram ( new Program ( "Adobe/Reader 8.0/Reader/AcroRd32.exe", item ) );
//     item->setPossibleProgram ( new Program ( "Adobe/Reader 9.0/Reader/AcroRd32.exe", item ) );
//     // add adobe acrobats
//     item->setPossibleProgram ( new Program ( "Adobe/Acrobat 6.0/Acrobat/Acrobat.exe", item ) );
#else
    item->setPossibleProgram ( new EMimeProgram ( "konqueror", item ) );
    item->setPossibleProgram ( new EMimeProgram ( "firefox", item ) );
#endif
    addMimeType ( item );
  }
  
  return true;
}

bool EMimeTypeModel::saveModel()
{
	// first cleanup existing settings
	ESettings settings;
	settings.beginGroup(EOpenFileExternal::group());
	// this call removes all entries in the current group
	settings.remove("");
	settings.endGroup();

	bool result = true;
	foreach(EMimeType *item, mimeTypes)
		if(!item->saveMimeType())
			result = false;
	return result;
}
