/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "elogobjecttest.h"
#include "elogobjecttest.moc"

#include "testwidget.h"
#include "testobject.h"
#include "testthread.h"

#include <elogobject.h>

#include <QTest>
#include <QTextEdit>
#include <QDockWidget>
#include <QTemporaryFile>

#include <QDebug>

void ELogObjectTest::fetchDockWidget(QDockWidget *widget, const Qt::DockWidgetArea &area)
{
	Q_UNUSED(area);
	if(!widget)
		return;
	logEdit = qobject_cast<QTextEdit*>(widget->widget());
}

void ELogObjectTest::initTestCase()
{
	widget = NULL;
	object = NULL;
	logObject = NULL;
}

void ELogObjectTest::setup()
{
	initTest();
	EObject *nullObject = NULL;
	EWidget *nullWidget = NULL;
	logObject->setup(nullObject);
	logObject->setup(nullWidget);
	cleanupTest();
}

void ELogObjectTest::information()
{
	initTest();
	QString string = "test info";
	// with trimming
	object->informationMsg("\n" + string + "\n");
	checkLogEdit(string, Qt::black);

	QString string2 = "test info 2\nwith break";
	widget->informationMsg(string2);
	checkLogEdit(string +"\n"+ string2, Qt::black);
	cleanupTest();
}

void ELogObjectTest::warnings()
{
	initTest();
	QString string = "test string";
	// with trimming
	object->warningMsg("\n" + string + "\n");
	checkLogEdit(string, Qt::darkYellow);

	QString string2 = "test info 2\nwith break";
	widget->warningMsg(string2);
	checkLogEdit(string +"\n"+ string2, Qt::darkYellow);
	cleanupTest();
}

void ELogObjectTest::errors()
{
	initTest();
	QString string = "test string";
	// with trimming
	object->errorMsg("\n" + string + "\n");
	checkLogEdit(string, Qt::red);

	QString string2 = "test info 2\nwith break";
	widget->errorMsg(string2);
	checkLogEdit(string +"\n"+ string2, Qt::red);
	cleanupTest();
}

void ELogObjectTest::messageHandler()
{
	cleanupTest();
	logObject = new ELogObject(true);
	connect(logObject, SIGNAL(addNewDockWidget(QDockWidget*,Qt::DockWidgetArea)),
					this, SLOT(fetchDockWidget(QDockWidget*,Qt::DockWidgetArea)));
	logObject->init();
	QVERIFY(logEdit);

	QStringList strings;
	strings << "test debug";
	qDebug() << strings[0];
	checkLogEdit("\""+ strings[0] +"\"", Qt::lightGray);

	strings << "test warning";
	qWarning() << strings[1];
	checkLogEdit("\""+ strings.join("\"\n\"") +"\"", Qt::darkYellow);

	strings << "test critical";
	qCritical() << strings[2];
	checkLogEdit("\""+ strings.join("\"\n\"") +"\"", Qt::red);
	cleanupTest();
}

void ELogObjectTest::destinationFile()
{
	cleanupTest();

	QTemporaryFile logFile;
	QCOMPARE(logFile.open(), true);
	logObject = new ELogObject(true);
	logObject->init(logFile.fileName());

	QStringList strings;
	strings << "test debug";
	qDebug() << strings[0];

	strings << "test warning";
	qWarning() << strings[1];

	strings << "test critical";
	qCritical() << strings[2];

	QCOMPARE(logFile.reset(), true);
	QString logFileContent(logFile.readAll());
	QCOMPARE(logFileContent, "\""+ strings.join("\" \n\"") + "\" \n");
}

void ELogObjectTest::threadSafety()
{
	cleanupTest();
	logObject = new ELogObject(true);
	connect(logObject, SIGNAL(addNewDockWidget(QDockWidget*,Qt::DockWidgetArea)),
					this, SLOT(fetchDockWidget(QDockWidget*,Qt::DockWidgetArea)));
	logObject->init();
	QVERIFY(logEdit);

	TestThread thread;
	thread.start();
	// wait for thread to be finished
	QTime dieTime = QTime::currentTime().addSecs(2);
	// Userevents should be excluded see http://lists.trolltech.com/qt-interest/2007-01/thread00133-0.html
	while( QTime::currentTime() < dieTime ){
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
		if(thread.isFinished())
			break;
	}
	thread.quit();

	QStringList strings;
	strings << "test debug";
	strings << "test warning";
	strings << "test critical";
	QCOMPARE(logEdit->toPlainText(), strings.join("\n"));


	cleanupTest();
}

void ELogObjectTest::initTest()
{
	// cleanup before
	cleanupTest();

	logObject = new ELogObject(false);
	connect(logObject, SIGNAL(addNewDockWidget(QDockWidget*,Qt::DockWidgetArea)),
					this, SLOT(fetchDockWidget(QDockWidget*,Qt::DockWidgetArea)));
	logObject->init();
	QVERIFY(logEdit);

	object = new TestObject();
	widget = new TestWidget();
	logObject->setup(object);
	logObject->setup(widget);
}

void ELogObjectTest::cleanupTest()
{
	if(widget)
		delete widget;
	if(object)
		delete object;
	if(logObject)
		delete logObject;

	initTestCase();
}

void ELogObjectTest::checkLogEdit(QString text, QColor color)
{
	QVERIFY(logEdit);

	QStringList lines = logEdit->toPlainText().split("\n");
	QStringList stringList = text.split("\n");

	QCOMPARE(lines.size(), stringList.size());
	for(int i = 0; i < lines.size(); i++)
		QCOMPARE(lines[i], stringList[i]);
	QVERIFY(logEdit->textColor() == color);
}

QTEST_MAIN(ELogObjectTest)
