/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "emimetypetests.h"
#include "emimetypetests.moc"

#include "internal/emimetypemodel.h"
#include <modeltest.h>
#include "internal/emimetype.h"
#include "internal/emimeprogram.h"
#include <eopenfileexternal.h>

#include <QtTest/QtTest>
#include <QFileInfo>

void EMimeTypeTests::initTestCase()
{
	// set test application name for QSettings usage
	QApplication::setOrganizationName("EngSaS");
	QApplication::setApplicationName("Testing");

	origState = EOpenFileExternal::mimeTypes();
}

void EMimeTypeTests::cleanupTestCase()
{
	origState->saveModel();
}

void EMimeTypeTests::programmTests()
{
#ifdef Q_OS_WIN32
	QFileInfo app("C:/WINDOWS/system32/cmd.exe");
#else
	QFileInfo app("/usr/bin/which");
#endif
	EMimeProgram prog;
	
	// parent
	EMimeType *parent = new EMimeType();
	prog.setParent(parent);
	QCOMPARE(prog.parent(), parent);
	prog.setParent(0);
	delete parent;

	// Costructor
	QString path = "/a/b/c";
#ifdef Q_OS_WIN32
	path = "C:/a/b/c";
#endif
	EMimeProgram progC(path);
	QCOMPARE(progC.completePath(), QString(path));
	
	// path functions
	prog.setPath(app.absolutePath());
	QCOMPARE(prog.path(), app.absoluteDir());
	prog.setProgram(QFileInfo());
	QCOMPARE(prog.path(), QDir());
	prog.setPath(app.absoluteDir());
	QCOMPARE(prog.path(), app.absoluteDir());
	
	// program functions
	prog.setPath(app.absoluteDir());
	prog.setProgram(app.fileName());
	QCOMPARE(prog.file().absoluteFilePath(), app.absoluteFilePath());
	prog.setProgram(app.fileName());
	prog.setPath(app.absoluteDir());
	QCOMPARE(prog.file().absoluteFilePath(), app.absoluteFilePath());
	app.setFile("/d/e/f");
	prog.setProgram(app);
	QCOMPARE(prog.file().absoluteFilePath(), app.absoluteFilePath());
	QCOMPARE(prog.path(), app.absoluteDir());
	
	// operator == and !=
	EMimeProgram prog1;
	prog.setProgram(QFileInfo("/a/b/f"));
	prog1.setProgram(QFileInfo("/z/d/e/f"));
	QVERIFY(prog == prog1);
	prog1.setProgram(QFileInfo("/f/b/a"));
	QVERIFY(prog != prog1);
	
	EMimeProgram *prog2 = new EMimeProgram("/a/b/c");
	QList<EMimeProgram*> lists;
	lists << prog2;
	QVERIFY(lists.contains(prog2));
	QVERIFY(!lists.contains(&prog1));
}


void EMimeTypeTests::mimeTypeTests()
{
	QString name = "application/pdf";
	QString suffix = "pdf";
	QString prefix = "http";
	EMimeType mimeType(name);
	QCOMPARE(mimeType.name(), name);
	QVERIFY(!mimeType.defaultProgram());
	mimeType.setSuffix(suffix);
	QCOMPARE(mimeType.suffix(), suffix);
	mimeType.setPrefix(prefix);
	QCOMPARE(mimeType.prefix(), prefix);
	QCOMPARE(mimeType.suffix(), QString());
	mimeType.setSuffix(suffix);
	QCOMPARE(mimeType.prefix(), QString());
	
	QString prog1 = "/a/b/c";
	QString prog2 = "/c/d";
#ifdef Q_OS_WIN32
	prog1 = "C:/a/b/c";
	prog2 = "C:/c/d";
#endif
	mimeType.setPossibleProgram(prog1);
	mimeType.setDefaultProgram(prog2);
	QCOMPARE(mimeType.rowCount(), 2);
	QCOMPARE(mimeType.possiblePrograms().size(), mimeType.rowCount());
	QCOMPARE(mimeType.defaultProgram()->file().absoluteFilePath(), prog2);
	mimeType.setDefaultProgram(prog1);
	QCOMPARE(mimeType.rowCount(), 2);
	mimeType.saveMimeType();
	EMimeType mime2;
	mime2.loadMimeType("application/pdf");
	QCOMPARE(mimeType, mime2);
	QCOMPARE(mimeType.defaultProgram()->file().absoluteFilePath(),
					 mime2.defaultProgram()->file().absoluteFilePath());
	QCOMPARE(mimeType.suffix(), mime2.suffix());
	QCOMPARE(mimeType.prefix(), mime2.prefix());
}

void EMimeTypeTests::standardModelTests()
{
	EMimeTypeModel *model = new EMimeTypeModel(this);
	ModelTest *modeltest = new ModelTest(model, this);
	QList<EMimeType*> items;
	items << new EMimeType();
	items << new EMimeType("Test");
	items << new EMimeType("Test 2");
	items[items.size() - 1]->setPossibleProgram("Bla1");
	items[items.size() - 1]->setPossibleProgram("Bla2");
	items[items.size() - 1]->setDefaultProgram(0);
	items << new EMimeType("Test 3");
	items[items.size() - 1]->setPossibleProgram("Bla1");
	items[items.size() - 1]->setPossibleProgram("Bla2");
	items[items.size() - 1]->setPossibleProgram("Bla3");
	items[items.size() - 1]->setDefaultProgram(items[items.size() - 1]->possiblePrograms()[2]);
	
	foreach(EMimeType *type, items)
		model->addMimeType(type);
	
	QCOMPARE(model->rowCount(), items.size());
	
	// remove a min from the beginning, the middle and the end
	if(model->rowCount() > 0)
		model->removeRow(0);
	
	if(model->rowCount() > 0)
		model->removeRow(model->rowCount() - 1);
	
	if(model->rowCount() > 2){
		int half = model->rowCount() / 2;
		model->removeRow(half);
	}
	
	delete modeltest;
// 	delete model;
}

void EMimeTypeTests::mimeModelTests()
{
	EMimeTypeModel *model = new EMimeTypeModel(this);
	QVERIFY(model->loadModel());
	QCOMPARE(model->rowCount(), 2);
	
	// getMimeType(QString)
	QString getType = "application/pdf";
	EMimeType *pdfType = model->getMimeType(getType);
	QVERIFY(pdfType);
	QCOMPARE(pdfType->name(), getType);
	
	// getMimeType(int)
	// first get row
	bool found = false;
	for(int i = 0; i < model->rowCount(); i++){
		if(model->data(model->index(i, 0, QModelIndex())).toString() == pdfType->name()){
			found = true;
			QCOMPARE(model->getMimeType(i), pdfType);
		}
	}
	QVERIFY(found);
	
	// data function on QModelIndex() (MimeType-Level)
	for(int i = 0; i < model->rowCount(); i++){
		EMimeType *tmp = model->getMimeType(i);
		QVERIFY(tmp);
		QCOMPARE(tmp->name(),
						model->data(model->index(i,0, QModelIndex())).toString());
		QCOMPARE(tmp->suffix(),
						 model->data(model->index(i,1, QModelIndex())).toString());
		QCOMPARE(tmp->prefix(),
						 model->data(model->index(i,2, QModelIndex())).toString());
		QString result;
		if(tmp->defaultProgram()){
			result = tmp->defaultProgram()->file().absoluteFilePath();
		}
		QCOMPARE(result,
							model->data(model->index(i,3, QModelIndex())).toString());
		if(tmp->rowCount() > 0){
			// data function on Program-Level
			QCOMPARE(tmp->possiblePrograms()[0]->file().fileName(),
							 model->data(model->index(0, 0, model->index(i, 0, QModelIndex()))).toString()
			);
		}
	}
	
	delete model;
}

QTEST_MAIN(EMimeTypeTests)
