/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "exmlmodeltest.h"
#include "exmlmodeltest.moc"

#include "exmlmodeltestconfig.h"

#include <edependencymodel.h>
#include <edependencyitem.h>
#include <modeltest.h>
#include "edependencytestmodel.h"

#include <QTest>
#include <QTemporaryFile>
#include <QTextStream>

void EXmlModelTest::testEXmlItem()
{
	QString tagName = "testTag";
	QString name = "testName";
	QString version = "testVersion";
	QString url = "testWebsite";
	EDependencyItem item;
	item.setTagName(tagName);
	item.setName(name);
	item.setVersion(version);
	item.setWebsite(url);
	QCOMPARE(item.tagName(), tagName.toLower());
	QCOMPARE(item.name(), name);
	QCOMPARE(item.version(), version);
	QCOMPARE(item.website(), url);
}

void EXmlModelTest::testEXmlItemCopy()
{
	EDependencyItem item;
	item.setTagName("testTag");
	item.setName("testName");
	item.setVersion("testVersion");
	item.setWebsite("testWebsite");
	
	EDependencyItem *copy = dynamic_cast<EDependencyItem*>(item.copy());
	QVERIFY(copy);
	QCOMPARE(copy->tagName(), item.tagName());
	QCOMPARE(copy->name(), item.name());
	QCOMPARE(copy->version(), item.version());
	QCOMPARE(copy->website(), item.website());
}

void EXmlModelTest::testEXmlModel()
{
	QString file = QString( TESTFILEDIR ) +"/dependencies.xml";
	
	EDependencyTestModel *model = new EDependencyTestModel(this);
	connect(model, SIGNAL(error(QString)), this, SLOT(catchErrors(QString)));
	new ModelTest(model, this);

	QDomDocument document;
	QFile qFile("asdasd");
	QDomElement testElem;
	// testing all stuff where an empty rootElem is needed
	QCOMPARE(model->save(testElem), false);
	QCOMPARE(model->save(&qFile), false);
	QTemporaryFile tmpFile;
	if(!tmpFile.open())
		QSKIP("Failed to create temporary file", SkipSingle);
	QCOMPARE(model->save(tmpFile.fileName()), false);
	QCOMPARE(model->isValidXML(), false);
	
	// testing header
	QCOMPARE(model->columnCount(), 3);
	QCOMPARE(model->rowCount(), 0);
	QCOMPARE(model->headerData(0, Qt::Horizontal).toString(), QString("Name"));
	QCOMPARE(model->headerData(1, Qt::Horizontal).toString(), QString("Version"));
	QCOMPARE(model->headerData(2, Qt::Horizontal).toString(), QString("Website"));
	QCOMPARE(model->headerData(0, Qt::Vertical).toInt(), 1);
	QCOMPARE(model->headerData(1, Qt::Vertical).toInt(), 2);
	
	// testing loading
	QCOMPARE(model->load(QString()), false);
	QCOMPARE(model->load(QByteArray()), false);
	QCOMPARE(model->load(0), false);
	// we currently assume, that empty documents are loaded correctly anytime
	QCOMPARE(model->load(QDomElement()), true);
	testElem = document.createElement("notparseabletag");
	QCOMPARE(model->load(testElem), false);
	QCOMPARE(model->load(&qFile), false);
	qFile.close();
	qFile.setFileName(file);
	QCOMPARE(model->load(&qFile), true);
	QCOMPARE(document.setContent(&qFile, false), true);
	QDomElement element = document.documentElement();
	QCOMPARE(element.tagName(), QString("dependency"));
	QCOMPARE(model->load(element), true);
	// check success of loading
	QCOMPARE(model->rowCount(), 1);
	QModelIndex meganizer = model->index(0, 0);
	QCOMPARE(meganizer.isValid(), true);
	QModelIndex qt = model->index(0, 0, meganizer);
	QCOMPARE(qt.isValid(), true);
	QModelIndex qtcore = model->index(0, 0, qt);
	QCOMPARE(qtcore.isValid(), true);
	QModelIndex libmeganizer = model->index(1, 0, meganizer);
	QCOMPARE(libmeganizer.isValid(), true);
	
	QCOMPARE(model->rowCount(qt), 4);
	QCOMPARE(model->hasChildren(qt), true);
	QCOMPARE(model->rowCount(libmeganizer), 0);
	QCOMPARE(model->hasChildren(libmeganizer), false);
	
	// testing data
	QCOMPARE(model->data(meganizer).toString(), QString("Meganizer"));
	QCOMPARE(model->data(model->index(meganizer.row(), 1)).toString(), QString(""));
	QCOMPARE(model->data(model->index(meganizer.row(), 2)).toString(), QString(""));
	QCOMPARE(model->data(qt).toString(), QString("Qt"));
	QCOMPARE(model->data(model->index(qt.row(), 1, meganizer)).toString(), QString("4"));
	QCOMPARE(model->data(model->index(qt.row(), 2, meganizer)).toString(), QString("http://qt.nokia.com/products"));
	QCOMPARE(model->data(qtcore).toString(), QString("Core"));
	QCOMPARE(model->data(model->index(qtcore.row(), 1, qt)).toString(), QString(""));
	QCOMPARE(model->data(model->index(qtcore.row(), 2, qt)).toString(), QString(""));
	
	
	// testing parent
	QCOMPARE(model->parent(qt), meganizer);
	QCOMPARE(model->parent(qtcore), qt);
	QCOMPARE(model->parent(libmeganizer), meganizer);
	
	// testing save
	QCOMPARE(model->save(""), false);
	QCOMPARE(model->save(0), false);
	testElem = QDomElement();
	QCOMPARE(model->save(testElem), false);
	qFile.close();
	qFile.setFileName("ZZ:C///asdasd/asdasd");
	QCOMPARE(model->save(&qFile), false);
	
	QDomDocument compare;
	QDomProcessingInstruction header = compare.createProcessingInstruction(
		"xml", "version='1.0' encoding='UTF-8'"
	);
  compare.appendChild(header);
	QDomElement rootElem = compare.createElement("tmptag");
	compare.appendChild(rootElem);
	QCOMPARE(model->save(rootElem), true);
	QCOMPARE(compare.toString(), document.toString());
	QTemporaryFile tmpFile1;
	if(!tmpFile1.open())
		QSKIP("Failed to create temporary file", SkipSingle);
	QCOMPARE(model->save(tmpFile1.fileName()), true);
	QCOMPARE(compare.setContent(&tmpFile1), true);
	QCOMPARE(compare.toString(), document.toString());
	
	// testing validator
	QCOMPARE(model->isValidXML(), false);
	model->setSchema(QString(SCHEMA));
 	QCOMPARE(model->isValidXML(), true);
}

void EXmlModelTest::catchErrors(QString msg)
{
	QWARN(msg.toLatin1());
}

QTEST_MAIN(EXmlModelTest)
