/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "testmodel.h"
#include "testmodel.moc"

#include "testitem.h"
#include "testsubitem.h"
#include "testmodelroot.h"

TestModel::TestModel(QObject *parent) :
		EXmlModel(QUrl(), parent)
{
	headerList << tr("Name") << tr("Value");
	setRootItem(new TestModelRoot());

	// register parser
	registerParser(&TestModelRoot::create);
	registerParser(&TestItem::create);
	registerParser(&TestSubItem::create);
}

TestItem* TestModel::addTestItem(QString label)
{
	if(!rootItem())
		return NULL;

	QModelIndex root = indexFromItem(rootItem());
	if(!insertRow(rowCount(root), root))
		return NULL;

	TestItem *item = dynamic_cast<TestItem*>(rootItem()->child(rootItem()->rowCount() - 1));
	item->setLabel(label);

	return item;
}

bool TestModel::removeTestItem(TestItem *testItem)
{
	return removeItem(indexFromItem(testItem));
}

void TestModel::clear()
{
	setRootItem(new TestModelRoot());
}

bool TestModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
	if(role != Qt::EditRole)
		return false;

	if(!index.isValid())
		return false;

	if(index == indexFromItem(rootItem()))
		return false;

	TestModelBaseItem *item = qobject_cast<TestModelBaseItem*>(itemFromIndex(index));
	if(!item)
		return false;

	if(index.column() == 0){
		item->setLabel(value.toString());
		return true;
	}
	if(index.column() > 1)
		return false;

	TestSubItem *attr = qobject_cast<TestSubItem*>(item);
	if(!attr)
		return false;
	attr->setValue(value.toString());
	return true;
}

Qt::ItemFlags TestModel::flags(const QModelIndex &index) const
{
	EXmlItem *item = itemFromIndex(index);
	if(!item)
		return Qt::ItemFlags();

	return Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsEditable;
}

bool TestModel::insertRows(int row, int count, const QModelIndex &parent)
{
	if(row < 0 || count < 0)
		return false;

	if(!parent.isValid()){
		if(row > 0)
			return false;

		// do not replace root item
		if(rootItem())
			return false;

		setRootItem(new TestModelRoot());
		return true;
	}

	TestItem *parentItem = qobject_cast<TestItem*>(itemFromIndex(parent));
	beginInsertRows(parent, row, row + count - 1);
	// we do not increment row here, because
	// row + 1 before deletion of row, is row
	// after deletion.
	for(int i = 0; i < count; i++){
		if(parentItem){
			parentItem->addChild(row, new TestSubItem("", this, parentItem), false);
		}
		else{
			TestItem *testItem = new TestItem(this);
			rootItem()->addChild(row, testItem, false);
		}
	}
	endInsertRows();

	return true;
}

bool TestModel::removeRows(int row, int count, const QModelIndex &parent)
{
	if(row < 0 || count < 0)
		return false;
	if(row >= rowCount(parent))
		return false;
	if(row + count > rowCount(parent))
		return false;

	if(!parent.isValid()){
		if(row > 0 || count > 1)
			return false;
		if(!rootItem())
			return false;
		clear();
		return true;
	}

	EXmlItem *parentItem = itemFromIndex(parent);
	if(!parentItem)
		return false;

	bool result = true;
	beginRemoveRows(parent, row, row + count - 1);
	// we do not increment row here, because
	// row + 1 before deletion of row, is row
	// after deletion.
	for(int i = 0; i < count; i++){
		if(!parentItem->removeChild(row))
				result = false;
	}
	endRemoveRows();

	return result;
}

bool TestModel::prepareForSaving()
{
	if(!rootItem())
		return false;
	for(int i = 0; i < rootItem()->rowCount(); i++){
		TestItem *testItem = qobject_cast<TestItem*>(rootItem()->child(i));
#if QT_VERSION >= 0x040800
		if(!Q_UNLIKELY(testItem))
#else
		if(!testItem)
#endif
			return false;
		testItem->setId(QString::number(i));
	}
	return true;
}
