/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef TESTMODELBASEITEM_H
#define TESTMODELBASEITEM_H

#include <exmlitem.h>

#include <QMetaType>

class EXmlModel;

/**
	* @brief Base class of the test model for TestEXmlModel
	*
	*/
class TestModelBaseItem : public EXmlItem
{
	Q_OBJECT
	public:
		TestModelBaseItem(QString tagName = QString(), EXmlItem *parent = NULL);
		virtual inline ~TestModelBaseItem(){ }

		/**
			* @brief Returns true, if the label is not empty.
			*
			* It may be useful to reimplement it at the sub class.
			*/
		virtual bool isValid() const;

		/**
			* @brief The content of this item with maximum length of 120 characters.
			*/
		QString label() const;
		void setLabel(QString label);
		/**
			* @brief More detailed description of the item with maxium length of 1000 characters.
			*
			* Use setComment() to set the value.
			*/
		QString comment() const;
		void setComment(QString comment);
		/**
			* @brief Used to identify this item in an XML Box structure.
			*
			* Per default it is empty, therefore set it with setId()
			* before calling save(), if id is needed.
			*
			* You may want to subclass this function to extand the returned id
			* e. g. with a prefix. To get the id, call this base implementation.
			* In other cases it may be useful to reimplement setId() instead.
			*/
		virtual QString id() const;
		/**
			* @brief Function to change the id() of this item.
			*
			* You may want to overload it, to extend the given id inside your
			* class. To set the id afterwards, call this base implementation. E. g.:
			* @code
			* void setId(QString id){
			*   id = "myPrefix_"+ id;
			*   TestModelBaseItem::setId(id);
			* }
			* @endcode
			* In other cases it may be useful to reimplement id() instead.
			*/
		virtual void setId(QString id);

		// EXmlItem API
		/**
			* @brief This create function can only be used to create plain TestModelBaseItems!
			*
			* @note To create an instance of your subclassed item, you must use
			*       your own create function!
			*/
		static inline EXmlItem *create(EXmlModel *model){ Q_UNUSED(model); return new TestModelBaseItem(""); }
		/**
			* @brief Function to load data from QDomElement.
			*
			* You should reimplement this function in your subclass and
			* remember to load the id() if needed by your subclass.
			*/
		virtual bool load(const QDomElement element, QMap< QString, EXmlParserItem * > parser);
		/**
			* @brief Function to save data to an QDomElement.
			*
			* This implementation changes the tagName of @p useElem to given
			* tagName().
			* You should reimplement this function in your subclass.
			*/
		virtual bool save(QDomElement &useElem);
		/**
			* @brief Used by EXmlModels to extract data from the item.
			*
			* You should reimplement this function in your subclass.
			*/
		virtual QVariant data (int column, int role=Qt::DisplayRole) const;
		/**
			* @brief Used to create copies of an item.
			*
			* You should reimplement this function in your subclass.
			*/
		virtual EXmlItem *copy() const;

	protected:
		/**
			* @brief Loads the basic tags of TestModelBaseItem.
			*
			* Reimplement it in your subclass, if you subclass it another
			* time and need to load your "custom" basic stuff also. In this
			* case, do not forget to call this base implementaton.
			*/
		virtual bool loadBasics(const QDomElement &element);

	private:
		QString myLabel, myComment, myId;

};

#endif // TESTMODELBASEITEM_H
