/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "testtestmodel.h"
#include "testtestmodel.moc"

#include "testsubitem.h"
#include "testitem.h"
#include "testmodel.h"
#include "testmodelroot.h"

#include <modeltest.h>

#include <QTest>
#include <QSignalSpy>

void TestTestModel::testModelBaseItem()
{
	qRegisterMetaType<EXmlItem*>();
	
	TestModelBaseItem item;
	QSignalSpy spy(&item, SIGNAL(changed(EXmlItem*)));

	QCOMPARE(item.isValid(), false);

	QCOMPARE(item.label(), QString());
	QString test = "asdasd";
	item.setLabel(test);
	QCOMPARE(spy.count(), 1);
	QCOMPARE(item.label(), test);
	QCOMPARE(item.isValid(), true);
	
	QCOMPARE(item.comment(), QString());
	QString testComment = "asdasdasdasdasdasd";
	item.setComment(testComment);
	QCOMPARE(item.comment(), testComment);
	QCOMPARE(spy.count(), 2);
	QCOMPARE(item.isValid(), true);
	item.setLabel("");
	QCOMPARE(item.isValid(), false);
}

void TestTestModel::testItemChilds()
{
	TestItem item(NULL);
	QSignalSpy layoutAboutToBeChanged(&item, SIGNAL(layoutAboutToBeChanged()));
	QSignalSpy layoutChanged(&item, SIGNAL(layoutChanged()));

	item.setLabel("testAttribute");
	QCOMPARE(item.rowCount(), 0);

	QStringList items;
	items << "1" << "2" << "3";

	// appending childs
	for(int i = 0; i < items.size(); i++){
		QVERIFY(item.appendChild(items[i], ""));
		QCOMPARE(item.rowCount(), i + 1);
		QCOMPARE(item.child(i)->testItem()->label(), item.label());
		QCOMPARE(layoutAboutToBeChanged.count(), i + 1);
		QCOMPARE(layoutChanged.count(), i + 1);
	}
	// appending item without emitting changes signals
	items << "4";
	QVERIFY(item.appendChild(items[items.size() - 1], "", false));
	QCOMPARE(layoutAboutToBeChanged.count(), items.size() - 1);
	QCOMPARE(layoutChanged.count(), items.size() - 1);
	layoutAboutToBeChanged.clear();
	layoutChanged.clear();

	QCOMPARE(item.row(item.child(1)), 1);

	// removing one in the middle
	QVERIFY(!item.removeChild(-1));
	QVERIFY(!item.removeChild(item.rowCount() + 2));
	QVERIFY(item.removeChild(1));
	QCOMPARE(item.rowCount(), items.size() - 1);
	// is not emitted, because items are added again, if needed
	QCOMPARE(layoutAboutToBeChanged.count(), 0);
	QCOMPARE(layoutChanged.count(), 0);

	// insert child
	QVERIFY(item.rowCount() > 1);
	TestSubItem *testSubItem = new TestSubItem("testInsertChild", NULL, &item);
	testSubItem->setValue("testValue");
	item.addChild(1, testSubItem);
	QCOMPARE(item.child(1)->label(), QString("testInsertChild"));
	QCOMPARE(item.child(1)->value(), QString("testValue"));
}

void TestTestModel::testItemClone()
{
	TestItem *source = new TestItem(NULL);
	source->setLabel("Source 1");
	source->appendChild("Sl 1", "Sv 1");
	source->appendChild("Sl 2", "Sv 2");
	QCOMPARE(source->label(), QString("Source 1"));
	QCOMPARE(source->rowCount(), 2);

	TestItem *target = dynamic_cast<TestItem*>(source->copy());
	QVERIFY(target);
	QCOMPARE(target->label(), QString("Source 1"));
	QCOMPARE(target->rowCount(), 2);
	QCOMPARE(target->child(1)->label(), QString("Sl 2"));

	TestItem *target2 = new TestItem(NULL);
	target2->clone(source);
	QCOMPARE(target2->label(), source->label());
	QCOMPARE(target2->rowCount(), source->rowCount());
	QCOMPARE(target2->child(1)->label(), source->child(1)->label());

	delete source;
	delete target;
	delete target2;
}

void TestTestModel::testSubItem()
{
	QString test = "asdasdasdasd";
	TestItem *parent = new TestItem(NULL);
	TestSubItem item(test, NULL, parent);
	QCOMPARE(item.label(), test);
	QCOMPARE(item.testItem(), parent);
	EXmlModel *testModel = NULL;
	QCOMPARE(item.model(), testModel);

	QString value = "gdgldfkgjdflkgj";
	QCOMPARE(item.isValid(), false);
	item.setValue(value);
	QCOMPARE(item.value(), value);
	QCOMPARE(item.isValid(), true);
	
	item.setLabel("");
	QCOMPARE(item.isValid(), false);
	
	delete parent;
}

void TestTestModel::testModelRoot()
{
	TestItem *item = new TestItem(NULL);
	TestModelRoot root;
	root.addChild(root.rowCount(), item);
	item->appendChild("Test", "Test Value");
	TestSubItem *subItem = qobject_cast<TestSubItem*>(item->child(0));
	QVERIFY(subItem);

	QSignalSpy dataChanged(&root, SIGNAL(changed(EXmlItem*)));
	item->setLabel("Test");
	QCOMPARE(dataChanged.count(), 1);

	subItem->setLabel("SubItem Tes");
	QCOMPARE(dataChanged.count(), 2);
}

void TestTestModel::testModel()
{
	TestModel *model = new TestModel(this);
	new ModelTest(model);

	QModelIndex root = model->indexFromItem(model->rootItem());

	// testing empty model
	QCOMPARE(model->rowCount(), 1);
	QCOMPARE(model->rowCount(root), 0);
	QCOMPARE(model->columnCount(), 2);

	// adding TestItems
	QList<TestItem*> testItems;
	testItems << model->addTestItem("test1");
	QVERIFY(testItems[0]);
	QCOMPARE(testItems[0]->model(), model);
	QCOMPARE(model->rowCount(root), 1);
	QCOMPARE(model->columnCount(), 2);

	testItems << model->addTestItem("test2");
	QCOMPARE(model->rowCount(root), 2);
	QCOMPARE(model->columnCount(), 2);

	testItems << model->addTestItem("test3");
	QCOMPARE(model->rowCount(root), 3);
	QCOMPARE(model->columnCount(), 2);

	// adding TestSubItems
	testItems[0]->appendChild("test11", "testValue11");
	testItems[0]->appendChild("test12", "testValue12");
	testItems[0]->appendChild("test13", "testValue13");
	testItems[0]->appendChild("test14", "testValue14");
	QCOMPARE(model->rowCount(root), 3);
	QCOMPARE(model->rowCount(model->index(0, 0, root)), 4);
	QCOMPARE(model->columnCount(root), 2);
	QCOMPARE(model->columnCount(model->index(0, 0, root)), 2);
	// Testing forwarding of EXmlModel
	TestSubItem *subItem = qobject_cast<TestSubItem*>(testItems[0]->child(0));
	QVERIFY(subItem);
	QCOMPARE(subItem->model(), model);

	// testing data
	QModelIndex parent = model->index(0, 0, root);
	QCOMPARE(model->data(parent).toString(), testItems[0]->label());
	QCOMPARE(model->data(model->index(1, 0, parent)).toString(), testItems[0]->child(1)->label());
	QCOMPARE(model->data(model->index(1, 1, parent)).toString(), testItems[0]->child(1)->value());

	// dataChanged signal
	QSignalSpy dataChanged(model, SIGNAL(dataChanged(QModelIndex, QModelIndex)));
	testItems[0]->setLabel("Test value changed");
	QCOMPARE(dataChanged.count(), 1);
}

void TestTestModel::testInsert()
{
	TestModel model;
	QModelIndex root = model.indexFromItem(model.rootItem());

	TestItem *test = model.addTestItem("test11");
	
	// insertRows
	QCOMPARE(model.rowCount(root), 1);
	QCOMPARE(model.rowCount(model.index(0, 0, root)), 0);
	QCOMPARE(model.insertRows(-1, 1, root), false);
	QCOMPARE(model.insertRows(0, -1, root), false);
	QCOMPARE(model.insertRows(0, 1, model.index(0, 0, root)), true);
	QCOMPARE(model.rowCount(root), 1);
	QCOMPARE(model.rowCount(model.index(0, 0, root)), 1);
	
	QCOMPARE(model.insertRows(0, 2, root), true);
	QCOMPARE(model.itemFromIndex(model.index(2, 0, root)), test);

	QSignalSpy layoutAboutChangedSpy(&model, SIGNAL(layoutAboutToBeChanged()));
	QSignalSpy layoutChangedSpy(&model, SIGNAL(layoutChanged()));
	test->appendChild("test", "test value");
	QCOMPARE(layoutAboutChangedSpy.count(), 1);
	QCOMPARE(layoutChangedSpy.count(), 1);
}

void TestTestModel::testRemove()
{
	TestModel *model = new TestModel(this);
	QModelIndex root = model->indexFromItem(model->rootItem());

	// testing empty model
	QCOMPARE(model->rowCount(), 1);
	QCOMPARE(model->rowCount(root), 0);
	QCOMPARE(model->columnCount(), 2);

	// adding TestItems
	QList<TestItem*> testItems;
	testItems << model->addTestItem("test1");
	QCOMPARE(model->rowCount(root), 1);
	QCOMPARE(model->columnCount(root), 2);

	testItems << model->addTestItem("test2");
	QCOMPARE(model->rowCount(root), 2);
	QCOMPARE(model->columnCount(root), 2);

	testItems << model->addTestItem("test3");
	QCOMPARE(model->rowCount(root), 3);
	QCOMPARE(model->columnCount(root), 2);

	// adding TestSubItems
	testItems[0]->appendChild("test11", "testValue11");
	testItems[0]->appendChild("test12", "testValue12");
	testItems[0]->appendChild("test13", "testValue13");
	QCOMPARE(model->rowCount(root), 3);
	QCOMPARE(model->columnCount(root), 2);
	
	// removing an TestItem
	QCOMPARE(model->removeTestItem(testItems[1]), true);
	QCOMPARE(model->rowCount(root), 2);
	testItems.removeAt(1);
	
	// removeRows
	QCOMPARE(model->removeRows(-1, 1), false);
	QCOMPARE(model->removeRows(0, -1, root), false);
	QCOMPARE(model->removeRows(0, 1, model->index(0, 0, root)), true);
	QCOMPARE(model->rowCount(model->index(0, 0, root)), 2);
	
	testItems.removeAt(1);
	model->addTestItem("test 3.1");
	QCOMPARE(model->removeRows(1, 2, root), true);
	QCOMPARE(model->rowCount(root), testItems.size());
}

QTEST_MAIN(TestTestModel)
