/*
    Copyright (C) 2013 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/


#include "esqllogintest.h"
#include "esqllogintest.moc"

#include <esqldatabase.h>

#include <QTemporaryFile>
#include <QApplication>
#include <QTcpSocket>
#include <QSqlQuery>
#include <QMetaProperty>
#include <QTest>

void ESqlLoginTest::initTestCase()
{
	serverReachable = true;
	if(!QSqlDatabase::drivers().contains("QSQLITE"))
		QSKIP("This test requires the SQLITE database driver", SkipAll);
	if(!QSqlDatabase::drivers().contains("QMYSQL"))
		QSKIP("This test requires the MYSQL database driver", SkipAll);
	 
	QString timeString = QDateTime::currentDateTime().toString("yyyyMMddhhmmsszzz");
	QApplication::setOrganizationName(timeString);
	username = "test";
	password = "test";
	host = "192.168.2.1";
	database = "test_"+ timeString;
	connName = "adminConnection";
	
	
	ESqlDatabase db = ESqlDatabase::addDatabase("QMYSQL", connName);
	configureDatabase(db);
	
	// test if server is reachable, because QSqlDatabase has a huge timeout
	// we use 2s as timeout here
	// Somehow, it doesn't work on windows
	QTcpSocket *socket = new QTcpSocket(this);
	socket->connectToHost(db.hostName(),db.port(),QIODevice::ReadOnly); 
	if(!socket->waitForConnected(2000)){
		serverReachable = false;
		return;
	}
	socket->close();

	QVERIFY(db.open());
	QSqlQuery query(db);
	// create database
	bool result = query.exec("CREATE DATABASE "+ database);
	if(!result){
		db.close();
		QTest::qWarn("Could not create new database");
		QVERIFY(result);
	}
}

void ESqlLoginTest::cleanupTestCase()
{
	if(!serverReachable)
		QSKIP("Clenup not needed, no database connection", SkipSingle);

	ESqlDatabase db = ESqlDatabase::database(connName);
	QVERIFY(db.open());
	
	QSqlQuery query(db);
	// create database
	bool result = query.exec("DROP DATABASE "+ database);
	db.close();
	QVERIFY(result);
}

void ESqlLoginTest::loadAndCheckSettings()
{
	checkGlobals();
	
	ESqlLogin login("", "testConn", false, this);
	QCOMPARE(login.loadAndCheckSettings(), false);
	QCOMPARE(login.loadAndCheckSettings("test", "test", "test", "GREATENONEXISTENTDRIVER", "test"), false);
	
	// SQLITE
	QCOMPARE(login.loadAndCheckSettings("", "" , "", "QSQLITE", ""), false);
	QCOMPARE(login.loadAndCheckSettings("test", "test" , "", "QSQLITE", ""), false);
	QCOMPARE(login.loadAndCheckSettings("test", "" , "test.sql", "QSQLITE", ""), true);
	QCOMPARE(login.loadAndCheckSettings("test", "test" , "test.sql", "QSQLITE", ""), true);
	QCOMPARE(login.loadAndCheckSettings("test", "test", "test.sql", "QSQLITE", "test"), true);
	QCOMPARE(login.loadAndCheckSettings("test", "", "test.sql", "QSQLITE", "test"), true);
	QCOMPARE(login.loadAndCheckSettings("test", "test", "", "QSQLITE", "test"), false);
	
	// MYSQL
	QCOMPARE(login.loadAndCheckSettings(username, "", "", "QMYSQL", ""), false);
	QCOMPARE(login.loadAndCheckSettings(username, host, "", "QMYSQL", ""), false);
	QCOMPARE(login.loadAndCheckSettings(username, "", database, "QMYSQL", ""), false);
	QCOMPARE(login.loadAndCheckSettings(username, host, database, "QMYSQL", ""), true);
	QCOMPARE(login.loadAndCheckSettings(username, "", "", "QMYSQL", password), false);
	QCOMPARE(login.loadAndCheckSettings(username, host, "", "QMYSQL", password), false);
	QCOMPARE(login.loadAndCheckSettings(username, "", database, "QMYSQL", password), false);
	QCOMPARE(login.loadAndCheckSettings(username, host, database, "QMYSQL", password), true);
}

void ESqlLoginTest::testGuiLess()
{
	if(!serverReachable)
		QSKIP("This test requires a database connection", SkipSingle);
		
	checkGlobals();
	
	QString dbConnName = "testConn";
	ESqlLogin login("", dbConnName, false, this);
	QCOMPARE(login.loadAndCheckSettings(), false);
	compareLogin(login.login(), ESqlLogin::Failed, 0);
	
	QTemporaryFile dbSqlite;
	QVERIFY(dbSqlite.open());
	dbSqlite.close();
	QCOMPARE(login.loadAndCheckSettings("", "", dbSqlite.fileName(), "QSQLITE", ""), true);
	compareLogin(login.login(), ESqlLogin::LoggedIn, 1);
	ESqlDatabase::removeDatabase(dbConnName);
	
	QCOMPARE(login.loadAndCheckSettings(username, "", "", "QMYSQL", ""), false);
	compareLogin(login.login(), ESqlLogin::Failed, 2);
	QCOMPARE(login.loadAndCheckSettings(username, host, "", "QMYSQL", ""), false);
	compareLogin(login.login(), ESqlLogin::Failed, 3);
	QCOMPARE(login.loadAndCheckSettings(username, "", database, "QMYSQL", ""), false);
	compareLogin(login.login(), ESqlLogin::Failed, 4);
	QCOMPARE(login.loadAndCheckSettings(username, host, database, "QMYSQL", ""), true);
	compareLogin(login.login(), ESqlLogin::Failed, 5);
	QCOMPARE(login.loadAndCheckSettings(username, "", "", "QMYSQL", password), false);
	compareLogin(login.login(), ESqlLogin::Failed, 6);
	QCOMPARE(login.loadAndCheckSettings(username, host, "", "QMYSQL", password), false);
	compareLogin(login.login(), ESqlLogin::LoggedIn, 7);
	ESqlDatabase::removeDatabase(dbConnName);
	QCOMPARE(login.loadAndCheckSettings(username, "", database, "QMYSQL", password), false);
	compareLogin(login.login(), ESqlLogin::Failed, 8);
	QCOMPARE(login.loadAndCheckSettings(username, host, database, "QMYSQL", password), true);
	compareLogin(login.login(), ESqlLogin::LoggedIn, 9);
	ESqlDatabase::removeDatabase(dbConnName);
}

void ESqlLoginTest::checkGlobals()
{
	QVERIFY(!username.isEmpty());
	QVERIFY(!host.isEmpty());
	QVERIFY(!password.isEmpty());
	QVERIFY(!database.isEmpty());
}

void ESqlLoginTest::configureDatabase(QSqlDatabase& db)
{
	db.setUserName(username);
	db.setPassword(password);
	db.setHostName(host);
	db.setPort(3306);
}

void ESqlLoginTest::compareLogin(ESqlLogin::Result result, ESqlLogin::Result expected, int testNum)
{
	if(result != expected){
		QString resultString = esqlLoginResultToString(result);
		QString expectedString = esqlLoginResultToString(expected);
		QTest::qWarn(QString("Test %1: %2 <> %3").arg(testNum).arg(resultString).arg(expectedString).toUtf8());
	}
	QCOMPARE(result, expected);
}

QString ESqlLoginTest::esqlLoginResultToString(ESqlLogin::Result result)
{
	QMetaObject metaObject = ESqlLogin::staticMetaObject;
	int index = metaObject.indexOfEnumerator("Result");
	QMetaEnum metaEnum = metaObject.enumerator(index);
	return QString(metaEnum.valueToKey(result));
}

QTEST_MAIN(ESqlLoginTest)
